;****************************************************************************
;****************************************************************************
;*
;* c64link.inc
;*
;****************************************************************************
;****************************************************************************





%ifndef C64LINK_INC
%define C64LINK_INC

%include "defines.inc"

%include "cpu65xx.inc"
%include "cia6526.inc"
%include "sid6581.inc"
%include "vic6569.inc"
%include "c64memory.inc"
%ifdef C64LINK_TEST_VIA
%include "via6522.inc"
%endif



;****************************************************************************
;* Exported Symbols *********************************************************
;****************************************************************************
%ifdef C64LINK_NASM
global c64link_initialize
global c64link_create_instance
global c64link_destroy_instance
global c64link_reset
global c64link_dump
global c64link_cia_counter_set
global c64link_cia_counter_refresh
global c64link_cia_counter_clear
global c64link_cia_counter_process
global c64link_keyboard
global c64link_cia1_irq
global c64link_cia1_pra
global c64link_cia1_prb
global c64link_cia1_sp
global c64link_cia1_pc
global c64link_cia2_nmi
global c64link_cia2_pra
global c64link_cia2_prb
global c64link_cia2_sp
global c64link_cia2_pc
global c64link_vic_irq
global c64link_status
%else
extern c64link_initialize
extern c64link_create_instance
extern c64link_destroy_instance
extern c64link_reset
extern c64link_dump
extern c64link_cia_counter_set
extern c64link_cia_counter_refresh
extern c64link_cia_counter_clear
extern c64link_cia_counter_process
extern c64link_keyboard
extern c64link_cia1_irq
extern c64link_cia1_pra
extern c64link_cia1_prb
extern c64link_cia1_sp
extern c64link_cia1_pc
extern c64link_cia2_nmi
extern c64link_cia2_pra
extern c64link_cia2_prb
extern c64link_cia2_sp
extern c64link_cia2_pc
extern c64link_vic_irq
extern c64link_status
%endif



;****************************************************************************
;* Module Data **************************************************************
;****************************************************************************
%assign C64LINK_COUNTER_TABLE_ENTRIES	4

				struc c64link
				alignb 4
d__c64link_modul_id:		resd	1
d__c64link_counter_base:	resd	1
d__c64link_counter:		resd	1
ad__c64link_counter_table:	resd	(2 * C64LINK_COUNTER_TABLE_ENTRIES)
ad__c64link_key_codes:		resd	5
b__c64link_vic_high_bits:	resb	1
b__c64link_serial_bus:		resb	1
				endstruc



				struc c64
				alignb 32
cpu65xx__c64:			resb	cpu65xx_size
				alignb 32
link__c64:			resb	c64link_size
				alignb 32
cia6526__c64_1:			resb	cia6526_size
				alignb 32
cia6526__c64_2:			resb	cia6526_size
%ifdef C64LINK_TEST_VIA
				alignb 32
via6522_c64_1:			resb	via6522_size
%endif
				alignb 32
sid6581__c64:			resb	sid6581_size
				alignb 32
vic6569__c64:			resb	vic6569_size
				alignb 32
memory__c64:			resb	c64memory_size
				endstruc



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%assign C64LINK_COUNTER_DEFAULT_VALUE	010000000h

%define C64LINK_MODUL_ID		esi + link__c64 + d__c64link_modul_id
%define C64LINK_COUNTER_BASE		esi + link__c64 + d__c64link_counter_base
%define C64LINK_COUNTER			esi + link__c64 + d__c64link_counter
%define C64LINK_COUNTER_TABLE		esi + link__c64 + ad__c64link_counter_table
%define C64LINK_KEY_CODES		esi + link__c64 + ad__c64link_key_codes
%define C64LINK_VIC_HIGH_BITS		esi + link__c64 + b__c64link_vic_high_bits
%define C64LINK_SERIAL_BUS		esi + link__c64 + b__c64link_serial_bus

%assign C64LINK_CIA1_COUNTER_OFFSET	000h
%assign C64LINK_CIA2_COUNTER_OFFSET	002h

%assign C64LINK_CIA1_IRQ_NUMBER		001h
%assign C64LINK_VIC_IRQ_NUMBER		002h



;****************************************************************************
;* Macro's ******************************************************************
;****************************************************************************
%macro C64LINK_CALL_CREATE_INSTANCE 0
			call	c64link_create_instance
%endmacro



%macro C64LINK_CALL_DESTROY_INSTANCE 0
			call	c64link_destroy_instance
%endmacro



%macro C64LINK_CALL_RESET 0
			call	c64link_reset
%endmacro



%macro C64LINK_CALL_DECREMENT_COUNTER 0
			dec	dword [C64LINK_COUNTER]
			jnz	%%no_call
			call	c64link_cia_counter_process
%%no_call:
%endmacro


%endif
;********************************************* karsten.scheibler@bigfoot.de *

