;****************************************************************************
;****************************************************************************
;*
;* c64link.nasm
;*
;****************************************************************************
;****************************************************************************





%define C64LINK_NASM
%include "c64link.inc"
%include "core.inc"
%include "link.inc"
%include "status.inc"
%include "print.inc"
%ifdef C64LINK_LOG
%include "log.inc"
%endif



%assign COUNTER_TABLE_ENTRIES	C64LINK_COUNTER_TABLE_ENTRIES
%assign COUNTER_DEFAULT_VALUE	C64LINK_COUNTER_DEFAULT_VALUE

%define MODUL_ID		C64LINK_MODUL_ID
%define COUNTER_BASE		C64LINK_COUNTER_BASE
%define COUNTER			C64LINK_COUNTER
%define COUNTER_TABLE		C64LINK_COUNTER_TABLE
%define KEY_CODES		C64LINK_KEY_CODES
%define VIC_HIGH_BITS		C64LINK_VIC_HIGH_BITS
%define SERIAL_BUS		C64LINK_SERIAL_BUS

%assign CIA1_IRQ_NUMBER		C64LINK_CIA1_IRQ_NUMBER
%assign VIC_IRQ_NUMBER		C64LINK_VIC_IRQ_NUMBER



;****************************************************************************
;* c64link_initialize *******************************************************
;****************************************************************************
section .text
c64link_initialize:
			ret



;****************************************************************************
;* c64link_create_instance **************************************************
;****************************************************************************
section .text
c64link_create_instance:
			push	dword eax
			mov	dword eax, [d__c64link_global_id]
			mov	dword [MODUL_ID], eax
%ifdef C64LINK_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__c64link_global_id], eax
			pop	dword eax
			ret

section .data
			align 4
d__c64link_global_id:	dd	C64LINK_GLOBAL_ID



;****************************************************************************
;* c64link_destroy_instance *************************************************
;****************************************************************************
section .text
c64link_destroy_instance:
%ifdef C64LINK_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			call	log_close
			pop	dword eax
%endif
			ret



%ifdef C64LINK_LOG
;****************************************************************************
;* c64link_log **************************************************************
;****************************************************************************
;* <=eax  data length
;* <=ebx  pointer to data
;****************************************************************************
section .text
c64link_log:
			mov	dword eax, [COUNTER]
			mov	dword ebx, [COUNTER_BASE]
			mov	dword [.ab__status], eax
			mov	dword [.ab__status + 4], ebx
			xor	dword ebx, ebx
.loop:			mov	dword eax, [COUNTER_TABLE + 4 * ebx]
			mov	dword [.ab__status + 16 + 4 * ebx], eax
			inc	byte  bl
			cmp	byte  bl, (2 * COUNTER_TABLE_ENTRIES)
			jb	.loop
			mov	dword eax, 64
			mov	dword ebx, .ab__status
			ret

section .data
			align 4
.ab__status:		times 64 db 0
%endif



;****************************************************************************
;* c64link_reset ************************************************************
;****************************************************************************
section .text
c64link_reset:
			xor	dword eax, eax
			xor	dword ebx, ebx
.clear_table:		mov	dword [COUNTER_TABLE + 8 * ebx], eax
			mov	dword [COUNTER_TABLE + 8 * ebx + 4], eax
			inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.clear_table
			mov	dword eax, COUNTER_DEFAULT_VALUE
			mov	dword [COUNTER_BASE], eax
			mov	dword [COUNTER], eax
			ret



;****************************************************************************
;* c64link_dump *************************************************************
;****************************************************************************
section .text
c64link_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			PRINT_TEXT  .cb__error, eax, [COUNTER]
.end:			ret

.cb__error:		db	"-<C64LINK:", STRING_INSERT_HEX32, ">-", 10
			db	"Counter: ", STRING_INSERT_HEX32, 10, 0



;****************************************************************************
;* counter_substract ********************************************************
;****************************************************************************
section .text
counter_substract:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, [COUNTER_BASE]
			xor	dword ebx, ebx
			sub	dword eax, [COUNTER]
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			test	dword ecx, ecx
			jz	.result_skip
			sub	dword ecx, eax
			jz	.result_wrong
			jnc	.result_ok
.result_wrong:		mov	dword eax, [MODUL_ID]
			jmp	core_exception
.result_ok:		mov	dword [COUNTER_TABLE + 8 * ebx], ecx
.result_skip:		inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* counter_select ***********************************************************
;****************************************************************************
section .text
counter_select:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, COUNTER_DEFAULT_VALUE
			xor	dword ebx, ebx
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			test	dword ecx, ecx
			jz	.skip
			cmp	dword ecx, eax
			jnb	.skip
			mov	dword eax, ecx
.skip:			inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			mov	dword [COUNTER_BASE], eax
			mov	dword [COUNTER], eax
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* c64link_cia_counter_set **************************************************
;****************************************************************************
;* eax=>  counter offset
;* ebx=>  handler address
;* ecx=>  counter value
;****************************************************************************
section .text
c64link_cia_counter_set:
			call	counter_substract
			mov	dword [COUNTER_TABLE + 8 * eax], ecx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			jmp	counter_select



;****************************************************************************
;* c64link_cia_counter_refresh **********************************************
;****************************************************************************
;* eax=>  counter offset
;* ebx=>  handler address
;* ecx=>  counter value
;****************************************************************************
section .text
c64link_cia_counter_refresh:
			mov	dword [COUNTER_TABLE + 8 * eax], ecx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			ret



;****************************************************************************
;* c64link_cia_counter_clear ************************************************
;****************************************************************************
;* eax=>  counter offset
;****************************************************************************
section .text
c64link_cia_counter_clear:
			push	dword ebx
			xor	dword ebx, ebx
			call	counter_substract
			mov	dword [COUNTER_TABLE + 8 * eax], ebx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			pop	dword ebx
			jmp	counter_select



;****************************************************************************
;* c64link_cia_counter_process **********************************************
;****************************************************************************
section .text
c64link_cia_counter_process:
%ifdef C64LINK_LOG
%ifdef C64LINK_LOG_EVERY_UNDERFLOW
			call	c64link_log
			mov	dword ecx, eax
			mov	dword eax, [MODUL_ID]
			call	log_write
%endif
%endif
			mov	dword eax, [COUNTER_BASE]
			xor	dword ebx, ebx
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			mov	dword edx, [COUNTER_TABLE + 8 * ebx + 4]
			sub	dword ecx, eax
			jc	.counter_disabled
			mov	dword [COUNTER_TABLE + 8 * ebx], ecx
			jnz	.counter_disabled
			push	dword eax
			push	dword ebx
			mov	dword eax, [.ap__cia_offsets + 4 * ebx]
			push	dword ecx
			lea	dword edi, [esi + eax]
			call	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
.counter_disabled:	inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			jmp	counter_select

			align 4
.ap__cia_offsets:	dd	cia6526__c64_1
			dd	cia6526__c64_1
			dd	cia6526__c64_2
			dd	cia6526__c64_2



;****************************************************************************
;* c64link_keyboard *********************************************************
;****************************************************************************
;* eax=>  4 x C64 Keys
;****************************************************************************
section .text
c64link_keyboard:		
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword edi
			xor	dword ebx, ebx
			xor	dword ecx, ecx
.next_key_code:		mov	byte  cl, al
			mov	dword edx, [.ad__c64_key_codes + 4 * ecx]
			mov	dword [KEY_CODES + 4 * ebx], edx
			inc	dword ebx
			shr	dword eax, 8
			cmp	byte  bl, 4
			jb	.next_key_code
			xor	dword ecx, ecx
			lea	dword edi, [esi + cia6526__c64_1]
			mov	dword [KEY_CODES + 4 * ebx], ecx
			call	c64link_cia1_prb
			pop	dword edi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

			align 4
.ad__c64_key_codes:	
			;-------
			;a ... o
                        ;-------

			dd	00000h, 00204h, 00810h, 00410h
			dd	00404h, 00240h, 00420h, 00804h
			dd	00820h, 01002h, 01004h, 01020h
			dd	02004h, 01010h, 01080h, 01040h

			;-------------------------------
			;p ... z SHIFT_LEFT  SHIFT_RIGHT
                        ;-------------------------------

			dd	02002h, 08040h, 00402h, 00220h
			dd	00440h, 00840h, 00880h, 00202h
			dd	00480h, 00802h, 00210h, 00280h
			dd	04001h, 04010h, 04040h, 08002h

			;----------------------------------
			;SPACE  CLR/HOME  INST/DEL  CONTROL
			;RUN_STOP  RETURN  C=  CRSR_DOWN  
			;CRSR_RIGHT  F1  * +  
			;F3  -  F5  F7
			;----------------------------------

			dd	08010h, 04008h, 00101h, 08004h
			dd	08080h, 00102h, 08020h, 00180h
			dd	00104h, 00110h, 04002h, 02001h
			dd	00120h, 02008h, 00140h, 00108h

			;---------------------
			;0 ... 9 : ; , = . / @
			;---------------------

			dd	01008h, 08001h, 08008h, 00201h
			dd	00208h, 00401h, 00408h, 00801h
			dd	00808h, 01001h, 02020h, 04004h
			dd	02080h, 04020h, 02010h, 04080h
			dd	02040h



;****************************************************************************
;* c64link_cia1_irq *********************************************************
;****************************************************************************
section .text
c64link_cia1_irq:
			push	dword eax
			test	byte  al, al
			jnz	.set
.clear:			mov	byte  al, CIA1_IRQ_NUMBER
			call	cpu65xx_signal_clear_irq
			pop	dword eax
			ret
.set:			mov	byte  al, CIA1_IRQ_NUMBER
			call	cpu65xx_signal_set_irq
			pop	dword eax
			ret



;****************************************************************************
;* c64link_cia1_pra, c64link_cia1_prb ***************************************
;****************************************************************************
section .text
c64link_cia1_pra:
c64link_cia1_prb:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			xor	dword ebx, ebx
			mov	byte  ah, [CIA6526_LATCH_PRA]
			mov	byte  dh, [CIA6526_LATCH_PRB]
			xor	byte  ah, NOT8
			xor	byte  dh, NOT8
			and	byte  ah, [CIA6526_REGISTERS + CIA6526_DDRA]
			and	byte  dh, [CIA6526_REGISTERS + CIA6526_DDRB]
			xor	byte  al, al
			xor	byte  dl, dl
.next_key_code:		mov	dword ecx, [KEY_CODES + 4 * ebx]
			test	dword ecx, ecx
			jz	.end
			test	byte  ch, ah
			jz	._1
			or	byte  dl, cl
._1:			test	byte  cl, dh
			jz	._2
			or	byte  al, ch
._2:			inc	dword ebx
			cmp	byte  bl, 4
			jb	.next_key_code
.end:			xor	byte  al, NOT8
			xor	byte  dl, NOT8
			call	cia6526_signal_pra
			mov	byte  al, dl
			call	cia6526_signal_prb
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* c64link_cia1_sp **********************************************************
;****************************************************************************
section .text
c64link_cia1_sp:
			ret



;****************************************************************************
;* c64link_cia1_pc **********************************************************
;****************************************************************************
section .text
c64link_cia1_pc:
			ret



;****************************************************************************
;* c64link_cia2_nmi *********************************************************
;****************************************************************************
section .text
c64link_cia2_nmi:
			push	dword eax
			test	byte  al, al
			jz	.end
.set:			call	cpu65xx_signal_set_nmi
.end:			pop	dword eax
			ret



;****************************************************************************
;* c64link_cia2_pra *********************************************************
;****************************************************************************
section .text
c64link_cia2_pra:
			push	dword eax
			push	dword ebx
			mov	byte  al, [CIA6526_LATCH_PRA]
			mov	byte  bl, [CIA6526_REGISTERS + CIA6526_DDRA]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			xor	byte  al, NOT8
			and	byte  al, 3
			cmp	byte  [VIC_HIGH_BITS], al
			je	.skip_high_bits
			mov	byte  [VIC_HIGH_BITS], al
			call	vic6569_signal_high_bits
.skip_high_bits:	mov	byte  al, [CIA6526_LATCH_PRA]
			mov	byte  bl, [CIA6526_REGISTERS + CIA6526_DDRA]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			and	byte  al, 038h
			shr	byte  al, 3
			cmp	byte  [SERIAL_BUS], al
			je	.skip_serial_bus
			mov	byte  [SERIAL_BUS], al
			call	link_serial_bus
.skip_serial_bus:	pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* c64link_cia2_prb *********************************************************
;****************************************************************************
section .text
c64link_cia2_prb:
			ret



;****************************************************************************
;* c64link_cia2_sp **********************************************************
;****************************************************************************
section .text
c64link_cia2_sp:
			ret



;****************************************************************************
;* c64link_cia2_pc **********************************************************
;****************************************************************************
section .text
c64link_cia2_pc:
			ret



;****************************************************************************
;* c64link_vic_irq **********************************************************
;****************************************************************************
section .text
c64link_vic_irq:
			push	dword eax
			test	byte  al, al
			jnz	.set
.clear:			mov	byte  al, VIC_IRQ_NUMBER
			call	cpu65xx_signal_clear_irq
			pop	dword eax
			ret
.set:			mov	byte  al, VIC_IRQ_NUMBER
			call	cpu65xx_signal_set_irq
			pop	dword eax
			ret



;****************************************************************************
;* c64link_status ***********************************************************
;****************************************************************************
section .text
c64link_status:
			push	dword eax
			push	dword ebx
			mov	dword eax, [MODUL_ID]
			mov	dword ebx, [VIC6569_DECODED_LINES]
			call	status_c64_decoded_lines
			pop	dword ebx
			pop	dword eax
			ret
;********************************************* karsten.scheibler@bigfoot.de *

