;****************************************************************************
;****************************************************************************
;*
;* c64memory.nasm
;*
;****************************************************************************
;****************************************************************************





%define C64MEMORY_NASM
%include "c64memory.inc"
%include "c64link.inc"
%include "core.inc"
%include "print.inc"
%ifdef C64MEMORY_TEST_VIA
%include "via6522.inc"
%endif
%ifdef C64MEMORY_LOG
%include "log.inc"
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & define's
;*
;****************************************************************************
;****************************************************************************





%define MODUL_ID		C64MEMORY_MODUL_ID
%define STATE			C64MEMORY_STATE
%define CPU6510_DDR		C64MEMORY_CPU6510_DDR
%define CPU6510_IO		C64MEMORY_CPU6510_IO
%define CPU6510_DATA		C64MEMORY_CPU6510_DATA
%define CPU6510_MASK		C64MEMORY_CPU6510_MASK
%define READ_OPCODE_LOW_LIMIT	C64MEMORY_READ_OPCODE_LOW_LIMIT
%define READ_OPCODE_HIGH_LIMIT	C64MEMORY_READ_OPCODE_HIGH_LIMIT
%define RAM			C64MEMORY_RAM
%define COLOR_RAM		C64MEMORY_COLOR_RAM
%define BASIC_ROM		C64MEMORY_BASIC_ROM
%define CHARACTER_ROM		C64MEMORY_CHARACTER_ROM
%define KERNEL_ROM		C64MEMORY_KERNEL_ROM



%assign STACK_LIMIT_EXCEEDED	C64MEMORY_STACK_LIMIT_EXCEEDED



;****************************************************************************
;****************************************************************************
;*
;* PART 2: ROMs
;*
;****************************************************************************
;****************************************************************************





section .text
ab__c64_basic_rom:	incbin "../roms/c64_basic.rom"
ab__c64_character_rom:	incbin "../roms/c64_character.rom"
ab__c64_kernel_rom:	incbin "../roms/c64_kernel.rom"
%ifdef C64MEMORY_INCLUDE_PROGRAM
ab__c64_program:	incbin C64MEMORY_PROGRAM_PATH
%endif
%ifdef C64MEMORY_INCLUDE_MONITOR
ab__c64_monitor:	incbin C64MEMORY_MONITOR_PATH
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Memory Access Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c64memory_read ***********************************************************
;****************************************************************************
section .text
c64memory_read:
			and	dword ebx, 0ffffh
			mov	dword eax, ebx
			shr	dword eax, 12
			add	byte  al, [.b__functions_offset]
			jmp	dword [.ap__read_functions + 4 * eax]
.read_cpu6510:		cmp	word  bx, 00002h
			jb	.read_cpu6510_1
.read_ram:		mov	byte  al, [RAM + ebx]
			ret
.read_cpu6510_1:	mov	byte  al, [CPU6510_DDR + ebx]
			ret
.read_basic_rom:	mov	byte  al, [BASIC_ROM - 0a000h + ebx]
			ret
.read_io:		cmp	word  bx, 0d800h
			jae	.read_io_1
			cmp	word  bx, 0d400h
			jae	.read_sid
.read_vic:		call	vic6569_read
			ret
.read_sid:		call	sid6581_read
			ret
.read_io_1:		cmp	word  bx, 0dc00h
			jae	.read_io_2
.read_color_ram:	mov	byte  al, [COLOR_RAM - 0d800h + ebx]
			or	byte  al, 0f0h
			ret
.read_io_2:		cmp	word  bx, 0dd00h
			jb	.read_cia1
			cmp	word  bx, 0de00h
			jb	.read_cia2
%ifdef C64MEMORY_TEST_VIA
			cmp	word  bx, 0df00h
			jb	.read_via1
%endif
.read_rubbish:		mov	byte  al, 05ah
			ret
.read_cia1:		CIA6526_CALL_READ  cia6526__c64_1
			ret
.read_cia2:		CIA6526_CALL_READ  cia6526__c64_2
			ret
%ifdef C64MEMORY_TEST_VIA
.read_via1:		lea	dword edi, [esi + via6522__c64_1]
			call	via6522_read
			ret
%endif
.read_character_rom:	mov	byte  al, [CHARACTER_ROM - 0d000h + ebx]
			ret
.read_kernel_rom:	mov	byte  al, [KERNEL_ROM - 0e000h + ebx]
			ret

.ap__read_functions:	
			;BASIC ROM - I/O - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_basic_rom		;$a000 - $afff
			dd	.read_basic_rom		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - I/O - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - I/O - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_io		;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

			;RAM - RAM - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_ram		;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

			;BASIC ROM - CHARACTER ROM - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_basic_rom		;$a000 - $afff
			dd	.read_basic_rom		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - CHARACTER ROM - KERNEL ROM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_kernel_rom	;$e000 - $efff
			dd	.read_kernel_rom	;$f000 - $ffff

			;RAM - CHARACTER ROM - RAM

			dd	.read_cpu6510		;$0000 - $0fff
			dd	.read_ram		;$1000 - $1fff
			dd	.read_ram		;$2000 - $2fff
			dd	.read_ram		;$3000 - $3fff
			dd	.read_ram		;$4000 - $4fff
			dd	.read_ram		;$5000 - $5fff
			dd	.read_ram		;$6000 - $6fff
			dd	.read_ram		;$7000 - $7fff
			dd	.read_ram		;$8000 - $8fff
			dd	.read_ram		;$9000 - $9fff
			dd	.read_ram		;$a000 - $afff
			dd	.read_ram		;$b000 - $bfff
			dd	.read_ram		;$c000 - $cfff
			dd	.read_character_rom	;$d000 - $dfff
			dd	.read_ram		;$e000 - $efff
			dd	.read_ram		;$f000 - $ffff

section .data
.b__functions_offset:	db	0



;****************************************************************************
;* c64memory_write **********************************************************
;****************************************************************************
section .text
c64memory_write:
			and	dword ebx, 0ffffh
			cmp	word  bx, 00002h
			jb	.write_cpu6510
			jmp	dword [.p__write_function]
.write_cpu6510:		cmp	byte  bl, 001h
			je	.write_cpu6510_io
.write_cpu6510_ddr:	mov	byte  [CPU6510_DDR], al
			not	byte  al
			and	byte  al, 0dfh
			test	byte  al, 008h
			jz	.write_cpu6510_ddr_1
			mov	byte  ah, [CPU6510_IO]
			and	byte  al, 0f7h
			and	byte  ah, 008h
			or	byte  al, ah
.write_cpu6510_ddr_1:	and	byte  al, 03fh
			mov	byte  [CPU6510_MASK], al
			mov	byte  al, [CPU6510_DATA]
			jmp	.calculate_offset
.write_cpu6510_io:	mov	byte  [CPU6510_DATA], al
.calculate_offset:	and	byte  al, [CPU6510_DDR]
			or	byte  al, [CPU6510_MASK]
			mov	byte  [CPU6510_IO], al
			and	byte  al, 007h
			jnz	.calculate_offset_1
			mov	byte  al, 004h
.calculate_offset_1:	xor	byte  al, 007h
			shl	byte  al, 4
			mov	byte  [c64memory_read.b__functions_offset], al
			cmp	byte  al, 030h
			jb	.calculate_offset_2
			mov	dword [.p__write_function], .write_ram
			ret
.calculate_offset_2:	mov	dword [.p__write_function], .write_io
			ret
.write_io:		cmp	word  bx, 0d000h
			jb	.write_ram
			cmp	word  bx, 0e000h
			jb	.write_io_1
.write_ram:		mov	byte  [RAM + ebx], al
			ret
.write_io_1:		cmp	word  bx, 0d800h
			jae	.write_io_2
			cmp	word  bx, 0d400h
			jae	.write_sid
.write_vic:		call	vic6569_write
			ret
.write_sid:		call	sid6581_write
			ret
.write_io_2:		cmp	word  bx, 0dc00h
			jae	.write_io_3
.write_color_ram:	mov	byte  [COLOR_RAM - 0d800h + ebx], al
			ret
.write_io_3:		cmp	word  bx, 0dd00h
			jb	.write_cia1
%ifdef C64MEMORY_TEST_VIA
			cmp	word  bx, 0de00h
			jb	.write_cia2
			cmp	word  bx, 0df00h
			jae	.write_io_4
.write_via1:		lea	dword edi, [esi + via6522__c64_1]
			call	via6522_write
			ret
%else
			cmp	word  bx, 0de00h
			jae	.write_io_4
%endif
.write_cia2:		CIA6526_CALL_WRITE  cia6526__c64_2
.write_io_4:		ret
.write_cia1:		CIA6526_CALL_WRITE  cia6526__c64_1
			ret

section .data
.p__write_function:	dd	.write_io



%ifdef NOT_IN_USE
;****************************************************************************
;* read_opcode **************************************************************
;****************************************************************************
section .text
read_opcode_ram:
			cmp	dword ebx, [READ_OPCODE_LOW_LIMIT]
			jae	.check_high_limit
			jmp	read_opcode_switch_routine
.check_high_limit:	cmp	dword ebx, [READ_OPCODE_HIGH_LIMIT]
			jb	.read
			jmp	read_opcode_switch_routine
.read:			mov	byte  al, [RAM + ebx]
			ret

read_opcode_io:
			cmp	dword ebx, 0d000h
			jae	.read
			jmp	read_opcode_switch_routine
.read:			cmp	dword ebx, 0d400h
			jb	near  vic6569_read
			cmp	dword ebx, 0d800h
			jb	near  sid6581_read
			cmp	dword ebx, 0dc00h
			jae	.read_cias
			mov	byte  al, [COLOR_RAM + ebx - 0d800h]
			or	byte  al, 0f0h
			ret
.read_cias:		lea	dword edi, [esi + cia6526__c64_1]
			cmp	dword ebx, 0dd00h
			jb	near  cia6526_read
			lea	dword edi, [esi + cia6526__c64_2]
			cmp	dword ebx, 0de00h
			jb	near  cia6526_read
%ifdef C64MEMORY_TEST_VIA
			lea	dword edi, [esi + via6522__c64_1]
			cmp	dword ebx, 0df00h
			jb	near  via6522_read
%endif
			cmp	dword ebx, 0e000h
			jae	read_opcode_switch_routine
			mov	byte  al, 05ah
			ret


read_opcode_basic_rom:
			cmp	dword ebx, 0a000h
			jae	.check_high_limit
			jmp	read_opcode_switch_routine
.check_high_limit:	cmp	dword ebx, 0c000h
			jb	.read
			jmp	read_opcode_switch_routine
.read:			mov	byte  al, [BASIC_ROM + ebx - 0a000h]
			ret

read_opcode_character_rom:
			cmp	dword ebx, 0d000h
			jae	.check_high_limit
			jmp	read_opcode_switch_routine
.check_high_limit:	cmp	dword ebx, 0e000h
			jb	.read
			jmp	read_opcode_switch_routine
.read:			mov	byte  al, [CHARACTER_ROM + ebx - 0d000h]
			ret

read_opcode_kernel_rom:
			cmp	dword ebx, 0e000h
			jae	.check_high_limit
			jmp	read_opcode_switch_routine
.check_high_limit:	cmp	dword ebx, 010000h
			jb	.read
			jmp	read_opcode_switch_routine
.read:			mov	byte  al, [KERNEL_ROM + ebx - 0e000h]
			ret

read_opcode_switch_routine:
			;error if ebx > 0ffffh
			ret
%endif


;****************************************************************************
;* read_stack ***************************************************************
;****************************************************************************
section .text
read_stack:
%ifdef C64MEMORY_CHECK
%ifdef C64MEMORY_CHECK_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.read
			jmp	.limit_exceeded
.read:
%endif
%endif
			mov	byte  al, [RAM + ebx]
			ret



;****************************************************************************
;* read_data ****************************************************************
;****************************************************************************
section .text
read_data:
			ret



;****************************************************************************
;* write_stack **************************************************************
;****************************************************************************
section .text
write_stack:
%ifdef C64MEMORY_CHECK
%ifdef C64MEMORY_CHECK_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.write
			jmp	.limit_exceeded
.write:
%endif
%endif
			mov	byte  [RAM + ebx], al
			ret



;****************************************************************************
;* write_data ***************************************************************
;****************************************************************************
section .text
write_data:
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 4: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* c64memory_initialize *****************************************************
;****************************************************************************
section .text
c64memory_initialize:
			ret



;****************************************************************************
;* c64memory_create_instance ************************************************
;****************************************************************************
section .text
c64memory_create_instance:
			push	dword eax
			mov	dword eax, [d__c64memory_global_id]
			mov	dword [C64MEMORY_MODUL_ID], eax
%ifdef C64MEMORY_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__c64memory_global_id], eax
			pop	dword eax
			ret

section .data
				align 4
d__c64memory_global_id:	dd	C64MEMORY_GLOBAL_ID



;****************************************************************************
;* c64memory_destroy_instance ***********************************************
;****************************************************************************
section .text
c64memory_destroy_instance:
%ifdef C64MEMORY_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			push	dword ebx
			push	dword ecx
%ifdef C64MEMORY_LOG_SAVE_RAM
			lea	dword ebx, [RAM]
			mov	dword ecx, 010000h
			call	log_write
%endif
			call	log_close
			pop	dword eax
%endif
			ret



;****************************************************************************
;* c64memory_reset **********************************************************
;****************************************************************************
section .text
c64memory_reset:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edi
			mov	dword ebx, 0200h
			xor	dword ecx, ecx
			mov	dword [STATE], ecx
			lea	dword edi, [RAM]
			cld
.fill_memory:		xor	dword eax, eax
			mov	byte  cl, 010h
			rep stosd
			dec	dword eax
			mov	byte  cl, 010h
			rep stosd
			dec	dword ebx
			jnz	.fill_memory

%ifdef C64MEMORY_INCLUDE_PROGRAM
			lea	dword ebx, [RAM + 01000h]
			push	dword esi
			mov	dword esi, ab__c64_program
			mov	dword edi, ebx
			mov	dword ecx, 00800h
			rep movsd
			pop	dword esi
%endif

%ifdef C64MEMORY_INCLUDE_MONITOR
			lea	dword ebx, [RAM + 0c000h]
			push	dword esi
			mov	dword esi, ab__c64_monitor
			mov	dword edi, ebx
			mov	dword ecx, 00400h
			rep movsd
			pop	dword esi
%endif

			xor	word  bx, bx		;set the correct 
			mov	byte  al, 02fh		;data direction
			call	c64memory_write	;on the 6510
			inc	word  bx
			mov	byte  al, 027h		;Basic, I/O, Kernel
			call	c64memory_write
			pop	dword edi
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

c64memory_read_opcode:		equ	c64memory_read
c64memory_read_stack:		equ	read_stack
c64memory_read_data:		equ	c64memory_read
c64memory_write_stack:		equ	write_stack
c64memory_write_data:		equ	c64memory_write



;****************************************************************************
;* c64memory_dump ***********************************************************
;****************************************************************************
section .text
c64memory_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			PRINT_TEXT  .cb__error, eax, [STATE]
.end:			ret

.cb__error:		db	"-<C64MEMORY:", STRING_INSERT_HEX32, ">-", 10
			db	"State: ", STRING_INSERT_HEX32, 10, 0
;********************************************* karsten.scheibler@bigfoot.de *

