;****************************************************************************
;****************************************************************************
;*
;* cia6526.inc
;*
;****************************************************************************
;****************************************************************************




%ifndef CIA6526_INC
%define CIA6526_INC

%include "defines.inc"

;****************************************************************************
;* Exported Symbols *********************************************************
;****************************************************************************
%ifdef CIA6526_NASM
global cia6526_initialize
global cia6526_create_instance
global cia6526_destroy_instance
global cia6526_reset
global cia6526_dump
global cia6526_read
global cia6526_write
global cia6526_signal_pra
global cia6526_signal_prb
%else
extern cia6526_initialize
extern cia6526_create_instance
extern cia6526_destroy_instance
extern cia6526_reset
extern cia6526_dump
extern cia6526_read
extern cia6526_write
extern cia6526_signal_pra
extern cia6526_signal_prb
%endif



;****************************************************************************
;* Module Data **************************************************************
;****************************************************************************
				struc cia6526
				alignb 4
d__cia6526_modul_id:		resd	1
d__cia6526_state:		resd	1
p__cia6526_link_irq:		resd	1
p__cia6526_link_pra:		resd	1
p__cia6526_link_prb:		resd	1
d__cia6526_counter_offset:	resd	1
q__cia6526_timestamp_ta:	resd	2
d__cia6526_counter_ta:		resd	1
q__cia6526_timestamp_tb:	resd	2
d__cia6526_counter_tb:		resd	1
b__cia6526_latch_pra:		resb	1
b__cia6526_signal_pra:		resb	1
b__cia6526_latch_prb:		resb	1
b__cia6526_signal_prb:		resb	1
b__cia6526_latch_ta_hi:		resb	1
b__cia6526_latch_tb_hi:		resb	1
b__cia6526_irq_mask:		resb	1
ab__cia6526_registers:		resb	16
				endstruc



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%assign CIA6526_PRA		000h
%assign CIA6526_PRB		001h
%assign CIA6526_DDRA		002h
%assign CIA6526_DDRB		003h
%assign CIA6526_TA_LO		004h
%assign CIA6526_TA_HI		005h
%assign CIA6526_TB_LO		006h
%assign CIA6526_TB_HI		007h
%assign CIA6526_TOD_10THS	008h
%assign CIA6526_TOD_SEC		009h
%assign CIA6526_TOD_MIN		00ah
%assign CIA6526_TOD_HR		00bh
%assign CIA6526_SDR		00ch
%assign CIA6526_ICR		00dh
%assign CIA6526_CRA		00eh
%assign CIA6526_CRB		00fh

%define CIA6526_MODUL_ID		byte edi + d__cia6526_modul_id
%define CIA6526_STATE			byte edi + d__cia6526_state
%define CIA6526_LINK_IRQ		byte edi + p__cia6526_link_irq
%define CIA6526_LINK_PRA		byte edi + p__cia6526_link_pra
%define CIA6526_LINK_PRB		byte edi + p__cia6526_link_prb
%define CIA6526_COUNTER_OFFSET		byte edi + d__cia6526_counter_offset
%define CIA6526_TIMESTAMP_TA		byte edi + q__cia6526_timestamp_ta
%define CIA6526_COUNTER_TA		byte edi + d__cia6526_counter_ta
%define CIA6526_TIMESTAMP_TB		byte edi + q__cia6526_timestamp_tb
%define CIA6526_COUNTER_TB		byte edi + d__cia6526_counter_tb
%define CIA6526_LATCH_PRA		byte edi + b__cia6526_latch_pra
%define CIA6526_SIGNAL_PRA		byte edi + b__cia6526_signal_pra
%define CIA6526_LATCH_PRB		byte edi + b__cia6526_latch_prb
%define CIA6526_SIGNAL_PRB		byte edi + b__cia6526_signal_prb
%define CIA6526_LATCH_TA_HI		byte edi + b__cia6526_latch_ta_hi
%define CIA6526_LATCH_TB_HI		byte edi + b__cia6526_latch_tb_hi
%define CIA6526_IRQ_MASK		byte edi + b__cia6526_irq_mask
%define CIA6526_REGISTERS		byte edi + ab__cia6526_registers

%assign CIA6526_TA_FORCE_LOAD		000000001h
%assign CIA6526_TA_STOPPED		000000002h
%assign CIA6526_TA_LOCKED		000000004h
%assign CIA6526_TB_FORCE_LOAD		000000008h
%assign CIA6526_TB_STOPPED		000000010h
%assign CIA6526_TB_LOCKED		000000020h
%assign CIA6526_TB_CASCADED		000000040h



;****************************************************************************
;* Macro's ******************************************************************
;****************************************************************************
%macro CIA6526_CALL_CREATE_INSTANCE 5
			lea	dword edi, [esi + %1]
			mov	dword eax, %2
			mov	dword ebx, %3
			mov	dword ecx, %4
			mov	dword edx, %5
			mov	dword [CIA6526_COUNTER_OFFSET], eax
			mov	dword [CIA6526_LINK_IRQ], ebx
			mov	dword [CIA6526_LINK_PRA], ecx
			mov	dword [CIA6526_LINK_PRB], edx
			call	cia6526_create_instance
%endmacro



%macro CIA6526_CALL_DESTROY_INSTANCE 1
			lea	dword edi, [esi + %1]
			call	cia6526_destroy_instance
%endmacro



%macro CIA6526_CALL_RESET 1
			lea	dword edi, [esi + %1]
			call	cia6526_reset
%endmacro



%macro CIA6526_CALL_READ 1
			lea	dword edi, [esi + %1]
			call	cia6526_read
%endmacro



%macro CIA6526_CALL_WRITE 1
			lea	dword edi, [esi + %1]
			call	cia6526_write
%endmacro



%endif
;********************************************* karsten.scheibler@bigfoot.de *

