;****************************************************************************
;****************************************************************************
;*
;* cia6526.nasm
;*
;****************************************************************************
;****************************************************************************





%define CIA6526_NASM
%include "cia6526.inc"
%include "core.inc"
%include "c64link.inc"
%include "print.inc"
%ifdef CIA6526_LOG
%include "log.inc"
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & defines's
;*
;****************************************************************************
;****************************************************************************





%assign PRA		CIA6526_PRA
%assign PRB		CIA6526_PRB
%assign DDRA		CIA6526_DDRA
%assign DDRB		CIA6526_DDRB
%assign TA_LO		CIA6526_TA_LO
%assign TA_HI		CIA6526_TA_HI
%assign TB_LO		CIA6526_TB_LO
%assign TB_HI		CIA6526_TB_HI
%assign TOD_10THS	CIA6526_TOD_10THS
%assign TOD_SEC		CIA6526_TOD_SEC
%assign TOD_MIN		CIA6526_TOD_MIN
%assign TOD_HR		CIA6526_TOD_HR
%assign SDR		CIA6526_SDR
%assign ICR		CIA6526_ICR
%assign CRA		CIA6526_CRA
%assign CRB		CIA6526_CRB



%define MODUL_ID	CIA6526_MODUL_ID
%define STATE		CIA6526_STATE
%define LINK_IRQ	CIA6526_LINK_IRQ
%define LINK_PRA	CIA6526_LINK_PRA
%define LINK_PRB	CIA6526_LINK_PRB
%define COUNTER_OFFSET	CIA6526_COUNTER_OFFSET
%define TIMESTAMP_TA	CIA6526_TIMESTAMP_TA
%define COUNTER_TA	CIA6526_COUNTER_TA
%define TIMESTAMP_TB	CIA6526_TIMESTAMP_TB
%define COUNTER_TB	CIA6526_COUNTER_TB
%define LATCH_PRA	CIA6526_LATCH_PRA
%define SIGNAL_PRA	CIA6526_SIGNAL_PRA
%define LATCH_PRB	CIA6526_LATCH_PRB
%define SIGNAL_PRB	CIA6526_SIGNAL_PRB
%define LATCH_TA_HI	CIA6526_LATCH_TA_HI
%define LATCH_TB_HI	CIA6526_LATCH_TB_HI
%define IRQ_MASK	CIA6526_IRQ_MASK
%define REGISTERS	CIA6526_REGISTERS



%assign TA_FORCE_LOAD	CIA6526_TA_FORCE_LOAD
%assign TA_STOPPED	CIA6526_TA_STOPPED
%assign TA_LOCKED	CIA6526_TA_LOCKED
%assign TB_FORCE_LOAD	CIA6526_TB_FORCE_LOAD
%assign TB_STOPPED	CIA6526_TB_STOPPED
%assign TB_LOCKED	CIA6526_TB_LOCKED
%assign TB_CASCADED	CIA6526_TB_CASCADED






;****************************************************************************
;****************************************************************************
;*
;* PART 2: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cia6526_initialize *******************************************************
;****************************************************************************
section .text
cia6526_initialize:
			ret



;****************************************************************************
;* cia6526_create_instance **************************************************
;****************************************************************************
section .text
cia6526_create_instance:
			push	dword eax
			mov	dword eax, [d__cia6526_global_id]
			mov	dword [MODUL_ID], eax
%ifdef CIA6526_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__cia6526_global_id], eax
			pop	dword eax
			ret

section .data
			align 4
d__cia6526_global_id:	dd	CIA6526_GLOBAL_ID



;****************************************************************************
;* cia6526_destroy_instance *************************************************
;****************************************************************************
section .text
cia6526_destroy_instance:
%ifdef CIA6526_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			call	log_close
			pop	dword eax
%endif
			ret



;****************************************************************************
;* cia6526_reset ************************************************************
;****************************************************************************
section .text
cia6526_reset:
			xor	dword eax, eax
			xor	dword ecx, ecx
			push	dword edi
			mov	byte  cl, 010h
			lea	dword edi, [REGISTERS]
			cld
			rep stosb
			pop	dword edi
			mov	dword [STATE], (TA_STOPPED | TB_STOPPED)
			mov	dword [COUNTER_TA], eax
			mov	dword [COUNTER_TB], eax
			mov	byte  [LATCH_PRA], al
			mov	byte  [SIGNAL_PRA], al
			mov	byte  [LATCH_PRB], al
			mov	byte  [SIGNAL_PRB], al
			mov	byte  [LATCH_TA_HI], al
			mov	byte  [LATCH_TB_HI], al
			mov	byte  [IRQ_MASK], al
			ret



;****************************************************************************
;* cia6526_dump *************************************************************
;****************************************************************************
section .text
cia6526_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			PRINT_TEXT  .cb__error, eax, [STATE]
.end:			ret

.cb__error:		db	"-<CIA6526:", STRING_INSERT_HEX32, ">-", 10
			db	"State: ", STRING_INSERT_HEX32, 10, 0




;****************************************************************************
;****************************************************************************
;*
;* PART 3: Emulation Code
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cia6526_read *************************************************************
;****************************************************************************
section .text
cia6526_read:
			mov	dword ebp, ebx
			push	dword ecx
			and	dword ebp, 00000000fh
			mov	dword ecx, [STATE]
			jmp	dword [.ap__registers + 4 * ebp]

.ap__registers:		dd	read_register
			dd	read_register
			dd	read_register
			dd	read_register
			dd	read_ta_lo
			dd	read_ta_hi
			dd	read_tb_lo
			dd	read_tb_hi
			dd	read_register
			dd	read_register
			dd	read_register
			dd	read_tod_hr
			dd	read_register
			dd	read_icr
			dd	read_register
			dd	read_register

read_register:
			mov	byte  al, [REGISTERS + ebp]
			pop	dword ecx
			ret

read_ta_lo:
			test	dword ecx, TA_STOPPED
			jnz	.stopped
			or	dword ecx, TA_LOCKED
			call	read_timer_a
			mov	dword [STATE], ecx
			mov	byte  [LATCH_TA_HI], ah
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TA]
			pop	dword ecx
			ret

read_ta_hi:
			test	dword ecx, TA_STOPPED
			jnz	.stopped
			test	dword ecx, TA_LOCKED
			jnz	.read_from_latch
			call	read_timer_a
			mov	byte  al, ah
			pop	dword ecx
			ret
.read_from_latch:	and	dword ecx, (NOT32 - TA_LOCKED)
			mov	byte  al, [LATCH_TA_HI]
			mov	dword [STATE], ecx
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TA + 1]
			pop	dword ecx
			ret

read_timer_a:
			push	dword ebx
			push	dword edx
			call	core_get_c64_cycles
			mov	dword ebx, [COUNTER_TA]
			sub	dword eax, [TIMESTAMP_TA]
			mov	dword edx, ebx
			add	dword ebx, 2
			test	dword [STATE], TA_FORCE_LOAD
			jz	.no_force_load
			inc	dword ebx
.no_force_load:		sub	dword ebx, eax
			cmp	dword ebx, edx
			jb	.no_correction
			mov	dword ebx, edx
.no_correction:		pop	dword edx
			mov	dword eax, ebx
			pop	dword ebx
			ret

read_tb_lo:
			test	dword ecx, TB_STOPPED
			jnz	.stopped
			or	dword ecx, TB_LOCKED
			call	read_timer_b
			mov	dword [STATE], ecx
			mov	byte  [LATCH_TB_HI], ah
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TB]
			pop	dword ecx
			ret

read_tb_hi:
			test	dword ecx, TB_STOPPED
			jnz	.stopped
			test	dword ecx, TB_LOCKED
			jnz	.read_from_latch
			call	read_timer_b
			mov	byte  al, ah
			pop	dword ecx
			ret
.read_from_latch:	and	dword ecx, (NOT32 - TB_LOCKED)
			mov	byte  al, [LATCH_TB_HI]
			mov	dword [STATE], ecx
			pop	dword ecx
			ret
.stopped:		mov	byte  al, [COUNTER_TB + 1]
			pop	dword ecx
			ret

read_timer_b:
			push	dword ebx
			push	dword edx
			call	core_get_c64_cycles
			mov	dword ebx, [COUNTER_TB]
			sub	dword eax, [TIMESTAMP_TB]
			mov	dword edx, ebx
			add	dword ebx, 2
			test	dword [STATE], TB_FORCE_LOAD
			jz	.no_force_load
			inc	dword ebx
.no_force_load:		sub	dword ebx, eax
			cmp	dword ebx, edx
			jb	.no_correction
			mov	dword ebx, edx
.no_correction:		pop	dword edx
			mov	dword eax, ebx
			pop	dword ebx
			ret

read_tod_hr:
			mov	byte  al, 080h
			pop	dword ecx
			ret

read_icr:
			xor	byte  al, al
			call	dword [LINK_IRQ]
			xor	byte  cl, cl
			mov	byte  al, [REGISTERS + ICR]
			mov	byte  [REGISTERS + ICR], cl
			pop	dword ecx
			ret



;****************************************************************************
;* cia6526_write ************************************************************
;****************************************************************************
section .text
cia6526_write:
			mov	dword ebp, ebx
			and	dword ebp, 00000000fh
			jmp	dword [.ap__registers + 4 * ebp]

.ap__registers:		dd	write_pra
			dd	write_prb
			dd	write_ddra
			dd	write_ddrb
			dd	write_ta_lo
			dd	write_ta_hi
			dd	write_tb_lo
			dd	write_tb_hi
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_ignore
			dd	write_icr
			dd	write_cra
			dd	write_crb

write_ignore:
			ret

write_pra:
			mov	byte  [LATCH_PRA], al
			mov	byte  ah, [REGISTERS + DDRA]
			and	byte  al, ah
			not	byte  ah
			and	byte  ah, [SIGNAL_PRA]
			or	byte  al, ah
			mov	byte  [REGISTERS + PRA], al
			jmp	dword [LINK_PRA]

write_prb:
			mov	byte  [LATCH_PRB], al
			mov	byte  ah, [REGISTERS + DDRB]
			and	byte  al, ah
			not	byte  ah
			and	byte  ah, [SIGNAL_PRB]
			or	byte  al, ah
			mov	byte  [REGISTERS + PRB], al
			jmp	dword [LINK_PRB]

write_ddra:
			mov	byte  [REGISTERS + DDRA], al
			mov	byte  ah, [LATCH_PRA]
			and	byte  ah, al
			not	byte  al
			and	byte  al, [SIGNAL_PRA]
			or	byte  ah, al
			mov	byte  [REGISTERS + PRA], ah
			jmp	dword [LINK_PRA]

write_ddrb:
			mov	byte  [REGISTERS + DDRB], al
			mov	byte  ah, [LATCH_PRB]
			and	byte  ah, al
			not	byte  al
			and	byte  al, [SIGNAL_PRB]
			or	byte  ah, al
			mov	byte  [REGISTERS + PRB], ah
			jmp	dword [LINK_PRB]

write_ta_lo:
			mov	byte  [REGISTERS + TA_LO], al
			ret

write_ta_hi:
			mov	byte  [REGISTERS + TA_HI], al
			test	byte  [REGISTERS + CRA], 001h
			jnz	.timer_in_use
			mov	byte  ah, al
			mov	byte  al, [REGISTERS + TA_LO]
			and	dword eax, NOT16
			mov	dword [COUNTER_TA], eax
.timer_in_use:		ret

write_tb_lo:
			mov	byte  [REGISTERS + TB_LO], al
			ret

write_tb_hi:
			mov	byte  [REGISTERS + TB_HI], al
			test	byte  [REGISTERS + CRB], 001h
			jnz	.timer_in_use
			mov	byte  ah, al
			mov	byte  al, [REGISTERS + TB_LO]
			and	dword eax, NOT16
			mov	dword [COUNTER_TA], eax
.timer_in_use:		ret

write_icr:
			test	byte  al, 080h
			jz	.clear_bits
			and	byte  al, 07fh
			or	byte  [IRQ_MASK], al
			ret
.clear_bits:		xor	byte  al, 07fh
			and	byte  [IRQ_MASK], al
			ret

write_cra:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;fix the behavior if the timer is already running
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword edx, [STATE]
			mov	dword ecx, [COUNTER_TA]
			and	dword edx, (NOT32 - TA_FORCE_LOAD)
			test	byte  al, 010h
			jz	.no_force_load
			mov	byte  cl, [REGISTERS + TA_LO]
			and	byte  al, 0efh
			mov	byte  ch, [REGISTERS + TA_HI]
			or	dword edx, TA_FORCE_LOAD
			mov	dword [COUNTER_TA], ecx
			inc	dword ecx
.no_force_load:		mov	byte  [REGISTERS + CRA], al
			test	byte  al, 001h
			jz	.stop
			and	dword edx, (NOT32 - TA_LOCKED)
			test	byte  al, 020h
			jz	.count_phi2
;!!!!!!!!!!!!!!!!!!!
;implement count cnt
;!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODUL_ID]
			jmp	core_exception
.stop:			call	read_timer_a
			sub	dword eax, 2
			or	dword edx, TA_STOPPED
			mov	dword ebx, eax
			mov	dword [STATE], edx
			and	dword ebx, NOT16
			mov	dword eax, [COUNTER_OFFSET]
			mov	dword [COUNTER_TA], ebx
			call	c64link_cia_counter_clear
			jmp	.end
.count_phi2:		mov	dword eax, [COUNTER_OFFSET]
			mov	dword ebx, process_timer_a
			and	dword edx, (NOT32 - TA_STOPPED)
			add	dword ecx, 2
			mov	dword [STATE], edx
			call	c64link_cia_counter_set
			call	core_get_c64_cycles
			mov	dword [TIMESTAMP_TA], eax
			mov	dword [TIMESTAMP_TA + 4], edx
.end:			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			ret

process_timer_a:
			mov	dword edx, [STATE]
			test	byte  [REGISTERS + CRA], 008h
			jnz	.one_shot
			xor	dword ecx, ecx
			mov	dword eax, [COUNTER_OFFSET]
			mov	byte  cl, [REGISTERS + TA_LO]
			mov	dword ebx, process_timer_a
			mov	byte  ch, [REGISTERS + TA_HI]
			and	dword edx, (NOT32 - TA_FORCE_LOAD)
			mov	dword [COUNTER_TA], ecx
			mov	dword [STATE], edx
			inc	dword ecx
			call	c64link_cia_counter_refresh
			call	core_get_c64_cycles
			mov	dword [TIMESTAMP_TA], eax
			mov	dword [TIMESTAMP_TA + 4], edx
			jmp	.end
.one_shot:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;whats the correct behavior ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			or	dword edx, TA_STOPPED
			mov	dword [STATE], edx
.end:			test	byte  [IRQ_MASK], 001h
			mov	byte  bl, [REGISTERS + ICR]
			jz	.no_irq
			or	byte  bl, 081h
			mov	byte  al, 001h
			mov	byte  [REGISTERS + ICR], bl
			jmp	dword [LINK_IRQ]
.no_irq:		or	byte  bl, 001h
			mov	byte  [REGISTERS + ICR], bl
			ret

write_crb:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;fix the behavior if the timer is already running
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword edx, [STATE]
			mov	dword ecx, [COUNTER_TB]
			and	dword edx, (NOT32 - TB_FORCE_LOAD)
			test	byte  al, 010h
			jz	.no_force_load
			mov	byte  cl, [REGISTERS + TB_LO]
			and	byte  al, 0efh
			mov	byte  ch, [REGISTERS + TB_HI]
			or	dword edx, TB_FORCE_LOAD
			mov	dword [COUNTER_TB], ecx
			inc	dword ecx
.no_force_load:		mov	byte  [REGISTERS + CRB], al
			test	byte  al, 001h
			jz	.stop
			and	dword edx, (NOT32 - TB_LOCKED)
			test	byte  al, 060h
			jz	.count_phi2
			test	byte  al, 040h
			jnz	.cascaded
;!!!!!!!!!!!!!!!!!!!
;implement count cnt
;!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODUL_ID]
			jmp	core_exception
.stop:			call	read_timer_b
			sub	dword eax, 2
			or	dword edx, TB_STOPPED
			mov	dword ebx, eax
			mov	dword [STATE], edx
			and	dword ebx, NOT16
			mov	dword eax, [COUNTER_OFFSET]
			mov	dword [COUNTER_TB], ebx
			inc	dword eax
			call	c64link_cia_counter_clear
			jmp	.end
.cascaded:
;!!!!!!!!!!!!!!!!!!!!!!!
;implement cascaded mode
;!!!!!!!!!!!!!!!!!!!!!!!
			mov	dword eax, [MODUL_ID]
			jmp	core_exception
.count_phi2:		mov	dword eax, [COUNTER_OFFSET]
			mov	dword ebx, process_timer_b
			add	dword ecx, 2
			and	dword edx, (NOT32 - TB_STOPPED)
			inc	dword eax
			mov	dword [STATE], edx
			call	c64link_cia_counter_set
			call	core_get_c64_cycles
			mov	dword [TIMESTAMP_TB], eax
			mov	dword [TIMESTAMP_TB + 4], edx
.end:			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			ret

process_timer_b:
			mov	dword edx, [STATE]
			test	byte  [REGISTERS + CRB], 008h
			jnz	.one_shot
			xor	dword ecx, ecx
			mov	dword eax, [COUNTER_OFFSET]
			mov	byte  cl, [REGISTERS + TB_LO]
			mov	dword ebx, process_timer_b
			mov	byte  ch, [REGISTERS + TB_HI]
			and	dword edx, (NOT32 - TB_FORCE_LOAD)
			mov	dword [COUNTER_TB], ecx
			mov	dword [STATE], edx
			inc	dword ecx
			inc	dword eax
			call	c64link_cia_counter_refresh
			call	core_get_c64_cycles
			mov	dword [TIMESTAMP_TB], eax
			mov	dword [TIMESTAMP_TB + 4], edx
			jmp	.end
.one_shot:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;whats the correct behavior ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			or	dword edx, TB_STOPPED
			mov	dword [STATE], edx
.end:			test	byte  [IRQ_MASK], 002h
			mov	byte  bl, [REGISTERS + ICR]
			jz	.no_irq
			or	byte  bl, 082h
			mov	byte  al, 001h
			mov	byte  [REGISTERS + ICR], bl
			jmp	dword [LINK_IRQ]
.no_irq:		or	byte  bl, 002h
			mov	byte  [REGISTERS + ICR], bl
			ret



;****************************************************************************
;* cia6526_signal_pra *******************************************************
;****************************************************************************
section .text
cia6526_signal_pra:
			push	dword eax
			mov	byte  [SIGNAL_PRA], al
			mov	byte  ah, [REGISTERS + DDRA]
			xor	byte  ah, NOT8
			and	byte  al, ah
			mov	byte  ah, [LATCH_PRA]
			and	byte  ah, [REGISTERS + DDRA]
			or	byte  ah, al
			mov	byte  [REGISTERS + PRA], ah
			pop	dword eax
			ret



;****************************************************************************
;* cia6526_signal_prb *******************************************************
;****************************************************************************
section .text
cia6526_signal_prb:
			push	dword eax
			mov	byte  [SIGNAL_PRB], al
			mov	byte  ah, [REGISTERS + DDRB]
			xor	byte  ah, NOT8
			and	byte  al, ah
			mov	byte  ah, [LATCH_PRB]
			and	byte  ah, [REGISTERS + DDRB]
			or	byte  ah, al
			mov	byte  [REGISTERS + PRB], ah
			pop	dword eax
			ret
;********************************************* karsten.scheibler@bigfoot.de *

