;****************************************************************************
;****************************************************************************
;*
;* core.nasm
;*
;****************************************************************************
;****************************************************************************





%define CORE_NASM
%include "core.inc"
%include "system.inc"
%include "link.inc"
%include "c64link.inc"
%include "vc1541link.inc"
%include "status.inc"
%include "print.inc"
%ifdef CORE_FRAME_PROFILING
%include "sync.inc"
%endif



;****************************************************************************
;* some data ****************************************************************
;****************************************************************************
section .data
			align 4
q__c64_cycles:		dd	0, 0
q__vc1541_cycles:	dd	0, 0
d__cycle_difference1:	dd	0
d__cycle_difference2:	dd	0
d__cycle_counter:	dd	0
d__cycle_counter_base:	dd	0
d__cycles_counted:	dd	0
d__cycles_left:		dd	0

%ifdef CORE_CYCLE_PROFILING
q__vc1541_cpu_cycles:	dd	0, 0
q__vc1541_via1_cycles:	dd	0, 0
q__vc1541_via2_cycles:	dd	0, 0
q__vc1541_disc_cycles:	dd	0, 0
q__c64_cpu_cycles:	dd	0, 0
q__c64_vic_cycles:	dd	0, 0
q__c64_sid_cycles:	dd	0, 0
q__c64_cia1_cycles:	dd	0, 0
q__c64_cia2_cycles:	dd	0, 0
%endif



;****************************************************************************
;* data of all needed Modules ***********************************************
;****************************************************************************
section .bss
c64__1:			resb	c64_size

vc1541__1:		resb	vc1541_size



;****************************************************************************
;* Macro: START_TIMER *******************************************************
;****************************************************************************
%macro START_TIMER 0
%ifdef CORE_CYCLE_PROFILING
			rdtsc
			push	dword eax
			push	dword edx
%endif
%endmacro



;****************************************************************************
;* Macro: STOP_TIMER ********************************************************
;****************************************************************************
%macro STOP_TIMER 1
%ifdef CORE_CYCLE_PROFILING
			rdtsc
			pop	dword ecx
			pop	dword ebx
			sub	dword eax, ebx
			sbb	dword edx, ecx
			add	dword [%1], eax
			adc	dword [%1 + 4], edx
%endif
%endmacro



;****************************************************************************
;* core_start ***************************************************************
;****************************************************************************
section .text
core_start:
%ifdef CORE_TEST_MODULES
%include "core_test.nasm"
%else
			;--------------------------------
			;initialize the emulation modules
			;--------------------------------

			call	c64link_initialize
			call	c64memory_initialize
			call	cpu65xx_initialize
			call	vic6569_initialize
			call	sid6581_initialize
			call	cia6526_initialize
			call	vc1541link_initialize
			call	vc1541memory_initialize
			call	via6522_initialize
			call	disc_initialize
			call	status_initialize

			mov	dword esi, c64__1
			C64LINK_CALL_CREATE_INSTANCE
			C64MEMORY_CALL_CREATE_INSTANCE
%ifdef CPU65XX_EFFECTIVE_ADDRESS
			CPU65XX_CALL_CREATE_INSTANCE  c64memory_read_opcode, c64memory_read_stack, c64memory_read_data, c64memory_write_stack, c64memory_write_data, core_get_c64_cycles
%else
			CPU65XX_CALL_CREATE_INSTANCE  c64memory_read_data, c64memory_write_data, core_get_c64_cycles
%endif
			VIC6569_CALL_CREATE_INSTANCE
			SID6581_CALL_CREATE_INSTANCE
%ifndef CIA6526_EXECUTE_CYCLE
			CIA6526_CALL_CREATE_INSTANCE  cia6526__c64_1, C64LINK_CIA1_COUNTER_OFFSET, c64link_cia1_irq, c64link_cia1_pra, c64link_cia1_prb
			CIA6526_CALL_CREATE_INSTANCE  cia6526__c64_2, C64LINK_CIA2_COUNTER_OFFSET, c64link_cia2_nmi, c64link_cia2_pra, c64link_cia2_prb
%endif

			mov	dword esi, vc1541__1
			VC1541LINK_CALL_CREATE_INSTANCE
			VC1541MEMORY_CALL_CREATE_INSTANCE
%ifdef CPU65XX_EFFECTIVE_ADDRESS
			CPU65XX_CALL_CREATE_INSTANCE  vc1541memory_read_opcode, vc1541memory_read_stack, vc1541memory_read_data, vc1541memory_write_stack, vc1541memory_write_data, core_get_vc1541_cycles
%else
			CPU65XX_CALL_CREATE_INSTANCE  vc1541memory_read_data, vc1541memory_write_data, core_get_vc1541_cycles
%endif
%ifndef VIA6522_EXECUTE_CYCLE
			VIA6522_CALL_CREATE_INSTANCE  via6522__vc1541_1, VC1541LINK_VIA1_COUNTER_OFFSET, vc1541link_via1_irq, vc1541link_via1_pb, vc1541link_via1_cb2, vc1541link_via1_pa, vc1541link_via1_ca2
			VIA6522_CALL_CREATE_INSTANCE  via6522__vc1541_2, VC1541LINK_VIA2_COUNTER_OFFSET, vc1541link_via2_irq, vc1541link_via2_pb, vc1541link_via2_cb2, vc1541link_via2_pa, vc1541link_via2_ca2
%endif

			;--------------------------------------------------
			;reset all emulated devices. remember: cpu, cia and
			;via are emulated with the same programcode but
			;with different data for each device, the according
			;pointer is loaded into esi for the cpu and edi for
			;cia and via
			;--------------------------------------------------

			mov	dword esi, c64__1
			C64LINK_CALL_RESET
			C64MEMORY_CALL_RESET
			CPU65XX_CALL_RESET
			VIC6569_CALL_RESET
			SID6581_CALL_RESET
%ifdef CIA6526_EXECUTE_CYCLE
			CIA6526_CALL_RESET  cia6526__c64_1, 065260001h, c64link_cia1_irq, c64link_cia1_pra, c64link_cia1_prb
			CIA6526_CALL_RESET  cia6526__c64_2, 065260002h, c64link_cia2_nmi, c64link_cia2_pra, c64link_cia2_prb
%else
			CIA6526_CALL_RESET  cia6526__c64_1
			CIA6526_CALL_RESET  cia6526__c64_2
%endif
%ifdef CORE_TEST_VIA
			VIA6522_CALL_RESET  via6522__c64_1, 06522ffffh, link_cia2_irq, link_nothing, link_nothing, link_nothing, link_nothing
%endif
			mov	dword esi, vc1541__1
			VC1541LINK_CALL_RESET
			VC1541MEMORY_CALL_RESET
			CPU65XX_CALL_RESET
%ifdef VIA6522_EXECUTE_CYCLE
			VIA6522_CALL_RESET  via6522__vc1541_1, 065220001h, vc1541link_via1_irq, vc1541link_via1_pb, vc1541link_via1_cb2, vc1541link_via1_pa, vc1541link_via1_ca2
			VIA6522_CALL_RESET  via6522__vc1541_2, 065220002h, vc1541link_via2_irq, vc1541link_via2_pb, vc1541link_via2_cb2, vc1541link_via2_pa, vc1541link_via2_ca2
%else
			VIA6522_CALL_RESET  via6522__vc1541_1
			VIA6522_CALL_RESET  via6522__vc1541_2
%endif
			DISC_CALL_RESET  015410002h

			;----------------------------
			;initialize the clockcounters
			;----------------------------

			mov	dword ebx, VC1541_CLOCK
			mov	dword ecx, C64_CLOCK
			mov	dword eax, ebx
			xor	dword edx, edx
			sub	dword ebx, ecx
			mov	dword [d__cycle_difference1], ebx
			div	dword ebx
			mov	dword [d__cycle_counter_base], eax
			mov	dword [d__cycle_difference2], edx
			mov	dword ecx, edx
			xor	dword edx, edx
			jmp	.calculate_cycles

.run_c64_cycle:		
			;-------------------
			;one c64 clock cycle
			;-------------------

			START_TIMER
			mov	dword esi, c64__1
			CPU65XX_CALL_CYCLE
			STOP_TIMER  q__c64_cpu_cycles
			START_TIMER
			VIC6569_CALL_CYCLE
			STOP_TIMER  q__c64_vic_cycles
			START_TIMER
			SID6581_CALL_CYCLE
			STOP_TIMER  q__c64_sid_cycles
%ifdef CIA6526_EXECUTE_CYCLE
			START_TIMER
			CIA6526_CALL_CYCLE  cia6526__c64_1
			STOP_TIMER  q__c64_cia1_cycles
			START_TIMER
			CIA6526_CALL_CYCLE  cia6526__c64_2
			STOP_TIMER  q__c64_cia2_cycles
%else
			START_TIMER
			C64LINK_CALL_DECREMENT_COUNTER
			STOP_TIMER  q__c64_cia1_cycles
%endif
%ifdef CORE_TEST_VIA
			VIA6522_CALL_CYCLE  via6522__c64_1
%endif

.run_vc1541_cycle:
			;----------------------
			;one vc1541 clock cycle
			;----------------------

			START_TIMER
			mov	dword esi, vc1541__1
			CPU65XX_CALL_CYCLE
			STOP_TIMER  q__vc1541_cpu_cycles
%ifdef VIA6522_EXECUTE_CYCLE
			START_TIMER
			VIA6522_CALL_CYCLE  via6522__vc1541_1
			STOP_TIMER  q__vc1541_via1_cycles
			START_TIMER
			VIA6522_CALL_CYCLE  via6522__vc1541_2
			STOP_TIMER  q__vc1541_via2_cycles
%else
			START_TIMER
			VC1541LINK_CALL_DECREMENT_COUNTER
			STOP_TIMER  q__vc1541_via1_cycles
%endif
			START_TIMER
			DISC_CALL_CYCLE
			STOP_TIMER  q__vc1541_disc_cycles
			dec	dword [d__cycle_counter]
			jnz	near  .run_c64_cycle

			;-----------------------------------
			;add counted cycles to main counters
			;-----------------------------------

			mov	dword eax, [d__cycles_counted]
			mov	dword ebx, [q__vc1541_cycles]
			mov	dword ecx, [q__vc1541_cycles + 4]
			add	dword ebx, eax
			mov	dword edx, [q__c64_cycles]
			adc	dword ecx, 0
			dec	dword eax
			mov	dword ebp, [q__c64_cycles + 4]
			add	dword edx, eax
			mov	dword [q__vc1541_cycles], ebx
			mov	dword [q__vc1541_cycles + 4], ecx
			mov	dword eax, [d__cycle_counter_base]
			adc	dword ebp, 0
			mov	dword ebx, [d__cycle_difference1]
			mov	dword [q__c64_cycles], edx
			mov	dword ecx, [d__cycle_difference2]
			mov	dword edx, [d__cycles_left]
			mov	dword [q__c64_cycles + 4], ebp

			;-------------------------------------
			;calculate cycles for d__cycle_counter
			;-------------------------------------

.calculate_cycles:	add	dword edx, ecx
			cmp	dword edx, ebx
			jb	.no_counter_increase
			inc	dword eax
			sub	dword edx, ebx
.no_counter_increase:	mov	dword [d__cycles_counted], eax
			mov	dword [d__cycle_counter], eax
			mov	dword [d__cycles_left], edx
			jmp	.run_vc1541_cycle
%endif



;****************************************************************************
;* core_exit ****************************************************************
;****************************************************************************
section .text
core_exit:
			mov	dword esi, c64__1
			C64LINK_CALL_DESTROY_INSTANCE
			C64MEMORY_CALL_DESTROY_INSTANCE
			VIC6569_CALL_DESTROY_INSTANCE
			SID6581_CALL_DESTROY_INSTANCE
			mov	dword esi, vc1541__1
			VC1541LINK_CALL_DESTROY_INSTANCE
			ret



;****************************************************************************
;* core_print_info **********************************************************
;****************************************************************************
section .text
core_print_info:
			call	core_get_c64_cycles
			PRINT_TEXT  .cb__c64_cycles, edx, eax
			call	core_get_vc1541_cycles
			PRINT_TEXT  .cb__vc1541_cycles, edx, eax
%ifdef CORE_CYCLE_PROFILING
			call	core_get_vc1541_cycles
			test	dword edx, edx
			jne	near  .c64
.vc1541:		test	dword eax, eax
			je	near  .c64
			mov	dword ecx, eax
			mov	dword eax, [q__vc1541_cpu_cycles]
			mov	dword edx, [q__vc1541_cpu_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__vc1541_cpu, eax
			mov	dword eax, [q__vc1541_via1_cycles]
			mov	dword edx, [q__vc1541_via1_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__vc1541_via1, eax
			mov	dword eax, [q__vc1541_via2_cycles]
			mov	dword edx, [q__vc1541_via2_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__vc1541_via2, eax
			mov	dword eax, [q__vc1541_disc_cycles]
			mov	dword edx, [q__vc1541_disc_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__vc1541_disc, eax
.c64:			call	core_get_c64_cycles
			test	dword edx, edx
			jne	near  .end_cycle_output
			test	dword eax, eax
			je	near  .end_cycle_output
			mov	dword ecx, eax
			mov	dword eax, [q__c64_cpu_cycles]
			mov	dword edx, [q__c64_cpu_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__c64_cpu, eax
			mov	dword eax, [q__c64_vic_cycles]
			mov	dword edx, [q__c64_vic_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__c64_vic, eax
			mov	dword eax, [q__c64_sid_cycles]
			mov	dword edx, [q__c64_sid_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__c64_sid, eax
			mov	dword eax, [q__c64_cia1_cycles]
			mov	dword edx, [q__c64_cia1_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__c64_cia1, eax
			mov	dword eax, [q__c64_cia2_cycles]
			mov	dword edx, [q__c64_cia2_cycles + 4]
			div	dword ecx
			PRINT_TEXT  .cb__c64_cia2, eax
.end_cycle_output:
%endif
%ifdef CORE_FRAME_PROFILING
			call	sync_get_time
			test	dword eax, eax
			jz	.end_frame_output
			PRINT_TEXT  .cb__usec_per_frame, eax
.end_frame_output:
%endif
			ret

.cb__c64_cycles:	db	10, "Overall emulated C64 Cycles:    ", STRING_INSERT_HEX64, "h", 10, 0
.cb__vc1541_cycles:	db	"Overall emulated VC1541 Cycles: ", STRING_INSERT_HEX64, "h", 10, 0

%ifdef CORE_CYCLE_PROFILING
.cb__vc1541_cpu:	db	10, "Cycles needed by VC1541 Emulation:", 10
			db	"  cpu65xx__vc1541:   ", STRING_INSERT_DEC32, 10, 0
.cb__vc1541_via1:	db	"  via6522__vc1541_1: ", STRING_INSERT_DEC32, 10, 0
.cb__vc1541_via2:	db	"  via6522__vc1541_2: ", STRING_INSERT_DEC32, 10, 0
.cb__vc1541_disc:	db	"  disc__vc1541:      ", STRING_INSERT_DEC32, 10, 0
.cb__c64_cpu:		db	10, "Cycles needed by C64 Emulation:", 10
			db	"  cpu65xx__c64:      ", STRING_INSERT_DEC32, 10, 0
.cb__c64_vic:		db	"  vic6569__c64:      ", STRING_INSERT_DEC32, 10, 0
.cb__c64_sid:		db	"  sid6581__c64:      ", STRING_INSERT_DEC32, 10, 0
.cb__c64_cia1:		db	"  cia6526__c64_1:    ", STRING_INSERT_DEC32, 10, 0
.cb__c64_cia2:		db	"  cia6526__c64_2:    ", STRING_INSERT_DEC32, 10, 0
%endif

%ifdef CORE_FRAME_PROFILING
.cb__usec_per_frame:	db	10, "usec per frame:      ", STRING_INSERT_DEC32, 10, 0
%endif



;****************************************************************************
;* core_dump ****************************************************************
;****************************************************************************
section .text
core_dump:
			PRINT_TEXT  .cb__exception, eax
			mov	dword esi, c64__1
			call	c64link_dump
			call	c64memory_dump
			call	cpu65xx_dump
			call	vic6569_dump
			call	sid6581_dump
			lea	dword edi, [esi + cia6526__c64_1]
			call	cia6526_dump
			lea	dword edi, [esi + cia6526__c64_2]
			call	cia6526_dump
			mov	dword esi, vc1541__1
			call	vc1541link_dump
			call	vc1541memory_dump
			call	cpu65xx_dump
			lea	dword edi, [esi + via6522__vc1541_1]
			call	via6522_dump
			lea	dword edi, [esi + via6522__vc1541_2]
			call	via6522_dump
			call	disc_dump
			ret

.cb__exception:		db	10, "An exception occured in Modul ", STRING_INSERT_HEX32, ", dump follows:", 10, 0



;****************************************************************************
;* core_exception ***********************************************************
;****************************************************************************
section .text
core_exception:
			call	system_restore
			call	system_print_info
			call	core_dump
			call	core_print_info
			call	core_exit
			jmp	system_exit



;****************************************************************************
;* core_get_c64_cycles ******************************************************
;****************************************************************************
;* <=eax  low 32 Bits of cycle counter
;* <=edx  high 32 Bits of cycle counter
;****************************************************************************
section .text
core_get_c64_cycles:
			mov	dword eax, [d__cycles_counted]
			sub	dword eax, [d__cycle_counter]
			jnz	.decrease
			jz	.no_decrease
.decrease:		dec	dword eax
.no_decrease:		mov	dword edx, [q__c64_cycles + 4]
			add	dword eax, [q__c64_cycles]
			adc	dword edx, 0
			ret



;****************************************************************************
;* core_get_vc1541_cycles ***************************************************
;****************************************************************************
;* <=eax  low 32 Bits of cycle counter
;* <=edx  high 32 Bits of cycle counter
;****************************************************************************
section .text
core_get_vc1541_cycles:
			mov	dword eax, [d__cycles_counted]
			sub	dword eax, [d__cycle_counter]
			mov	dword edx, [q__vc1541_cycles + 4]
			add	dword eax, [q__vc1541_cycles]
			adc	dword edx, 0
			ret
;********************************************* karsten.scheibler@bigfoot.de *

