;****************************************************************************
;****************************************************************************
;*
;* cpu65xx.inc
;*
;****************************************************************************
;****************************************************************************





%ifndef CPU65XX_INC
%define CPU65XX_INC

%include "defines.inc"

;****************************************************************************
;* Exported Symbols *********************************************************
;****************************************************************************
%ifdef CPU65XX_NASM
global cpu65xx_initialize
global cpu65xx_create_instance
global cpu65xx_destroy_instance
global cpu65xx_reset
global cpu65xx_dump
global cpu65xx_signal_set_irq
global cpu65xx_signal_clear_irq
global cpu65xx_signal_set_nmi
global cpu65xx_signal_set_dma
global cpu65xx_signal_clear_dma
global cpu65xx_signal_set_overflow
%else
extern cpu65xx_initialize
extern cpu65xx_create_instance
extern cpu65xx_destroy_instance
extern cpu65xx_reset
extern cpu65xx_dump
extern cpu65xx_signal_set_irq
extern cpu65xx_signal_clear_irq
extern cpu65xx_signal_set_nmi
extern cpu65xx_signal_set_dma
extern cpu65xx_signal_clear_dma
extern cpu65xx_signal_set_overflow
%endif



;****************************************************************************
;* Module Data **************************************************************
;****************************************************************************
				struc cpu65xx
				alignb 4
d__cpu65xx_modul_id:		resd	1
d__cpu65xx_state:		resd	1
p__cpu65xx_execute_cycle:	resd	1
p__cpu65xx_optional_address:	resd	1
p__cpu65xx_read_opcode:		resd	1
p__cpu65xx_read_opcode_temp:	resd	1
p__cpu65xx_read_stack:		resd	1
p__cpu65xx_read_stack_temp:	resd	1
p__cpu65xx_read_data:		resd	1
p__cpu65xx_read_data_temp:	resd	1
p__cpu65xx_write_stack:		resd	1
p__cpu65xx_write_data:		resd	1
p__cpu65xx_get_cycles:		resd	1
q__cpu65xx_irq_timestamp:	resd	2
q__cpu65xx_nmi_timestamp:	resd	2
q__cpu65xx_dma_timestamp:	resd	2
d__cpu65xx_program_counter:	resd	1
d__cpu65xx_stack_pointer:	resd	1
d__cpu65xx_zeropage_address:	resd	1
d__cpu65xx_effective_address:	resd	1
b__cpu65xx_opcode:		resb	1
b__cpu65xx_data_byte:		resb	1
b__cpu65xx_alu_byte:		resb	1
b__cpu65xx_accumulator:		resb	1
b__cpu65xx_index_x:		resb	1
b__cpu65xx_index_y:		resb	1
b__cpu65xx_flag_register:	resb	1
				endstruc



;****************************************************************************
;* assign's & defines's *****************************************************
;****************************************************************************
%define CPU65XX_MODUL_ID		byte esi + d__cpu65xx_modul_id
%define CPU65XX_STATE			byte esi + d__cpu65xx_state
%define CPU65XX_EXECUTE_CYCLE		byte esi + p__cpu65xx_execute_cycle
%define CPU65XX_OPTIONAL_ADDRESS	byte esi + p__cpu65xx_optional_address
%define CPU65XX_READ_OPCODE		byte esi + p__cpu65xx_read_opcode
%define CPU65XX_READ_OPCODE_TEMP	byte esi + p__cpu65xx_read_opcode_temp
%define CPU65XX_READ_STACK		byte esi + p__cpu65xx_read_stack
%define CPU65XX_READ_STACK_TEMP		byte esi + p__cpu65xx_read_stack_temp
%define CPU65XX_READ_DATA		byte esi + p__cpu65xx_read_data
%define CPU65XX_READ_DATA_TEMP		byte esi + p__cpu65xx_read_data_temp
%define CPU65XX_WRITE_STACK		byte esi + p__cpu65xx_write_stack
%define CPU65XX_WRITE_DATA		byte esi + p__cpu65xx_write_data
%define CPU65XX_GET_CYCLES		byte esi + p__cpu65xx_get_cycles
%define CPU65XX_IRQ_TIMESTAMP		byte esi + q__cpu65xx_irq_timestamp
%define CPU65XX_NMI_TIMESTAMP		byte esi + q__cpu65xx_nmi_timestamp
%define CPU65XX_DMA_TIMESTAMP		byte esi + q__cpu65xx_dma_timestamp
%define CPU65XX_PROGRAM_COUNTER		byte esi + d__cpu65xx_program_counter
%define CPU65XX_STACK_POINTER   	byte esi + d__cpu65xx_stack_pointer
%define CPU65XX_ZEROPAGE_ADDRESS	byte esi + d__cpu65xx_zeropage_address
%define CPU65XX_EFFECTIVE_ADDRESS	byte esi + d__cpu65xx_effective_address
%define CPU65XX_OPCODE			byte esi + b__cpu65xx_opcode
%define CPU65XX_DATA_BYTE		byte esi + b__cpu65xx_data_byte
%define CPU65XX_ALU_BYTE		byte esi + b__cpu65xx_alu_byte
%define CPU65XX_ACCUMULATOR		byte esi + b__cpu65xx_accumulator
%define CPU65XX_INDEX_X			byte esi + b__cpu65xx_index_x
%define CPU65XX_INDEX_Y			byte esi + b__cpu65xx_index_y
%define CPU65XX_FLAG_REGISTER		byte esi + b__cpu65xx_flag_register



;****************************************************************************
;* CPU65XX_STATE Flags ******************************************************
;****************************************************************************
%assign CPU65XX_ILLEGAL_OPCODE		000000001h
%assign	CPU65XX_ILLEGAL_IRQ		000000002h
%assign CPU65XX_SIGNAL_IRQ1		000000100h
%assign CPU65XX_SIGNAL_IRQ2		000000200h
%assign CPU65XX_SIGNAL_IRQ3		000000400h
%assign CPU65XX_SIGNAL_IRQ4		000000800h
%assign CPU65XX_SIGNAL_IRQ		000001000h
%assign CPU65XX_SIGNAL_NMI		000002000h
%assign CPU65XX_SIGNAL_IRQ_OR_NMI	000004000h
%assign CPU65XX_SIGNAL_DMA		000008000h
%assign CPU65XX_PROCESSING_IRQ  	000200000h
%assign CPU65XX_PROCESSING_NMI  	000400000h
%assign CPU65XX_PROCESSING_RESET	000800000h



;****************************************************************************
;* The 6502/6510 Flags ******************************************************
;****************************************************************************
;* The reserved flag is always set. The break flag is only cleared if an irq
;* or nmi occured, otherwise it is set.
;****************************************************************************
%assign CPU65XX_FLAG_N		080h	;negative, 1 = result is negative
%assign CPU65XX_FLAG_N_BIT	7
%assign CPU65XX_FLAG_V		040h	;overflow, 1 = true
%assign CPU65XX_FLAG_V_BIT	6
%assign CPU65XX_FLAG_R		020h	;reserved flag
%assign CPU65XX_FLAG_R_BIT	5
%assign CPU65XX_FLAG_B		010h	;break
%assign CPU65XX_FLAG_B_BIT	4
%assign CPU65XX_FLAG_D		008h	;decimal mode, 1 = enabled
%assign CPU65XX_FLAG_D_BIT	3
%assign CPU65XX_FLAG_I		004h	;irq disable, 1 = irq's disabled
%assign CPU65XX_FLAG_I_BIT	2
%assign CPU65XX_FLAG_Z		002h	;zero, 1 = result is zero
%assign CPU65XX_FLAG_Z_BIT	1
%assign CPU65XX_FLAG_C		001h	;carry, 1 = true
%assign CPU65XX_FLAG_C_BIT	0



;****************************************************************************
;* Macro's ******************************************************************
;****************************************************************************
%macro CPU65XX_CALL_CREATE_INSTANCE 6
			mov	dword eax, %1
			mov	dword ebx, %2
			mov	dword ecx, %3
			mov	dword [CPU65XX_READ_OPCODE], eax
			mov	dword [CPU65XX_READ_OPCODE_TEMP], eax
			mov	dword [CPU65XX_READ_STACK], ebx
			mov	dword [CPU65XX_READ_STACK_TEMP], ebx
			mov	dword [CPU65XX_READ_DATA], ecx
			mov	dword [CPU65XX_READ_DATA_TEMP], ecx
			mov	dword eax, %4
			mov	dword ebx, %5
			mov	dword ecx, %6
			mov	dword [CPU65XX_WRITE_STACK], eax
			mov	dword [CPU65XX_WRITE_DATA], ebx
			mov	dword [CPU65XX_GET_CYCLES], ecx
			call	cpu65xx_create_instance
%endmacro



%macro CPU65XX_CALL_DESTROY_INSTANCE 0
			call	cpu65xx_destroy_instance
%endmacro



%macro CPU65XX_CALL_RESET 0
			call	cpu65xx_reset
%endmacro



%macro CPU65XX_CALL_CYCLE 0
			call	dword [CPU65XX_EXECUTE_CYCLE]
%endmacro



%macro CPU65XX_SET_READ_OPCODE 1
			mov	dword [CPU65XX_READ_OPCODE], %1
%endmacro



%macro CPU65XX_SET_READ_STACK 1
			mov	dword [CPU65XX_READ_STACK], %1
%endmacro



%macro CPU65XX_SET_READ_DATA 1
			mov	dword [CPU65XX_READ_DATA], %1
%endmacro



%macro CPU65XX_SET_WRITE_STACK 1
			mov	dword [CPU65XX_WRITE_STACK], %1
%endmacro



%macro CPU65XX_SET_WRITE_DATA 1
			mov	dword [CPU65XX_WRITE_DATA], %1
%endmacro



%endif
;********************************************* karsten.scheibler@bigfoot.de *

