;****************************************************************************
;****************************************************************************
;*
;* cpu65xx.nasm
;*
;****************************************************************************
;****************************************************************************





%define CPU65XX_NASM
%include "cpu65xx.inc"
%include "core.inc"
%include "print.inc"
%ifdef CPU65XX_LOG
%include "log.inc"
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & defines's
;*
;****************************************************************************
;****************************************************************************





%define MODUL_ID		CPU65XX_MODUL_ID
%define STATE			CPU65XX_STATE
%define EXECUTE_CYCLE		CPU65XX_EXECUTE_CYCLE
%define OPTIONAL_ADDRESS	CPU65XX_OPTIONAL_ADDRESS
%define READ_OPCODE		CPU65XX_READ_OPCODE
%define READ_OPCODE_TEMP	CPU65XX_READ_OPCODE_TEMP
%define READ_STACK		CPU65XX_READ_STACK
%define READ_STACK_TEMP		CPU65XX_READ_STACK_TEMP
%define READ_DATA		CPU65XX_READ_DATA
%define READ_DATA_TEMP		CPU65XX_READ_DATA_TEMP
%define WRITE_STACK		CPU65XX_WRITE_STACK
%define WRITE_DATA		CPU65XX_WRITE_DATA
%define GET_CYCLES		CPU65XX_GET_CYCLES
%define IRQ_TIMESTAMP		CPU65XX_IRQ_TIMESTAMP
%define NMI_TIMESTAMP		CPU65XX_NMI_TIMESTAMP
%define DMA_TIMESTAMP		CPU65XX_DMA_TIMESTAMP
%define PROGRAM_COUNTER		CPU65XX_PROGRAM_COUNTER
%define STACK_POINTER   	CPU65XX_STACK_POINTER
%define ZEROPAGE_ADDRESS	CPU65XX_ZEROPAGE_ADDRESS
%define EFFECTIVE_ADDRESS	CPU65XX_EFFECTIVE_ADDRESS
%define OPCODE			CPU65XX_OPCODE
%define DATA_BYTE		CPU65XX_DATA_BYTE
%define ALU_BYTE		CPU65XX_ALU_BYTE
%define ACCUMULATOR		CPU65XX_ACCUMULATOR
%define INDEX_X			CPU65XX_INDEX_X
%define INDEX_Y			CPU65XX_INDEX_Y
%define FLAG_REGISTER		CPU65XX_FLAG_REGISTER



%assign ILLEGAL_OPCODE		CPU65XX_ILLEGAL_OPCODE
%assign	ILLEGAL_IRQ		CPU65XX_ILLEGAL_IRQ
%assign SIGNAL_IRQ1		CPU65XX_SIGNAL_IRQ1
%assign SIGNAL_IRQ2		CPU65XX_SIGNAL_IRQ2
%assign SIGNAL_IRQ3		CPU65XX_SIGNAL_IRQ3
%assign SIGNAL_IRQ4		CPU65XX_SIGNAL_IRQ4
%assign SIGNAL_IRQ		CPU65XX_SIGNAL_IRQ
%assign SIGNAL_NMI		CPU65XX_SIGNAL_NMI
%assign SIGNAL_IRQ_OR_NMI	CPU65XX_SIGNAL_IRQ_OR_NMI
%assign SIGNAL_DMA		CPU65XX_SIGNAL_DMA
%assign PROCESSING_IRQ		CPU65XX_PROCESSING_IRQ
%assign PROCESSING_NMI		CPU65XX_PROCESSING_NMI
%assign PROCESSING_RESET	CPU65XX_PROCESSING_RESET



%assign FLAG_N			CPU65XX_FLAG_N
%assign FLAG_N_BIT		CPU65XX_FLAG_N_BIT
%assign FLAG_V			CPU65XX_FLAG_V
%assign FLAG_V_BIT		CPU65XX_FLAG_V_BIT
%assign FLAG_R			CPU65XX_FLAG_R
%assign FLAG_R_BIT		CPU65XX_FLAG_R_BIT
%assign FLAG_B			CPU65XX_FLAG_B
%assign FLAG_B_BIT		CPU65XX_FLAG_B_BIT
%assign FLAG_D			CPU65XX_FLAG_D
%assign FLAG_D_BIT		CPU65XX_FLAG_D_BIT
%assign FLAG_I			CPU65XX_FLAG_I
%assign FLAG_I_BIT		CPU65XX_FLAG_I_BIT
%assign FLAG_Z			CPU65XX_FLAG_Z
%assign FLAG_Z_BIT		CPU65XX_FLAG_Z_BIT
%assign FLAG_C			CPU65XX_FLAG_C
%assign FLAG_C_BIT		CPU65XX_FLAG_C_BIT





;****************************************************************************
;****************************************************************************
;*
;* PART 2: Opcode decoding tables
;*
;****************************************************************************
;****************************************************************************





%ifdef CPU65XX_DISABLE_ILLEGAL_OPCODES
section .text
			align 4
ap__opcodes:
			;---------
			;$00 - $0f
			;---------

			dd	opcode_brk,		opcode_illegal
			dd	address_x_indirect,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_ora.read
			dd	address_zeropage,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_php,		opcode_illegal
			dd	opcode_ora.immediate,	opcode_illegal
			dd	opcode_asl.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_ora.read
			dd	address_absolute,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$10 - $1f
			;---------

			dd	opcode_bpl,		opcode_illegal
			dd	address_indirect_y,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_ora.read
			dd	address_zeropage_x,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_clc,		opcode_illegal
			dd	address_absolute_y,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_ora.read
			dd	address_absolute_x_write,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$20 - $2f
			;---------

			dd	opcode_jsr,		opcode_illegal
			dd	address_x_indirect,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_bit.read
			dd	address_zeropage,	opcode_and.read
			dd	address_zeropage,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_plp,		opcode_illegal
			dd	opcode_and.immediate,	opcode_illegal
			dd	opcode_rol.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_bit.read
			dd	address_absolute,	opcode_and.read
			dd	address_absolute,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$30 - $3f
			;---------

			dd	opcode_bmi,		opcode_illegal
			dd	address_indirect_y,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_and.read
			dd	address_zeropage_x,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_sec,		opcode_illegal
			dd	address_absolute_y,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_and.read
			dd	address_absolute_x_write,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$40 - $4f
			;---------

			dd	opcode_rti,		opcode_illegal
			dd	address_x_indirect,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_eor.read
			dd	address_zeropage,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_pha,		opcode_illegal
			dd	opcode_eor.immediate,	opcode_illegal
			dd	opcode_lsr.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_jmp,		opcode_illegal
			dd	address_absolute,	opcode_eor.read
			dd	address_absolute,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$50 - $5f
			;---------

			dd	opcode_bvc,		opcode_illegal
			dd	address_indirect_y,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_eor.read
			dd	address_zeropage_x,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_cli,		opcode_illegal
			dd	address_absolute_y,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_eor.read
			dd	address_absolute_x_write,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$60 - $6f
			;---------

			dd	opcode_rts,		opcode_illegal
			dd	address_x_indirect,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_adc.read
			dd	address_zeropage,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_pla,		opcode_illegal
			dd	opcode_adc.immediate,	opcode_illegal
			dd	opcode_ror.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_jmp_indirect,	opcode_illegal
			dd	address_absolute,	opcode_adc.read
			dd	address_absolute,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$70 - $7f
			;---------

			dd	opcode_bvs,		opcode_illegal
			dd	address_indirect_y,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_adc.read
			dd	address_zeropage_x,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_sei,		opcode_illegal
			dd	address_absolute_y,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_adc.read
			dd	address_absolute_x_write,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$80 - $8f
			;---------

			dd	opcode_illegal,		opcode_illegal
			dd	address_x_indirect,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_sty
			dd	address_zeropage,	opcode_sta
			dd	address_zeropage,	opcode_stx
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_dey,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_txa,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_sty
			dd	address_absolute,	opcode_sta
			dd	address_absolute,	opcode_stx
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$90 - $9f
			;---------

			dd	opcode_bcc,		opcode_illegal
			dd	address_indirect_y_write,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_sty
			dd	address_zeropage_x,	opcode_sta
			dd	address_zeropage_y,	opcode_stx
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_tya,		opcode_illegal
			dd	address_absolute_y_write,	opcode_sta
			dd	opcode_txs,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x_write,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$a0 - $af
			;---------

			dd	opcode_ldy.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_lda.read
			dd	opcode_ldx.immediate,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_ldy.read
			dd	address_zeropage,	opcode_lda.read
			dd	address_zeropage,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_tay,		opcode_illegal
			dd	opcode_lda.immediate,	opcode_illegal
			dd	opcode_tax,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_ldy.read
			dd	address_absolute,	opcode_lda.read
			dd	address_absolute,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$b0 - $bf
			;---------

			dd	opcode_bcs,		opcode_illegal
			dd	address_indirect_y,	opcode_lda.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_ldy.read
			dd	address_zeropage_x,	opcode_lda.read
			dd	address_zeropage_y,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_clv,		opcode_illegal
			dd	address_absolute_y,	opcode_lda.read
			dd	opcode_tsx,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_ldy.read
			dd	address_absolute_x,	opcode_lda.read
			dd	address_absolute_y,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$c0 - $cf
			;---------

			dd	opcode_cpy.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_cpy.read
			dd	address_zeropage,	opcode_cmp.read
			dd	address_zeropage,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_iny,		opcode_illegal
			dd	opcode_cmp.immediate,	opcode_illegal
			dd	opcode_dex,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_cpy.read
			dd	address_absolute,	opcode_cmp.read
			dd	address_absolute,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$d0 - $df
			;---------

			dd	opcode_bne,		opcode_illegal
			dd	address_indirect_y,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_cmp.read
			dd	address_zeropage_x,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_cld,		opcode_illegal
			dd	address_absolute_y,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_cmp.read
			dd	address_absolute_x_write,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$e0 - $ef
			;---------

			dd	opcode_cpx.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_cpx.read
			dd	address_zeropage,	opcode_sbc.read
			dd	address_zeropage,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_inx,		opcode_illegal
			dd	opcode_sbc.immediate,	opcode_illegal
			dd	opcode_nop,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_cpx.read
			dd	address_absolute,	opcode_sbc.read
			dd	address_absolute,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$f0 - $ff
			;---------

			dd	opcode_beq,		opcode_illegal
			dd	address_indirect_y,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_sbc.read
			dd	address_zeropage_x,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal
%ifdef CPU65XX_DISABLE_SED
			dd	opcode_illegal,		opcode_illegal
%else
			dd	opcode_sed,		opcode_illegal
%endif
			dd	address_absolute_y,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_sbc.read
			dd	address_absolute_x_write,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal



%else
;!!!!!!!!!!!!!!!!!!!!!!!!!
;implement illegal opcodes
;!!!!!!!!!!!!!!!!!!!!!!!!!



			;---------
			;$00 - $0f
			;---------

			dd	opcode_brk,		opcode_illegal
			dd	address_x_indirect,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_ora.read
			dd	address_zeropage,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_php,		opcode_illegal
			dd	opcode_ora.immediate,	opcode_illegal
			dd	opcode_asl.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_ora.read
			dd	address_absolute,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$10 - $1f
			;---------

			dd	opcode_bpl,		opcode_illegal
			dd	address_indirect_y,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_ora.read
			dd	address_zeropage_x,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_clc,		opcode_illegal
			dd	address_absolute_y,	opcode_ora.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_ora.read
			dd	address_absolute_x_write,	opcode_asl.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$20 - $2f
			;---------

			dd	opcode_jsr,		opcode_illegal
			dd	address_x_indirect,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_bit.read
			dd	address_zeropage,	opcode_and.read
			dd	address_zeropage,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_plp,		opcode_illegal
			dd	opcode_and.immediate,	opcode_illegal
			dd	opcode_rol.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_bit.read
			dd	address_absolute,	opcode_and.read
			dd	address_absolute,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$30 - $3f
			;---------

			dd	opcode_bmi,		opcode_illegal
			dd	address_indirect_y,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_and.read
			dd	address_zeropage_x,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_sec,		opcode_illegal
			dd	address_absolute_y,	opcode_and.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_and.read
			dd	address_absolute_x_write,	opcode_rol.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$40 - $4f
			;---------

			dd	opcode_rti,		opcode_illegal
			dd	address_x_indirect,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_eor.read
			dd	address_zeropage,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_pha,		opcode_illegal
			dd	opcode_eor.immediate,	opcode_illegal
			dd	opcode_lsr.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_jmp,		opcode_illegal
			dd	address_absolute,	opcode_eor.read
			dd	address_absolute,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$50 - $5f
			;---------

			dd	opcode_bvc,		opcode_illegal
			dd	address_indirect_y,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_eor.read
			dd	address_zeropage_x,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_cli,		opcode_illegal
			dd	address_absolute_y,	opcode_eor.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_eor.read
			dd	address_absolute_x_write,	opcode_lsr.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$60 - $6f
			;---------

			dd	opcode_rts,		opcode_illegal
			dd	address_x_indirect,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_adc.read
			dd	address_zeropage,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_pla,		opcode_illegal
			dd	opcode_adc.immediate,	opcode_illegal
			dd	opcode_ror.accumulator,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_jmp_indirect,	opcode_illegal
			dd	address_absolute,	opcode_adc.read
			dd	address_absolute,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$70 - $7f
			;---------

			dd	opcode_bvs,		opcode_illegal
			dd	address_indirect_y,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_adc.read
			dd	address_zeropage_x,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_sei,		opcode_illegal
			dd	address_absolute_y,	opcode_adc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_adc.read
			dd	address_absolute_x_write,	opcode_ror.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$80 - $8f
			;---------

			dd	opcode_illegal,		opcode_illegal
			dd	address_x_indirect,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_sty
			dd	address_zeropage,	opcode_sta
			dd	address_zeropage,	opcode_stx
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_dey,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_txa,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_sty
			dd	address_absolute,	opcode_sta
			dd	address_absolute,	opcode_stx
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$90 - $9f
			;---------

			dd	opcode_bcc,		opcode_illegal
			dd	address_indirect_y_write,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_sty
			dd	address_zeropage_x,	opcode_sta
			dd	address_zeropage_y,	opcode_stx
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_tya,		opcode_illegal
			dd	address_absolute_y_write,	opcode_sta
			dd	opcode_txs,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x_write,	opcode_sta
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$a0 - $af
			;---------

			dd	opcode_ldy.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_lda.read
			dd	opcode_ldx.immediate,	opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_ldy.read
			dd	address_zeropage,	opcode_lda.read
			dd	address_zeropage,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_tay,		opcode_illegal
			dd	opcode_lda.immediate,	opcode_illegal
			dd	opcode_tax,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_ldy.read
			dd	address_absolute,	opcode_lda.read
			dd	address_absolute,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$b0 - $bf
			;---------

			dd	opcode_bcs,		opcode_illegal
			dd	address_indirect_y,	opcode_lda.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_ldy.read
			dd	address_zeropage_x,	opcode_lda.read
			dd	address_zeropage_y,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_clv,		opcode_illegal
			dd	address_absolute_y,	opcode_lda.read
			dd	opcode_tsx,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_ldy.read
			dd	address_absolute_x,	opcode_lda.read
			dd	address_absolute_y,	opcode_ldx.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$c0 - $cf
			;---------

			dd	opcode_cpy.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_cpy.read
			dd	address_zeropage,	opcode_cmp.read
			dd	address_zeropage,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_iny,		opcode_illegal
			dd	opcode_cmp.immediate,	opcode_illegal
			dd	opcode_dex,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_cpy.read
			dd	address_absolute,	opcode_cmp.read
			dd	address_absolute,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$d0 - $df
			;---------

			dd	opcode_bne,		opcode_illegal
			dd	address_indirect_y,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_cmp.read
			dd	address_zeropage_x,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_cld,		opcode_illegal
			dd	address_absolute_y,	opcode_cmp.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_cmp.read
			dd	address_absolute_x_write,	opcode_dec.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$e0 - $ef
			;---------

			dd	opcode_cpx.immediate,	opcode_illegal
			dd	address_x_indirect,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage,	opcode_cpx.read
			dd	address_zeropage,	opcode_sbc.read
			dd	address_zeropage,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_inx,		opcode_illegal
			dd	opcode_sbc.immediate,	opcode_illegal
			dd	opcode_nop,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute,	opcode_cpx.read
			dd	address_absolute,	opcode_sbc.read
			dd	address_absolute,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal

			;---------
			;$f0 - $ff
			;---------

			dd	opcode_beq,		opcode_illegal
			dd	address_indirect_y,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_zeropage_x,	opcode_sbc.read
			dd	address_zeropage_x,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal
%ifdef CPU65XX_DISABLE_SED
			dd	opcode_illegal,		opcode_illegal
%else
			dd	opcode_sed,		opcode_illegal
%endif
			dd	address_absolute_y,	opcode_sbc.read
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	opcode_illegal,		opcode_illegal
			dd	address_absolute_x,	opcode_sbc.read
			dd	address_absolute_x_write,	opcode_inc.read
			dd	opcode_illegal,		opcode_illegal
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cpu65xx_initialize *******************************************************
;****************************************************************************
section .text
cpu65xx_initialize:
			ret



;****************************************************************************
;* cpu65xx_create_instance **************************************************
;****************************************************************************
section .text
cpu65xx_create_instance:
			push	dword eax
			mov	dword eax, [d__cpu65xx_global_id]
			mov	dword [MODUL_ID], eax
%ifdef CPU65XX_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__cpu65xx_global_id], eax
			pop	dword eax
			ret

section .data
			align 4
d__cpu65xx_global_id:	dd	CPU65XX_GLOBAL_ID



;****************************************************************************
;* cpu65xx_destroy_instance *************************************************
;****************************************************************************
section .text
cpu65xx_destroy_instance:
%ifdef CPU65XX_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			call	log_close
			pop	dword eax
%endif
			ret



%ifdef CPU65XX_LOG
;****************************************************************************
;* cpu65xx_log **************************************************************
;****************************************************************************
;* <=eax  data length
;* <=ebx  pointer to data
;****************************************************************************
section .text
cpu65xx_log:
			push	dword ecx
			push	dword edx
			mov	word  ax, [PROGRAM_COUNTER]
			mov	word  bx, [EFFECTIVE_ADDRESS]
			mov	word  cx, [ZEROPAGE_ADDRESS]
			mov	word  [.ab__status], ax
			mov	word  [.ab__status + 2], bx
;			mov	word  [.ab__status + 4], cx
			call	core_get_c64_cycles
			mov	dword [.ab__status + 4], eax
			mov	byte  al, [OPCODE]
			mov	byte  bl, [STACK_POINTER]
			mov	byte  cl, [FLAG_REGISTER]
			mov	byte  [.ab__status + 8], al
			mov	byte  [.ab__status + 9], bl
			mov	byte  [.ab__status + 10], cl
			mov	byte  al, [ACCUMULATOR]
			mov	byte  bl, [INDEX_X]
			mov	byte  cl, [INDEX_Y]
			mov	byte  [.ab__status + 12], al
			mov	byte  [.ab__status + 13], bl
			mov	byte  [.ab__status + 14], cl
			pop	dword edx
			pop	dword ecx
			mov	dword eax, 16
			mov	dword ebx, .ab__status
			ret

section .data
			align 4
.ab__status:		times 16 db 0
%endif



;****************************************************************************
;* cpu65xx_reset ************************************************************
;****************************************************************************
section .text
cpu65xx_reset:
			mov	byte  [FLAG_REGISTER], (FLAG_R | FLAG_B | FLAG_I)
%ifdef CPU65XX_USE_RESET_ADDRESS
			mov	dword eax, CPU65XX_USE_RESET_ADDRESS
			xor	dword ebx, ebx
			mov	dword [PROGRAM_COUNTER], eax
%else
			mov	dword ebx, 00000fffch
			call	dword [READ_OPCODE]
			inc	dword ebx
			mov	byte  [PROGRAM_COUNTER], al
			call	dword [READ_OPCODE]
			xor	dword ebx, ebx
			mov	byte  [PROGRAM_COUNTER + 1], al
%endif
			mov	dword [ZEROPAGE_ADDRESS], ebx
			xor	dword eax, eax
			mov	dword [EFFECTIVE_ADDRESS], ebx
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;what about PROCESSING_RESET ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;			or	dword eax, PROCESSING_RESET
			inc	byte  bh
			mov	dword [STATE], eax
			mov	dword ecx, get_opcode
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* cpu65xx_dump *************************************************************
;****************************************************************************
;* eax=>  Modul ID
;****************************************************************************
section .text
cpu65xx_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			PRINT_TEXT  .cb__registers, eax
			mov	byte  al, [ACCUMULATOR]
			mov	byte  bl, [INDEX_X]
			mov	byte  cl, [INDEX_Y]
			mov	byte  dl, [STACK_POINTER]
			PRINT_TEXT  .cb__values1, eax, ebx, ecx, edx
			mov	word  ax, [PROGRAM_COUNTER]
			mov	byte  bl, [FLAG_REGISTER]
			mov	byte  cl, [OPCODE]
			mov	dword edx, [STATE]
			PRINT_TEXT  .cb__values2, eax, ebx, ecx, edx
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
.end:			ret

.cb__registers:		db	"-<CPU65XX:", STRING_INSERT_HEX32,">------------------------------------------", 10
			db	"AC XR YR SP _PC_ NV-BDIZC OP --------RNI-----D&NI4321------IO", 10, 0
.cb__values1:		db	STRING_INSERT_HEX8, " "
			db	STRING_INSERT_HEX8, " "
			db	STRING_INSERT_HEX8, " "
			db	STRING_INSERT_HEX8, " ", 0
.cb__values2:		db	STRING_INSERT_HEX16, " "
			db	STRING_INSERT_FLAG8, " "
			db	STRING_INSERT_HEX8, " "
			db	STRING_INSERT_FLAG32, 10, 0





;****************************************************************************
;****************************************************************************
;*
;* PART 4: Handlers for external Signals
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* cpu65xx_signal_set_irq ***************************************************
;****************************************************************************
;* al==>  IRQ Number
;****************************************************************************
section .text
cpu65xx_signal_set_irq:
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	byte  cl, al
			mov	dword ebx, [STATE]
			mov	dword edx, (SIGNAL_IRQ1 / 2)
			test	dword ebx, SIGNAL_IRQ
			jnz	.no_new_timestamp
			push	dword eax
			push	dword edx
			call	dword [GET_CYCLES]
			mov	dword [IRQ_TIMESTAMP], eax
			mov	dword [IRQ_TIMESTAMP + 4], edx
			pop	dword edx
			pop	dword eax
.no_new_timestamp:	shl	dword edx, cl
			and	dword edx, (SIGNAL_IRQ4 | SIGNAL_IRQ3 | SIGNAL_IRQ2 | SIGNAL_IRQ1)
			jnz	.result_ok
			or	dword ebx, ILLEGAL_IRQ
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.result_ok:		or	dword edx, (SIGNAL_IRQ | SIGNAL_IRQ_OR_NMI)
			or	dword ebx, edx
			pop	dword edx
			mov	dword [STATE], ebx
			pop	dword ecx
			pop	dword ebx
			ret



;****************************************************************************
;* cpu65xx_signal_clear_irq *************************************************
;****************************************************************************
;* al==>  IRQ Number
;****************************************************************************
section .text
cpu65xx_signal_clear_irq:
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	byte  cl, al
			mov	dword edx, (SIGNAL_IRQ1 / 2)
			mov	dword ebx, [STATE]
			shl	dword edx, cl
			and	dword edx, (SIGNAL_IRQ4 | SIGNAL_IRQ3 | SIGNAL_IRQ2 | SIGNAL_IRQ1)
			jnz	.result_ok
			or	dword ebx, ILLEGAL_IRQ
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.result_ok:		not	dword edx
			and	dword ebx, edx
			test	dword ebx, (SIGNAL_IRQ4 | SIGNAL_IRQ3 | SIGNAL_IRQ2 | SIGNAL_IRQ1)
			jnz	.end
			and	dword ebx, (NOT32 - SIGNAL_IRQ)
			test	dword ebx, SIGNAL_NMI
			jnz	.end
			and	dword ebx, (NOT32 - SIGNAL_IRQ_OR_NMI)
.end:			pop	dword edx
			mov	dword [STATE], ebx
			pop	dword ecx
			pop	dword ebx
			ret



;****************************************************************************
;* cpu65xx_signal_set_nmi ***************************************************
;****************************************************************************
section .text
cpu65xx_signal_set_nmi:
			push	dword eax
			push	dword edx
			or	dword [STATE], (SIGNAL_NMI | SIGNAL_IRQ_OR_NMI)
			call	dword [GET_CYCLES]
			mov	dword [NMI_TIMESTAMP], eax
			mov	dword [NMI_TIMESTAMP + 4], edx
			pop	dword edx
			pop	dword eax
			ret



;****************************************************************************
;* cpu65xx_signal_set_dma ***************************************************
;****************************************************************************
section .text
cpu65xx_signal_set_dma:
			test	dword [STATE], SIGNAL_DMA
			jnz	.end
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword eax, [READ_OPCODE]
			mov	dword ebx, [READ_STACK]
			mov	dword ecx, [READ_DATA]
			mov	dword [READ_OPCODE_TEMP], eax
			mov	dword [READ_STACK_TEMP], ebx
			mov	dword [READ_DATA_TEMP], ecx
			mov	dword eax, .ignore_read
			mov	dword ebx, [STATE]
			mov	dword [READ_OPCODE], eax
			or	dword ebx, SIGNAL_DMA
			mov	dword [READ_STACK], eax
			mov	dword [STATE], ebx
			mov	dword [READ_DATA], eax
			call	dword [GET_CYCLES]
			mov	dword [DMA_TIMESTAMP], eax
			mov	dword [DMA_TIMESTAMP + 4], edx
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
.end:			ret

.ignore_read:		pop	dword eax
			ret



;****************************************************************************
;* cpu65xx_signal_clear_dma *************************************************
;****************************************************************************
section .text
cpu65xx_signal_clear_dma:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;influence the irq, nmi timestamp ??
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			test	dword [STATE], SIGNAL_DMA
			jz	.end
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, [READ_OPCODE_TEMP]
			mov	dword ebx, [READ_STACK_TEMP]
			mov	dword ecx, [READ_DATA_TEMP]
			mov	dword [READ_OPCODE], eax
			mov	dword [READ_STACK], ebx
			mov	dword eax, [STATE]
			mov	dword [READ_DATA], ecx
			and	dword eax, (NOT32 - SIGNAL_DMA)
			pop	dword ecx
			mov	dword [STATE], eax
			pop	dword ebx
			pop	dword eax
.end:			ret



;****************************************************************************
;* cpu65xx_signal_set_overflow **********************************************
;****************************************************************************
section .text
cpu65xx_signal_set_overflow:
			or	byte  [FLAG_REGISTER], FLAG_V
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 5: Emulation Code
;*
;****************************************************************************
;****************************************************************************










;****************************************************************************
;****************************************************************************
;*
;* PART 5.1: Addressing Modes
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* CHECK_FOR_DMA ************************************************************
;****************************************************************************
%macro CHECK_FOR_DMA 0
			test	dword [STATE], SIGNAL_DMA
			jz	%%no_dma
			ret
%%no_dma:
%endmacro



;****************************************************************************
;* address ******************************************************************
;****************************************************************************
section .text
address:
.delay:			CHECK_FOR_DMA
			mov	dword eax, [OPTIONAL_ADDRESS]
			mov	dword [EXECUTE_CYCLE], eax
			ret

.high_byte:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword ecx, [OPTIONAL_ADDRESS]
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.high_byte_increase:	mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	byte  al
			inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword ecx, .delay
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.high_byte_delay:	mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword ecx, .delay
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_zeropage *********************************************************
;****************************************************************************
section .text
address_zeropage:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			xor	dword ecx, ecx
			inc	word  bx
			mov	byte  cl, al
			mov	dword edx, [OPTIONAL_ADDRESS]
			mov	dword [EFFECTIVE_ADDRESS], ecx
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], edx
			ret



;****************************************************************************
;* address_zeropage_x *******************************************************
;****************************************************************************
section .text
address_zeropage_x:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			xor	dword ecx, ecx
			add	byte  al, [INDEX_X]
			inc	word  bx
			mov	byte  cl, al
			mov	dword edx, address.delay
			mov	dword [EFFECTIVE_ADDRESS], ecx
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], edx
			ret



;****************************************************************************
;* address_zeropage_y *******************************************************
;****************************************************************************
section .text
address_zeropage_y:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			xor	dword ecx, ecx
			add	byte  al, [INDEX_Y]
			inc	word  bx
			mov	byte  cl, al
			mov	dword edx, address.delay
			mov	dword [EFFECTIVE_ADDRESS], ecx
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], edx
			ret



;****************************************************************************
;* address_absolute *********************************************************
;****************************************************************************
section .text
address_absolute:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword ecx, address.high_byte
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_absolute_x *******************************************************
;****************************************************************************
section .text
address_absolute_x:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			add	byte  al, [INDEX_X]
			mov	dword ecx, address.high_byte
			jnc	.cycle1_1
			mov	dword ecx, address.high_byte_increase
.cycle1_1:		inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_absolute_y *******************************************************
;****************************************************************************
section .text
address_absolute_y:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			add	byte  al, [INDEX_Y]
			mov	dword ecx, address.high_byte
			jnc	.cycle1_1
			mov	dword ecx, address.high_byte_increase
.cycle1_1:		inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_x_indirect *******************************************************
;****************************************************************************
section .text
address_x_indirect:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			add	byte  al, [INDEX_X]
			inc	word  bx
			mov	dword ecx, .cycle2
			mov	byte  [ZEROPAGE_ADDRESS], al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		mov	dword ebx, [ZEROPAGE_ADDRESS]
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			mov	dword ecx, .cycle4
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle4:		mov	dword ebx, [ZEROPAGE_ADDRESS]
			mov	dword ecx, [OPTIONAL_ADDRESS]
			inc	byte  bl
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_indirect_y *******************************************************
;****************************************************************************
section .text
address_indirect_y:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [ZEROPAGE_ADDRESS], al
			mov	dword ecx, .cycle2
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		mov	dword ebx, [ZEROPAGE_ADDRESS]
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			add	byte  al, [INDEX_Y]
			mov	dword ecx, .cycle3_1
			jnc	.cycle2_1
			mov	dword ecx, .cycle3_2
.cycle2_1:		mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle3_1:		mov	dword ebx, [ZEROPAGE_ADDRESS]
			mov	dword ecx, [OPTIONAL_ADDRESS]
			inc	byte  bl
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			mov	dword [EXECUTE_CYCLE], ecx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			ret

.cycle3_2:		mov	dword ebx, [ZEROPAGE_ADDRESS]
			mov	dword ecx, address.delay
			inc	byte  bl
			call	dword [READ_DATA]
			inc	byte  al
			mov	dword [EXECUTE_CYCLE], ecx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			ret



;****************************************************************************
;* address_absolute_x_write *************************************************
;****************************************************************************
section .text
address_absolute_x_write:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			add	byte  al, [INDEX_X]
			mov	dword ecx, address.high_byte_delay
			jnc	.cycle1_1
			mov	dword ecx, address.high_byte_increase
.cycle1_1:		inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_absolute_y_write *************************************************
;****************************************************************************
section .text
address_absolute_y_write:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			add	byte  al, [INDEX_Y]
			mov	dword ecx, address.high_byte_delay
			jnc	.cycle1_1
			mov	dword ecx, address.high_byte_increase
.cycle1_1:		inc	word  bx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* address_indirect_y_write *************************************************
;****************************************************************************
section .text
address_indirect_y_write:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [ZEROPAGE_ADDRESS], al
			mov	dword ecx, .cycle2
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		mov	dword ebx, [ZEROPAGE_ADDRESS]
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			add	byte  al, [INDEX_Y]
			mov	dword ecx, .cycle3_1
			jnc	.cycle2_1
			mov	dword ecx, .cycle3_2
.cycle2_1:		mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle3_1:		mov	dword ebx, [ZEROPAGE_ADDRESS]
			mov	dword ecx, address.delay
			inc	byte  bl
;!!!!!!!!!!!!!!!
;READ_ZEROPAGE ?
;!!!!!!!!!!!!!!!
			call	dword [READ_DATA]
			mov	dword [EXECUTE_CYCLE], ecx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			ret

;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;use address_indirect_y.cycle3_2 ?
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
.cycle3_2:		mov	dword ebx, [ZEROPAGE_ADDRESS]
			mov	dword ecx, address.delay
			inc	byte  bl
			call	dword [READ_DATA]
			inc	byte  al
			mov	dword [EXECUTE_CYCLE], ecx
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			ret



;****************************************************************************
;* address_rmw **************************************************************
;****************************************************************************
section .text
address_rmw:
.delay:			mov	byte  al, [DATA_BYTE]
			mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	dword ecx, .write
			call	dword [WRITE_DATA]
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.write:			mov	byte  al, [ALU_BYTE]
			mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	dword ecx, get_opcode
			call	dword [WRITE_DATA]
			mov	dword [EXECUTE_CYCLE], ecx
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 5.2: Opcodes
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* opcode_adc ***************************************************************
;****************************************************************************
section .text
opcode_adc:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  ah, [ACCUMULATOR]
			mov	dword [PROGRAM_COUNTER], ebx
			test	byte  [FLAG_REGISTER], FLAG_D
			jz	.binary
.decimal:		test	byte  [FLAG_REGISTER], FLAG_C
			jz	.carry_clear_1
			stc
.carry_clear_1:		adc	byte  ah, al
			daa
			setc	byte  cl
			jmp	opcode_sbc.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  ah, [ACCUMULATOR]
			test	byte  [FLAG_REGISTER], FLAG_D
			jnz	.decimal
.binary:		test	byte  [FLAG_REGISTER], FLAG_C
			jz	.carry_clear_2
			stc
.carry_clear_2:		adc	byte  ah, al
			setc	byte  cl
			jmp	opcode_sbc.common



;****************************************************************************
;* opcode_sbc ***************************************************************
;****************************************************************************
section .text
opcode_sbc:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  ah, [ACCUMULATOR]
			mov	dword [PROGRAM_COUNTER], ebx
			test	byte  [FLAG_REGISTER], FLAG_D
			jz	.binary
.decimal:		test	byte  [FLAG_REGISTER], FLAG_C
			jz	.carry_clear_1
			stc
.carry_clear_1:		cmc
			sbb	byte  ah, al
			das
			setnc	byte  cl
			jmp	.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  ah, [ACCUMULATOR]
			test	byte  [FLAG_REGISTER], FLAG_D
			jnz	.decimal
.binary:		test	byte  [FLAG_REGISTER], FLAG_C
			jz	.carry_clear_2
			stc
.carry_clear_2:		cmc
			sbb	byte  ah, al
			setnc	byte  cl

.common:		setz	byte  ch
			seto	byte  dl
			sets	byte  dh
			shl	byte  ch, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_V_BIT
			mov	byte  [ACCUMULATOR], ah
			shl	byte  dh, FLAG_N_BIT
			or	byte  cl, ch
			or	byte  dl, dh
			and	byte  bl, (NOT8 - FLAG_N - FLAG_V - FLAG_Z - FLAG_C)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_cmp ***************************************************************
;****************************************************************************
section .text
opcode_cmp:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  ah, [ACCUMULATOR]
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_cpy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  ah, [ACCUMULATOR]
			jmp	opcode_cpy.common



;****************************************************************************
;* opcode_cpx ***************************************************************
;****************************************************************************
section .text
opcode_cpx:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  ah, [INDEX_X]
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_cpy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  ah, [INDEX_X]
			jmp	opcode_cpy.common



;****************************************************************************
;* opcode_cpy ***************************************************************
;****************************************************************************
section .text
opcode_cpy:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  ah, [INDEX_Y]
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_cpy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  ah, [INDEX_Y]

.common:		sub	byte  ah, al
			setnc	byte  cl
			setz	byte  ch
			sets	byte  dl
			shl	byte  ch, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			or	byte  cl, ch
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z - FLAG_C)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_and ***************************************************************
;****************************************************************************
section .text
opcode_and:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_OPCODE]
			inc	word  bx
			and	byte  ch, al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ora.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_DATA]
			and	byte  ch, al
			jmp	opcode_ora.common



;****************************************************************************
;* opcode_eor ***************************************************************
;****************************************************************************
section .text
opcode_eor:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_OPCODE]
			inc	word  bx
			xor	byte  ch, al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ora.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_DATA]
			xor	byte  ch, al
			jmp	opcode_ora.common



;****************************************************************************
;* opcode_ora ***************************************************************
;****************************************************************************
section .text
opcode_ora:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_OPCODE]
			inc	word  bx
			or	byte  ch, al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ora.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	byte  ch, [ACCUMULATOR]
			call	dword [READ_DATA]
			or	byte  ch, al

.common:		setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			mov	byte  [ACCUMULATOR], ch
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_asl ***************************************************************
;****************************************************************************
section .text
opcode_asl:
.accumulator:		CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			shl	byte  al, 1
			jmp	opcode_ror.accumulator_common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			shl	byte  al, 1
			jmp	opcode_ror.read_common



;****************************************************************************
;* opcode_lsr ***************************************************************
;****************************************************************************
section .text
opcode_lsr:
.accumulator:		CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			shr	byte  al, 1
			jmp	opcode_ror.accumulator_common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			shr	byte  al, 1
			jmp	opcode_ror.read_common



;****************************************************************************
;* opcode_rol ***************************************************************
;****************************************************************************
section .text
opcode_rol:
.accumulator:		CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			test	byte  [FLAG_REGISTER], CPU65XX_FLAG_C
			jz	.carry_clear_1
			stc
.carry_clear_1:		rcl	byte  al, 1
			jmp	opcode_ror.accumulator_common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			test	byte  [FLAG_REGISTER], CPU65XX_FLAG_C
			jz	.carry_clear_2
			stc
.carry_clear_2:		rcl	byte  al, 1
			jmp	opcode_ror.read_common



;****************************************************************************
;* opcode_ror ***************************************************************
;****************************************************************************
section .text
opcode_ror:
.accumulator:		CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			test	byte  [FLAG_REGISTER], CPU65XX_FLAG_C
			jz	.carry_clear_1
			stc
.carry_clear_1:		rcr	byte  al, 1

.accumulator_common:	setc	byte  cl
			test	byte  al, al
			setz	byte  ch
			sets	byte  dl
			shl	byte  ch, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			or	byte  cl, ch
			mov	byte  [ACCUMULATOR], al
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z - FLAG_C)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			test	byte  [FLAG_REGISTER], CPU65XX_FLAG_C
			jz	.carry_clear_2
			stc
.carry_clear_2:		rcr	byte  al, 1

.read_common:		setc	byte  cl
			test	byte  al, al
			setz	byte  ch
			sets	byte  dl
			shl	byte  ch, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			or	byte  cl, ch
			mov	byte  [ALU_BYTE], al
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z - FLAG_C)
			or	byte  cl, dl
			mov	dword eax, address_rmw
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_dec ***************************************************************
;****************************************************************************
section .text
opcode_dec:
.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			dec	byte  al
			jmp	opcode_inc.common


;****************************************************************************
;* opcode_inc ***************************************************************
;****************************************************************************
section .text
opcode_inc:
.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [DATA_BYTE], al
			inc	byte  al
.common:		setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			mov	byte  [ALU_BYTE], al
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, address_rmw
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_dex ***************************************************************
;****************************************************************************
section .text
opcode_dex:
			CHECK_FOR_DMA
			dec	byte  [INDEX_X]
			jmp	opcode_iny.common



;****************************************************************************
;* opcode_dey ***************************************************************
;****************************************************************************
section .text
opcode_dey:
			CHECK_FOR_DMA
			dec	byte  [INDEX_Y]
			jmp	opcode_iny.common



;****************************************************************************
;* opcode_inx ***************************************************************
;****************************************************************************
section .text
opcode_inx:
			CHECK_FOR_DMA
			inc	byte  [INDEX_X]
			jmp	opcode_iny.common


;****************************************************************************
;* opcode_iny ***************************************************************
;****************************************************************************
section .text
opcode_iny:
			CHECK_FOR_DMA
			inc	byte  [INDEX_Y]
.common:		setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_bcc ***************************************************************
;****************************************************************************
section .text
opcode_bcc:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_C
			mov	dword [PROGRAM_COUNTER], ebx
			jz	near  opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bcs ***************************************************************
;****************************************************************************
section .text
opcode_bcs:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_C
			mov	dword [PROGRAM_COUNTER], ebx
			jnz	near  opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_beq ***************************************************************
;****************************************************************************
section .text
opcode_beq:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_Z
			mov	dword [PROGRAM_COUNTER], ebx
			jnz	near  opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bne ***************************************************************
;****************************************************************************
section .text
opcode_bne:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_Z
			mov	dword [PROGRAM_COUNTER], ebx
			jz	opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bmi ***************************************************************
;****************************************************************************
section .text
opcode_bmi:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_N
			mov	dword [PROGRAM_COUNTER], ebx
			jnz	opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bpl ***************************************************************
;****************************************************************************
section .text
opcode_bpl:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_N
			mov	dword [PROGRAM_COUNTER], ebx
			jz	opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bvc ***************************************************************
;****************************************************************************
section .text
opcode_bvc:
			mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_V
			mov	dword [PROGRAM_COUNTER], ebx
			jz	opcode_bvs.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_bvs ***************************************************************
;****************************************************************************
section .text
opcode_bvs:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			test	byte  [FLAG_REGISTER], FLAG_V
			mov	dword [PROGRAM_COUNTER], ebx
			jnz	.common
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret

.common:		cbw
			mov	word  [EFFECTIVE_ADDRESS], ax
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		CHECK_FOR_DMA
			mov	dword eax, [EFFECTIVE_ADDRESS]
			mov	dword ebx, [PROGRAM_COUNTER]
			add	word  ax, bx
			mov	dword [PROGRAM_COUNTER], eax
			cmp	byte  ah, bh
			je	.cycle3_1
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		CHECK_FOR_DMA
.cycle3_1:		mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_clc ***************************************************************
;****************************************************************************
section .text
opcode_clc:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			and	byte  al, (NOT8 - FLAG_C)
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret



;****************************************************************************
;* opcode_cld ***************************************************************
;****************************************************************************
section .text
opcode_cld:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			and	byte  al, (NOT8 - FLAG_D)
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret



;****************************************************************************
;* opcode_cli ***************************************************************
;****************************************************************************
section .text
opcode_cli:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			test	byte  al, FLAG_I
			jz	.flag_cleared
			and	byte  al, (NOT8 - FLAG_I)
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			call	dword [GET_CYCLES]
			mov	dword [IRQ_TIMESTAMP], eax
			mov	dword [IRQ_TIMESTAMP + 4], edx
			ret
.flag_cleared:		mov	dword [EXECUTE_CYCLE], ebx
			ret



;****************************************************************************
;* opcode_clv ***************************************************************
;****************************************************************************
section .text
opcode_clv:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			and	byte  al, (NOT8 - FLAG_V)
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret



;****************************************************************************
;* opcode_sec ***************************************************************
;****************************************************************************
section .text
opcode_sec:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			or	byte  al, FLAG_C
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret



;****************************************************************************
;* opcode_sed ***************************************************************
;****************************************************************************
section .text
opcode_sed:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			or	byte  al, FLAG_D
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret



;****************************************************************************
;* opcode_sei ***************************************************************
;****************************************************************************
section .text
opcode_sei:
			CHECK_FOR_DMA
			mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, get_opcode
			or	byte  al, FLAG_I
			mov	dword [EXECUTE_CYCLE], ebx
			mov	byte  [FLAG_REGISTER], al
			ret


;****************************************************************************
;* opcode_tax ***************************************************************
;****************************************************************************
section .text
opcode_tax:
			CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			mov	byte  [INDEX_X], al
			jmp	opcode_tya.common



;****************************************************************************
;* opcode_tay ***************************************************************
;****************************************************************************
section .text
opcode_tay:
			CHECK_FOR_DMA
			mov	byte  al, [ACCUMULATOR]
			mov	byte  [INDEX_Y], al
			jmp	opcode_tya.common



;****************************************************************************
;* opcode_tsx ***************************************************************
;****************************************************************************
section .text
opcode_tsx:
			CHECK_FOR_DMA
			mov	byte  al, [STACK_POINTER]
			mov	byte  [INDEX_X], al
			jmp	opcode_tya.common



;****************************************************************************
;* opcode_txa ***************************************************************
;****************************************************************************
section .text
opcode_txa:
			CHECK_FOR_DMA
			mov	byte  al, [INDEX_X]
			mov	byte  [ACCUMULATOR], al
			jmp	opcode_tya.common



;****************************************************************************
;* opcode_txs ***************************************************************
;****************************************************************************
section .text
opcode_txs:
			CHECK_FOR_DMA
			mov	byte  al, [INDEX_X]
			mov	dword [EXECUTE_CYCLE], get_opcode
			mov	byte  [STACK_POINTER], al
			ret


;****************************************************************************
;* opcode_tya ***************************************************************
;****************************************************************************
section .text
opcode_tya:
			CHECK_FOR_DMA
			mov	byte  al, [INDEX_Y]
			mov	byte  [ACCUMULATOR], al
.common:		test	byte  al, al
			setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_lda ***************************************************************
;****************************************************************************
section .text
opcode_lda:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [ACCUMULATOR], al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ldy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [ACCUMULATOR], al
			jmp	opcode_ldy.common



;****************************************************************************
;* opcode_ldx ***************************************************************
;****************************************************************************
section .text
opcode_ldx:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [INDEX_X], al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ldy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [INDEX_X], al
			jmp	opcode_ldy.common



;****************************************************************************
;* opcode_ldy ***************************************************************
;****************************************************************************
section .text
opcode_ldy:
.immediate:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	byte  [INDEX_Y], al
			mov	dword [PROGRAM_COUNTER], ebx
			jmp	opcode_ldy.common

.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  [INDEX_Y], al

.common:		test	byte  al, al
			setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, get_opcode
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_sta ***************************************************************
;****************************************************************************
section .text
opcode_sta:
			mov	byte  al, [ACCUMULATOR]
			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [WRITE_DATA]
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_stx ***************************************************************
;****************************************************************************
section .text
opcode_stx:
			mov	byte  al, [INDEX_X]
			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [WRITE_DATA]
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_sty ***************************************************************
;****************************************************************************
section .text
opcode_sty:
			mov	byte  al, [INDEX_Y]
			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [WRITE_DATA]
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_jmp ***************************************************************
;****************************************************************************
section .text
opcode_jmp:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			mov	dword ecx, .cycle2
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		mov	dword ebx, [PROGRAM_COUNTER]
			xor	dword ecx, ecx
			inc	word  bx
			call	dword [READ_OPCODE]
			mov	byte  ch, al
			mov	dword edx, get_opcode
			mov	byte  cl, [EFFECTIVE_ADDRESS]
			mov	dword [EXECUTE_CYCLE], edx
			mov	dword [PROGRAM_COUNTER], ecx
			ret



;****************************************************************************
;* opcode_jmp_indirect ******************************************************
;****************************************************************************
section .text
opcode_jmp_indirect:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			mov	dword ecx, .cycle2
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		mov	dword ebx, [PROGRAM_COUNTER]
			mov	dword ecx, .cycle3
			inc	byte  bl		;no page crossing
			call	dword [READ_OPCODE]
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle3:		mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	dword ecx, .cycle4
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle4:		mov	dword ebx, [EFFECTIVE_ADDRESS]
			mov	dword ecx, get_opcode
			inc	word  bx
			call	dword [READ_DATA]
			mov	byte  [PROGRAM_COUNTER + 1], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret



;****************************************************************************
;* opcode_jsr ***************************************************************
;****************************************************************************
section .text
opcode_jsr:
.cycle1:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			inc	word  bx
			mov	dword ecx, .cycle2
			mov	dword [PROGRAM_COUNTER], ebx
			mov	byte  [EFFECTIVE_ADDRESS], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle2:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			mov	dword ecx, .cycle3
			mov	byte  [EFFECTIVE_ADDRESS + 1], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle3:		mov	byte  al, [PROGRAM_COUNTER + 1]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle4
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle4:		mov	byte  al, [PROGRAM_COUNTER]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle5
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle5:		CHECK_FOR_DMA
			mov	dword eax, [EFFECTIVE_ADDRESS]
			mov	dword [EXECUTE_CYCLE], get_opcode
			mov	dword [PROGRAM_COUNTER], eax
			ret



;****************************************************************************
;* opcode_bit ***************************************************************
;****************************************************************************
section .text
opcode_bit:
.read:			mov	dword ebx, [EFFECTIVE_ADDRESS]
			call	dword [READ_DATA]
			mov	byte  bl, [FLAG_REGISTER]
			and	byte  bl, (3fh - FLAG_Z)
			test	byte  [ACCUMULATOR], al
			jnz	.bit1
			or	byte  bl, FLAG_Z
.bit1:			and	byte  al, 0c0h
			mov	dword ecx, get_opcode
			or	byte  bl, al
			mov	dword [EXECUTE_CYCLE], ecx
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_brk ***************************************************************
;****************************************************************************
section .text
opcode_brk:
.cycle1:		CHECK_FOR_DMA
			inc	word  [PROGRAM_COUNTER]
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	dword ebx, [STACK_POINTER]
			mov	byte  al, [PROGRAM_COUNTER + 1]
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], .cycle3
			mov	dword [STACK_POINTER], ebx
			ret

.cycle3:		mov	dword ebx, [STACK_POINTER]
			mov	byte  al, [PROGRAM_COUNTER]
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], .cycle4
			mov	dword [STACK_POINTER], ebx
			ret

.cycle4:		mov	dword ebx, [STACK_POINTER]
			mov	byte  al, [FLAG_REGISTER]
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], .cycle5
			mov	dword [STACK_POINTER], ebx
			ret

.cycle5:		mov	dword ebx, 00000fffeh
			call	dword [READ_OPCODE]
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [EXECUTE_CYCLE], .cycle6
			ret

.cycle6:		mov	dword ebx, 00000ffffh
			call	dword [READ_OPCODE]
			mov	byte  bl, [FLAG_REGISTER]
			mov	byte  [PROGRAM_COUNTER + 1], al
			or	byte  bl, (FLAG_B | FLAG_I)
			mov	dword [EXECUTE_CYCLE], get_opcode
			mov	byte  [FLAG_REGISTER], bl
			ret



;****************************************************************************
;* opcode_nop ***************************************************************
;****************************************************************************
section .text
opcode_nop:
			CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_pha ***************************************************************
;****************************************************************************
section .text
opcode_pha:
.cycle1:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	dword ebx, [STACK_POINTER]
			mov	byte  al, [ACCUMULATOR]
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], get_opcode
			mov	dword [STACK_POINTER], ebx
			ret



;****************************************************************************
;* opcode_php ***************************************************************
;****************************************************************************
section .text
opcode_php:
.cycle1:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	dword ebx, [STACK_POINTER]
			mov	byte  al, [FLAG_REGISTER]
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], get_opcode
			mov	dword [STACK_POINTER], ebx
			ret



;****************************************************************************
;* opcode_pla ***************************************************************
;****************************************************************************
section .text
opcode_pla:
.cycle1:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			mov	byte  [ACCUMULATOR], al
			mov	dword [STACK_POINTER], ebx
			test	byte  al, al
			setz	byte  cl
			sets	byte  dl
			shl	byte  cl, FLAG_Z_BIT
			mov	byte  bl, [FLAG_REGISTER]
			shl	byte  dl, FLAG_N_BIT
			and	byte  bl, (NOT8 - FLAG_N - FLAG_Z)
			or	byte  cl, dl
			mov	dword eax, .cycle2
			or	byte  bl, cl
			mov	dword [EXECUTE_CYCLE], eax
			mov	byte  [FLAG_REGISTER], bl
			ret

.cycle2:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_plp ***************************************************************
;****************************************************************************
section .text
opcode_plp:
.cycle1:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			or	byte  al, (FLAG_R | FLAG_B)
			mov	byte  [FLAG_REGISTER], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret




;****************************************************************************
;* opcode_rti ***************************************************************
;****************************************************************************
section .text
opcode_rti:
.cycle1:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			or	byte  al, (FLAG_R | FLAG_B)
			mov	byte  [FLAG_REGISTER], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			mov	byte  [PROGRAM_COUNTER + 1], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle4
			ret

.cycle4:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle5
			ret

.cycle5:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_rts ***************************************************************
;****************************************************************************
section .text
opcode_rts:
.cycle1:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	dword ebx, [STACK_POINTER]
			inc	byte  bl
			call	dword [READ_STACK]
			mov	byte  [PROGRAM_COUNTER + 1], al
			mov	dword [STACK_POINTER], ebx
			mov	dword [EXECUTE_CYCLE], .cycle3
			ret

.cycle3:		CHECK_FOR_DMA
			mov	dword eax, [PROGRAM_COUNTER]
			mov	dword ebx, .cycle4
			inc	word  ax
			mov	dword [EXECUTE_CYCLE], ebx
			mov	dword [PROGRAM_COUNTER], eax
			ret

.cycle4:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle5
			ret

.cycle5:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], get_opcode
			ret



;****************************************************************************
;* opcode_illegal ***********************************************************
;****************************************************************************
section .text
opcode_illegal:
			or	dword [STATE], CPU65XX_ILLEGAL_OPCODE
			mov	dword eax, [MODUL_ID]
			jmp	core_exception



;****************************************************************************
;* get_opcode ***************************************************************
;****************************************************************************
;* fetch a byte from memory and interpret it as opcode
;****************************************************************************
section .text
get_opcode:
%ifdef CPU65XX_LOG
%ifdef CPU65XX_LOG_EVERY_OPCODE
			call	core_get_c64_cycles
			cmp	dword eax, 02d00000h
			jb	.skip_log
			cmp	dword eax, 03d00000h
			ja	.skip_log
			call	cpu65xx_log
			mov	dword ecx, eax
			mov	dword eax, [MODUL_ID]
			call	log_write
.skip_log:
%endif
%endif
			test	dword [STATE], SIGNAL_IRQ_OR_NMI
			jz	.no_irq_or_nmi

			mov	dword eax, [STATE]
			test	byte  [FLAG_REGISTER], FLAG_I
			jnz	.no_irq
			test	dword eax, SIGNAL_IRQ
			jz	.no_irq
			call	dword [GET_CYCLES]
			sub	dword eax, [IRQ_TIMESTAMP]
			sbb	dword edx, [IRQ_TIMESTAMP + 4]
			cmp	dword eax, 2
			jbe	.no_irq
			or	dword [STATE], PROCESSING_IRQ
			jmp	process_irq

.no_irq:		test	dword eax, SIGNAL_NMI
			jz	.no_irq_or_nmi
			call	dword [GET_CYCLES]
			sub	dword eax, [NMI_TIMESTAMP]
			sbb	dword edx, [NMI_TIMESTAMP + 4]
			cmp	dword eax, 2
			jbe	.no_irq_or_nmi
			or	dword [STATE], PROCESSING_NMI
			jmp	process_nmi

.no_irq_or_nmi:		mov	dword ebx, [PROGRAM_COUNTER]
			call	dword [READ_OPCODE]
			xor	dword ecx, ecx
			mov	byte  [OPCODE], al
			inc	word  bx
			mov	byte  cl, al
			mov	dword [PROGRAM_COUNTER], ebx
			mov	dword eax, [ap__opcodes + 8 * ecx]
			mov	dword ebx, [ap__opcodes + 8 * ecx + 4]
			mov	dword [EXECUTE_CYCLE], eax
			mov	dword [OPTIONAL_ADDRESS], ebx
			ret



;****************************************************************************
;* process_irq **************************************************************
;****************************************************************************
process_irq:
.cycle0:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle1
			ret

.cycle1:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	byte  al, [PROGRAM_COUNTER + 1]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle3
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle3:		mov	byte  al, [PROGRAM_COUNTER]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle4
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle4:		mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, [STACK_POINTER]
			and	byte  al, (NOT8 - FLAG_B)
			mov	dword ecx, .cycle5
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle5:		mov	dword ebx, 00000fffeh
			call	dword [READ_OPCODE]
			mov	dword ecx, .cycle6
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle6:		mov	dword ebx, 00000ffffh
			call	dword [READ_OPCODE]
			mov	byte  bl, [FLAG_REGISTER]
			mov	byte  [PROGRAM_COUNTER + 1], al
			or	byte  bl, FLAG_I
			mov	dword eax, [STATE]
			mov	dword [EXECUTE_CYCLE], get_opcode
			and	dword eax, (NOT32 - PROCESSING_IRQ)
			mov	byte  [FLAG_REGISTER], bl
			mov	dword [STATE], eax
			ret



;****************************************************************************
;* process_nmi **************************************************************
;****************************************************************************
process_nmi:
.cycle0:		CHECK_FOR_DMA
			mov	dword eax, [STATE]
			mov	dword ecx, .cycle1
			test	dword eax, SIGNAL_IRQ
			jnz	.cycle0_end
			and	dword eax, (NOT32 - SIGNAL_IRQ_OR_NMI)
.cycle0_end:		and	dword eax, (NOT32 - SIGNAL_NMI)
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STATE], eax
			ret

.cycle1:		CHECK_FOR_DMA
			mov	dword [EXECUTE_CYCLE], .cycle2
			ret

.cycle2:		mov	byte  al, [PROGRAM_COUNTER + 1]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle3
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle3:		mov	byte  al, [PROGRAM_COUNTER]
			mov	dword ebx, [STACK_POINTER]
			mov	dword ecx, .cycle4
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle4:		mov	byte  al, [FLAG_REGISTER]
			mov	dword ebx, [STACK_POINTER]
			and	byte  al, (NOT8 - FLAG_B)
			mov	dword ecx, .cycle5
			call	dword [WRITE_STACK]
			dec	byte  bl
			mov	dword [EXECUTE_CYCLE], ecx
			mov	dword [STACK_POINTER], ebx
			ret

.cycle5:		mov	dword ebx, 00000fffah
			call	dword [READ_OPCODE]
			mov	dword ecx, .cycle6
			mov	byte  [PROGRAM_COUNTER], al
			mov	dword [EXECUTE_CYCLE], ecx
			ret

.cycle6:		mov	dword ebx, 00000fffbh
			call	dword [READ_OPCODE]
			mov	byte  bl, [FLAG_REGISTER]
			mov	byte  [PROGRAM_COUNTER + 1], al
			or	byte  bl, FLAG_I
			mov	dword eax, [STATE]
			mov	dword [EXECUTE_CYCLE], get_opcode
			and	dword eax, (NOT32 - PROCESSING_NMI)
			mov	byte  [FLAG_REGISTER], bl
			mov	dword [STATE], eax
			ret
;********************************************* karsten.scheibler@bigfoot.de *

