;****************************************************************************
;****************************************************************************
;*
;* vc1541link.nasm
;*
;****************************************************************************
;****************************************************************************





%define VC1541LINK_NASM
%include "vc1541link.inc"
%include "core.inc"
%include "link.inc"
%include "status.inc"
%include "print.inc"
%ifdef VC1541LINK_LOG
%include "log.inc"
%endif



%assign COUNTER_TABLE_ENTRIES	VC1541LINK_COUNTER_TABLE_ENTRIES
%assign COUNTER_DEFAULT_VALUE	VC1541LINK_COUNTER_DEFAULT_VALUE

%define MODUL_ID		VC1541LINK_MODUL_ID
%define COUNTER_BASE		VC1541LINK_COUNTER_BASE
%define COUNTER			VC1541LINK_COUNTER
%define COUNTER_TABLE		VC1541LINK_COUNTER_TABLE
%define MOTOR_STATE		VC1541LINK_MOTOR_STATE
%define LED_STATE		VC1541LINK_LED_STATE
%define TRACK			VC1541LINK_TRACK
%define SERIAL_BUS		VC1541LINK_SERIAL_BUS

%assign VIA1_IRQ_NUMBER		VC1541LINK_VIA1_IRQ_NUMBER
%assign VIA2_IRQ_NUMBER		VC1541LINK_VIA2_IRQ_NUMBER



;****************************************************************************
;* vc1541link_initialize ****************************************************
;****************************************************************************
section .text
vc1541link_initialize:
			ret



;****************************************************************************
;* vc1541link_create_instance ***********************************************
;****************************************************************************
section .text
vc1541link_create_instance:
			push	dword eax
			mov	dword eax, [d__vc1541link_global_id]
			mov	dword [MODUL_ID], eax
%ifdef VC1541LINK_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__vc1541link_global_id], eax
			pop	dword eax
			ret

section .data
				align 4
d__vc1541link_global_id:	dd	VC1541LINK_GLOBAL_ID



;****************************************************************************
;* vc1541link_destroy_instance **********************************************
;****************************************************************************
section .text
vc1541link_destroy_instance:
%ifdef VC1541LINK_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			call	log_close
			pop	dword eax
%endif
			ret



%ifdef VC1541LINK_LOG
;****************************************************************************
;* vc1541link_log ***********************************************************
;****************************************************************************
;* <=eax  data length
;* <=ebx  pointer to data
;****************************************************************************
section .text
vc1541link_log:
			mov	dword eax, [COUNTER]
			mov	dword ebx, [COUNTER_BASE]
			mov	dword [.ab__status], eax
			mov	dword [.ab__status + 4], ebx
			xor	dword ebx, ebx
.loop:			mov	dword eax, [COUNTER_TABLE + 4 * ebx]
			mov	dword [.ab__status + 16 + 4 * ebx], eax
			inc	byte  bl
			cmp	byte  bl, (2 * COUNTER_TABLE_ENTRIES)
			jb	.loop
			mov	dword eax, 64
			mov	dword ebx, .ab__status
			ret

section .data
			align 4
.ab__status:		times 64 db 0
%endif



;****************************************************************************
;* vc1541link_reset *********************************************************
;****************************************************************************
section .text
vc1541link_reset:
			mov	dword eax, [MODUL_ID]
			xor	dword ebx, ebx
			mov	byte  [MOTOR_STATE], bl
			call	status_vc1541_motor
			mov	byte  [LED_STATE], bl
			call	status_vc1541_led
			mov	byte  [TRACK], bl
			call	status_vc1541_track
			xor	dword eax, eax
.clear_table:		mov	dword [COUNTER_TABLE + 8 * ebx], eax
			mov	dword [COUNTER_TABLE + 8 * ebx + 4], eax
			inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.clear_table
			mov	dword eax, COUNTER_DEFAULT_VALUE
			mov	dword [COUNTER_BASE], eax
			mov	dword [COUNTER], eax
			ret



;****************************************************************************
;* vc1541link_dump **********************************************************
;****************************************************************************
section .text
vc1541link_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			push	dword eax
			PRINT_TEXT  .cb__error, eax
			pop	dword eax
.end:			ret

.cb__error:		db	"-<VC1541LINK:", STRING_INSERT_HEX32, ">-", 10
			db	"nothing to say here", 10, 0



;****************************************************************************
;* counter_substract ********************************************************
;****************************************************************************
section .text
counter_substract:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, [COUNTER_BASE]
			xor	dword ebx, ebx
			sub	dword eax, [COUNTER]
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			test	dword ecx, ecx
			jz	.result_skip
			sub	dword ecx, eax
			jz	.result_wrong
			jnc	.result_ok
.result_wrong:		mov	dword eax, [MODUL_ID]
			jmp	core_exception
.result_ok:		mov	dword [COUNTER_TABLE + 8 * ebx], ecx
.result_skip:		inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* counter_select ***********************************************************
;****************************************************************************
section .text
counter_select:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, COUNTER_DEFAULT_VALUE
			xor	dword ebx, ebx
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			test	dword ecx, ecx
			jz	.skip
			cmp	dword ecx, eax
			jnb	.skip
			mov	dword eax, ecx
.skip:			inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			mov	dword [COUNTER_BASE], eax
			mov	dword [COUNTER], eax
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via_counter_set ***********************************************
;****************************************************************************
;* eax=>  counter offset
;* ebx=>  handler address
;* ecx=>  counter value
;****************************************************************************
section .text
vc1541link_via_counter_set:
			call	counter_substract
			mov	dword [COUNTER_TABLE + 8 * eax], ecx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			jmp	counter_select



;****************************************************************************
;* vc1541link_via_counter_refresh *******************************************
;****************************************************************************
;* eax=>  counter offset
;* ebx=>  handler address
;* ecx=>  counter value
;****************************************************************************
section .text
vc1541link_via_counter_refresh:
			mov	dword [COUNTER_TABLE + 8 * eax], ecx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			ret



;****************************************************************************
;* vc1541link_via_counter_clear *********************************************
;****************************************************************************
;* eax=>  counter offset
;****************************************************************************
section .text
vc1541link_via_counter_clear:
			push	dword ebx
			xor	dword ebx, ebx
			call	counter_substract
			mov	dword [COUNTER_TABLE + 8 * eax], ebx
			mov	dword [COUNTER_TABLE + 8 * eax + 4], ebx
			pop	dword ebx
			jmp	counter_select



;****************************************************************************
;* vc1541link_via_counter_process **********************************************
;****************************************************************************
section .text
vc1541link_via_counter_process:
%ifdef VC1541LINK_LOG
%ifdef VC1541LINK_LOG_EVERY_UNDERFLOW
			call	vc1541link_log
			mov	dword ecx, eax
			mov	dword eax, [MODUL_ID]
			call	log_write
%endif
%endif
			mov	dword eax, [COUNTER_BASE]
			xor	dword ebx, ebx
.loop:			mov	dword ecx, [COUNTER_TABLE + 8 * ebx]
			mov	dword edx, [COUNTER_TABLE + 8 * ebx + 4]
			sub	dword ecx, eax
			jc	.counter_disabled
			mov	dword [COUNTER_TABLE + 8 * ebx], ecx
			jnz	.counter_disabled
			push	dword eax
			push	dword ebx
			mov	dword eax, [.ap__via_offsets + 4 * ebx]
			push	dword ecx
			lea	dword edi, [esi + eax]
			call	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
.counter_disabled:	inc	byte  bl
			cmp	byte  bl, COUNTER_TABLE_ENTRIES
			jb	.loop
			jmp	counter_select

			align 4
.ap__via_offsets:	dd	via6522__vc1541_1
			dd	via6522__vc1541_1
			dd	via6522__vc1541_2
			dd	via6522__vc1541_2



;****************************************************************************
;* vc1541link_via1_irq ******************************************************
;****************************************************************************
section .text
vc1541link_via1_irq:
			push	dword eax
			test	byte  al, al
			jnz	.set
.clear:			mov	byte  al, VIA1_IRQ_NUMBER
			call	cpu65xx_signal_clear_irq
			pop	dword eax
			ret
.set:			mov	byte  al, VIA1_IRQ_NUMBER
			call	cpu65xx_signal_set_irq
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via1_pb *******************************************************
;****************************************************************************
section .text
vc1541link_via1_pb:
			push	dword eax
			push	dword ebx
			mov	byte  al, [VIA6522_LATCH_PB]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRB]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			xor	byte  bl, bl
			test	byte  al, 010h
			jz	.no_atn
			or	byte  bl, 1
.no_atn:		test	byte  al, 008h
			jz	.no_clk
			or	byte  bl, 2
.no_clk:		test	byte  al, 002h
			jz	.no_data
			or	byte  bl, 4
.no_data:		cmp	byte  [SERIAL_BUS], bl
			je	.skip_serial_bus
			mov	byte  [SERIAL_BUS], bl
			call	link_serial_bus
.skip_serial_bus:	pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via1_pa *******************************************************
;****************************************************************************
section .text
vc1541link_via1_pa:
			ret



;****************************************************************************
;* vc1541link_via1_cb2 ******************************************************
;****************************************************************************
section .text
vc1541link_via1_cb2:
			ret



;****************************************************************************
;* vc1541link_via1_ca2 ******************************************************
;****************************************************************************
section .text
vc1541link_via1_ca2:
			ret



;****************************************************************************
;* vc1541link_via2_irq ******************************************************
;****************************************************************************
section .text
vc1541link_via2_irq:
			push	dword eax
			test	byte  al, al
			jnz	.set
.clear:			mov	byte  al, VIA2_IRQ_NUMBER
			call	cpu65xx_signal_clear_irq
			pop	dword eax
			ret
.set:			mov	byte  al, VIA2_IRQ_NUMBER
			call	cpu65xx_signal_set_irq
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via2_pb *******************************************************
;****************************************************************************
section .text
vc1541link_via2_pb:
			push	dword eax
			mov	byte  al, [VIA6522_REGISTERS + VIA6522_PB]
			mov	byte  ah, al
			and	byte  al, 003h
			shr	byte  ah, 5
			call	disc_signal_head
			and	byte  ah, 003h
			mov	byte  al, ah
			call	disc_signal_speed
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via2_pa *******************************************************
;****************************************************************************
section .text
vc1541link_via2_pa:
			push	dword eax
			push	dword ebx
			mov	byte  al, [VIA6522_LATCH_PA]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRA]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			call	disc_signal_data
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_via2_cb2 ******************************************************
;****************************************************************************
section .text
vc1541link_via2_cb2:
			call	disc_signal_rw
			ret



;****************************************************************************
;* vc1541link_via2_ca2 ******************************************************
;****************************************************************************
section .text
vc1541link_via2_ca2:
			call	disc_signal_byte_ready
			ret



;****************************************************************************
;* vc1541link_disc_sync *****************************************************
;****************************************************************************
section .text
vc1541link_disc_sync:
			push	dword eax
			push	dword edi
			mov	dword edi, (vc1541__1 + via6522__vc1541_2)
			mov	byte  ah, [VIA6522_SIGNAL_PB]
			and	byte  ah, 07fh
			shl	byte  al, 7
			or	byte  al, ah
			or	byte  al, 010h
			call	via6522_signal_pb
			pop	dword edi
			pop	dword eax
			ret



;****************************************************************************
;* vc1541link_disc_data *****************************************************
;****************************************************************************
section .text
vc1541link_disc_data:
			push	dword edi
			mov	dword edi, (vc1541__1 + via6522__vc1541_2)
			call	via6522_signal_pa
			pop	dword edi
			ret



;****************************************************************************
;* vc1541link_status ********************************************************
;****************************************************************************
section .text
vc1541link_status:
			push	dword eax
			push	dword ebx
			push	dword edi
			lea	dword edi, [esi + via6522__vc1541_2]
			mov	byte  al, [VIA6522_LATCH_PB]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRB]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			shr	byte  al, 2
			and	byte  al, 1
			cmp	byte  [MOTOR_STATE], al
			je	.skip_motor
			mov	byte  [MOTOR_STATE], al
			mov	byte  bl, al
			mov	dword eax, [MODUL_ID]
			call	status_vc1541_motor
.skip_motor:		mov	byte  al, [VIA6522_LATCH_PB]
			mov	byte  bl, [VIA6522_REGISTERS + VIA6522_DDRB]
			and	byte  al, bl
			xor	byte  bl, NOT8
			or	byte  al, bl
			shr	byte  al, 3
			and	byte  al, 1
			cmp	byte  [LED_STATE], al
			je	.skip_led
			mov	byte  [LED_STATE], al
			mov	byte  bl, al
			mov	dword eax, [MODUL_ID]
			call	status_vc1541_led
.skip_led:		mov	byte  al, [DISC_TRACK]
			cmp	byte  [TRACK], al
			je	.skip_track
			mov	byte  [TRACK], al
			mov	byte  bl, al
			mov	dword eax, [MODUL_ID]
			call	status_vc1541_track
.skip_track:		pop	dword edi
			pop	dword ebx
			pop	dword eax
			ret
;********************************************* karsten.scheibler@bigfoot.de *

