;****************************************************************************
;****************************************************************************
;*
;* vc1541memory.nasm
;*
;****************************************************************************
;****************************************************************************




%define VC1541MEMORY_NASM
%include "vc1541memory.inc"
%include "vc1541link.inc"
%include "core.inc"
%include "print.inc"
%ifdef VC1541MEMORY_LOG
%include "log.inc"
%endif





;****************************************************************************
;****************************************************************************
;*
;* PART 1: assign's & define's
;*
;****************************************************************************
;****************************************************************************





%define MODUL_ID		VC1541MEMORY_MODUL_ID
%define STATE			VC1541MEMORY_STATE
%define RAM			VC1541MEMORY_RAM
%define KERNEL_ROM		VC1541MEMORY_KERNEL_ROM



%assign STACK_LIMIT_EXCEEDED	VC1541MEMORY_STACK_LIMIT_EXCEEDED
%assign WORD_LIMIT_EXCEEDED	VC1541MEMORY_WORD_LIMIT_EXCEEDED




;****************************************************************************
;****************************************************************************
;*
;* PART 2: ROMs
;*
;****************************************************************************
;****************************************************************************





section .text
;ab__vc1541_kernel_rom:	incbin "../roms/vc1541-2_kernel.rom"
ab__vc1541_kernel_rom:	incbin "../roms/vc1541_kernel.rom"





;****************************************************************************
;****************************************************************************
;*
;* PART 3: Memory Access Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* vc1541memory_read ********************************************************
;****************************************************************************
section .text
vc1541memory_read:
			and	dword ebx, 0ffffh
			cmp	dword ebx, 0c000h
			jae	.read_kernel_rom
			cmp	dword ebx, 00800h
			jb	.read_ram_1
			cmp	dword ebx, 01000h
			jb	.read_ram_2
			cmp	dword ebx, 02000h
			jae	.read_rubbish
			cmp	dword ebx, 01c00h
			jae	.read_via2
			cmp	dword ebx, 01800h
			jae	.read_via1
.read_rubbish:		mov	byte  al, bh
			ret
.read_ram_1:		lea	dword ebp, [RAM]
			mov	byte  al, [ebp + ebx]
			ret
.read_ram_2:		lea	dword ebp, [RAM - 00800h]
			mov	byte  al, [ebp + ebx]
			ret
.read_via1:		lea	dword edi, [esi + via6522__vc1541_1]
			call	via6522_read
			ret
.read_via2:		lea	dword edi, [esi + via6522__vc1541_2]
			call	via6522_read
			ret
.read_kernel_rom:	mov	byte  al, [KERNEL_ROM - 0c000h + ebx]
			ret



;****************************************************************************
;* vc1541memory_write *******************************************************
;****************************************************************************
section .text
vc1541memory_write:
			and	dword ebx, 0ffffh
			cmp	dword ebx, 00800h
			jb	.write_ram_1
			cmp	dword ebx, 01000h
			jb	.write_ram_2
			cmp	dword ebx, 02000h
			jae	.write_ignore
			cmp	dword ebx, 01c00h
			jae	.write_via2
			cmp	dword ebx, 01800h
			jae	.write_via1
.write_ignore:		ret
.write_ram_1:		lea	dword ebp, [RAM]
			mov	byte  [ebp + ebx], al
			ret
.write_ram_2:		lea	dword ebp, [RAM - 00800h]
			mov	byte  [ebp + ebx], al
			ret
.write_via1:		lea	dword edi, [esi + via6522__vc1541_1]
			call	via6522_write
			ret
.write_via2:		lea	dword edi, [esi + via6522__vc1541_2]
			call	via6522_write
			ret



;****************************************************************************
;* read *********************************************************************
;****************************************************************************
section .text
read_opcode_ram:		
			cmp	dword ebx, 00800h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, [RAM + ebx]
			ret

read_opcode_mirrored_ram:
			cmp	dword ebx, 00800h
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 01000h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, [RAM + ebx - 00800h]
			ret

read_opcode_io:
			cmp	dword ebx, 01000h
			jae	.read
			jmp	read_opcode_switch
.read:			cmp	dword ebx, 01800h
			jae	.read_vias
			mov	byte  al, bh
			ret
.read_vias:		lea	dword edi, [esi + via6522__vc1541_1]
			cmp	dword ebx, 01c00h
			jb	near  via6522_read
			lea	dword edi, [esi + via6522__vc1541_2]
			cmp	dword ebx, 02000h
			jb	near  via6522_read
			jmp	read_opcode_switch

read_opcode_rubbish:
			cmp	dword ebx, 02000h
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 0c000h
			jb	.read
			jmp	read_opcode_switch
.read:			mov	byte  al, bh
			ret

read_opcode_kernel_rom:
			cmp	dword ebx, 0c000h
%ifdef VC1541MEMORY_CHECK
%ifdef VC1541MEMORY_CHECK_KERNEL
			jae	.check_high_limit
			jmp	read_opcode_switch
.check_high_limit:	cmp	dword ebx, 010000h
			jb	.read
			mov	dword eax, [MODUL_ID]
			jmp	core_exception
%else
			jae	.read
			jmp	read_opcode_switch
%endif
%else
			jae	.read
			jmp	read_opcode_switch
%endif
.read:			mov	byte  al, [KERNEL_ROM - 0c000h + ebx]
			ret

read_opcode_switch:
			cmp	dword ebx, 010000h
			jb	.address_ok
			mov	dword ebx, WORD_LIMIT_EXCEEDED
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			call	core_exception
.address_ok:		mov	dword eax, read_opcode_kernel_rom
			cmp	dword ebx, 0c000h
			jae	.write_address
			mov	dword eax, read_opcode_ram
			cmp	dword ebx, 00800h
			jb	.write_address
			mov	dword eax, read_opcode_mirrored_ram
			cmp	dword ebx, 01000h
			jb	.write_address
			mov	dword eax, read_opcode_io
			cmp	dword ebx, 02000h
			jb	.write_address
			mov	dword eax, read_opcode_rubbish
.write_address:		CPU65XX_SET_READ_OPCODE  eax
%ifdef VC1541MEMORY_LOG
%ifdef VC1541MEMORY_LOG_SWITCHES
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, [MODUL_ID]
			mov	dword ebx, .b__dummy
			mov	dword ecx, 1
			call	log_write
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
%endif
%endif
			jmp	dword eax

.b__dummy:		db	05ah

;****************************************************************************
;* read_stack ***************************************************************
;****************************************************************************
section .text
read_stack:
%ifdef VC1541MEMORY_CHECK
%ifdef VC1541MEMORY_CHECK_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.read
			jmp	.limit_exceeded
.read:
%endif
%endif
			mov	byte  al, [RAM + ebx]
			ret



;****************************************************************************
;* write_stack **************************************************************
;****************************************************************************
section .text
write_stack:
%ifdef VC1541MEMORY_CHECK
%ifdef VC1541MEMORY_CHECK_STACK
			cmp	dword ebx, 00100h
			jae	.check_high_limit
.limit_exceeded:	mov	dword ebx, STACK_LIMIT_EXCEEDED
			mov	dword eax, [MODUL_ID]
			mov	dword [STATE], ebx
			jmp	core_exception
.check_high_limit:	cmp	dword ebx, 00200h
			jb	.write
			jmp	.limit_exceeded
.write:
%endif
%endif
			mov	byte  [RAM + ebx], al
			ret





;****************************************************************************
;****************************************************************************
;*
;* PART 4: Initialisation & Management Routines
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* vc1541memory_initialize **************************************************
;****************************************************************************
section .text
vc1541memory_initialize:
			ret



;****************************************************************************
;* vc1541memory_create_instance *********************************************
;****************************************************************************
section .text
vc1541memory_create_instance:
			push	dword eax
			mov	dword eax, [d__vc1541memory_global_id]
			mov	dword [VC1541MEMORY_MODUL_ID], eax
%ifdef VC1541MEMORY_LOG
			call	log_open
%endif
			inc	dword eax
			mov	dword [d__vc1541memory_global_id], eax
			pop	dword eax
			ret

section .data
				align 4
d__vc1541memory_global_id:	dd	VC1541MEMORY_GLOBAL_ID



;****************************************************************************
;* vc1541memory_destroy_instance ********************************************
;****************************************************************************
section .text
vc1541memory_destroy_instance:
%ifdef VC1541MEMORY_LOG
			push	dword eax
			mov	dword eax, [MODUL_ID]
			call	log_close
			pop	dword eax
%endif
			ret



;****************************************************************************
;* vc1541memory_reset *******************************************************
;****************************************************************************
section .text
vc1541memory_reset:
			push	dword eax
			push	dword ecx
			push	dword edi
			xor	dword eax, eax
			mov	dword ecx, 00200h
			lea	dword edi, [RAM]
			cld
			rep stosd
			pop	dword edi
			pop	dword ecx
			pop	dword eax
			ret

vc1541memory_read_opcode:	equ	read_opcode_ram
vc1541memory_read_stack:	equ	read_stack
vc1541memory_read_data:		equ	vc1541memory_read
vc1541memory_write_stack:	equ	write_stack
vc1541memory_write_data:	equ	vc1541memory_write



;****************************************************************************
;* vc1541memory_dump ********************************************************
;****************************************************************************
section .text
vc1541memory_dump:
			cmp	dword eax, [MODUL_ID]
			jne	.end
			push	dword eax
			PRINT_TEXT  .cb__error, eax, [STATE]
			pop	dword eax
.end:			ret

.cb__error:		db	"-<VC1541MEMORY:", STRING_INSERT_HEX32, ">-", 10
			db	"State: ", STRING_INSERT_HEX32, 10, 0
;********************************************* karsten.scheibler@bigfoot.de *

