;****************************************************************************
;****************************************************************************
;*
;* fb.nasm
;*
;****************************************************************************
;****************************************************************************





%define FB_NASM
%include "fb.inc"
%include "syscall.inc"
%include "print.inc"
%include "string.inc"
%include "c64memory.inc"



;****************************************************************************
;* Data *********************************************************************
;****************************************************************************
section .data
p__fb_device:			dd	0
p__fb_address			dd	0
p__fb_access_mode_text:		dd	0
d__fb_x_padding:		dd	0
d__fb_handle:			dd	0
b__fb_access_mode:		db	0
flag__fb_cmap_saved:		db	0
flag__fb_vga_registers_saved:	db	0

section .bss
				alignb 4
fbfunc__fb_functions:		resb	fbfunc_size
ad__fb_saved_screen_info:	resb	screen_size
ad__fb_screen_info:		resb	screen_size
ab__fb_vga_registers:		
.sequencer:			resb	FB_VGA_SEQUENCER_REGISTERS
.graphics:			resb	FB_VGA_GRAPHICS_REGISTERS




;****************************************************************************
;* fb_set_device ************************************************************
;****************************************************************************
section .text
fb_set_device:
			mov	dword [p__fb_device], eax
			ret



;****************************************************************************
;* fb_print_info ************************************************************
;****************************************************************************
section .text
fb_print_info:
			PRINT_TEXT  .cb__parameters, [p__fb_device], [ad__fb_screen_info + d__screen_x_resolution], [ad__fb_screen_info + d__screen_y_resolution], [ad__fb_screen_info + d__screen_bits_per_pixel], [p__fb_access_mode_text]
			ret

.cb__parameters:	db	10, "Video:     Device:         ", STRING_INSERT_STR, 10
			db	"           Resolution:     ", STRING_INSERT_DEC32, "x", STRING_INSERT_DEC32, 10
			db	"           Bits per Pixel: ", STRING_INSERT_DEC32, " ", STRING_INSERT_STR, 10, 0



;****************************************************************************
;* fb_open ******************************************************************
;****************************************************************************
section .text
fb_open:
			;----
			;open
			;----

			SYSE_OPEN  [p__fb_device], O_RDWR
			mov	dword [d__fb_handle], eax
			mov	dword [.mmap__data + d__mmap_handle], eax

			;--------------------
			;ioctl get screeninfo
			;--------------------

			SYSE_IOCTL  eax, FBIOGET_VSCREENINFO, ad__fb_saved_screen_info

			;---------------------------------------------
			;copy ad__fb_saved_screen_info to ad__fb_screen_info
			;---------------------------------------------

			push	dword esi
			push	dword edi
			mov	dword ecx, (screen_size / 4)
			mov	dword esi, ad__fb_saved_screen_info
			mov	dword edi, ad__fb_screen_info
			cld
			rep movsd
			pop	dword edi
			pop	dword esi

			;---------------------------------
			;if actual color depth is > 8 bpp,
			;try to change to to 8bpp
			;---------------------------------

			xor	dword ebp, ebp
			xor	dword edx, edx
			mov	dword eax, [ad__fb_screen_info + d__screen_bits_per_pixel]
			mov	byte  dl, 8
			mov	dword ebx, [ad__fb_screen_info + d__screen_x_resolution]
			mov	dword ecx, [ad__fb_screen_info + d__screen_y_resolution]
			cmp	dword eax, edx
			jbe	.color_depth_ok
			inc	dword ebp
			mov	dword eax, edx

.color_depth_ok:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;TODO: Change Resolution if needed
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

.resolution_ok:		test	dword ebp, ebp
			jz	.no_change
			mov	dword [ad__fb_screen_info + d__screen_bits_per_pixel], eax
			SYS_IOCTL  [d__fb_handle], FBIOPUT_VSCREENINFO, ad__fb_screen_info
			SYSE_IOCTL  [d__fb_handle], FBIOGET_VSCREENINFO, ad__fb_screen_info

			;-------------------------------------------------
			;do some calculations and check if the color depth
			;is ok, because the bpp change above may have
			;failed
			;-------------------------------------------------

.no_change:		mov	dword eax, [ad__fb_screen_info + d__screen_x_resolution]
			mov	dword ebx, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			mov	dword ecx, [ad__fb_screen_info + d__screen_bits_per_pixel]
			sub	dword ebx, eax
			mov	dword [d__fb_x_padding], ebx
%ifdef FB_ENABLE_4BPP
			cmp	dword ecx, 4
			je	.4bpp
%endif
%ifdef FB_ENABLE_8BPP
			cmp	dword ecx, 8
			je	.8bpp
%endif
			ERROR_EXIT  .cb__color_depth

%ifdef FB_ENABLE_4BPP
			;----------------------------
			;4bpp (vga16fb) with ioperm()
			;----------------------------

.4bpp:			SYS_IOPERM  FB_VGA_IO_BASE, FB_VGA_IO_SIZE, 1
			test	dword eax, eax
			jns	.4bpp_mmap
			ERROR_EXIT  .cb__4bpp_ioperm
.4bpp_mmap:		mov	dword [.mmap__data + d__mmap_length], 010000h
			SYSE_MMAP  .mmap__data
			mov	dword [p__fb_address], eax
			mov	dword eax, FB_4BPP_MODE
			jmp	.set_access_mode

%endif

%ifdef FB_ENABLE_8BPP
			;----
			;8bpp
			;----

.8bpp:			mov	dword eax, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			mul	dword [ad__fb_screen_info + d__screen_y_resolution]
			jnc	.8bpp_mmap
			ERROR_EXIT  .cb__error_message
.8bpp_mmap:		mov	dword [.mmap__data + d__mmap_length], eax
			SYSE_MMAP  .mmap__data
			mov	dword [p__fb_address], eax
			mov	dword eax, FB_8BPP_MODE
%endif
.set_access_mode:	call	fb_set_access_mode
			ret

.cb__error_message:	db	"Error while opening framebuffer device", 0
.cb__color_depth:	db	"Color depth not supported", 0
%ifdef FB_ENABLE_4BPP
.cb__4bpp_ioperm:	db	"4 bpp needs ioperm(), either you start this program as root or use setuid", 0
%endif

section .data
			align 4
.mmap__data:		istruc mmap
			at d__mmap_start,	dd 0
			at d__mmap_protection,	dd (PROT_READ | PROT_WRITE)
			at d__mmap_flags,	dd MAP_SHARED
			at d__mmap_offset,	dd 0
			iend



;****************************************************************************
;* fb_fade_out **************************************************************
;****************************************************************************
section .text
fb_fade_out:
			SYSP_IOCTL  [d__fb_handle], FBIOGETCMAP, ad__fb_saved_cmap
			inc	byte  [flag__fb_cmap_saved]
			ret

			align 4
ad__fb_saved_cmap:
			dd	0			;start
			dd	16			;length
			dd	aw__fb_saved_red	;pointer to color components
			dd	aw__fb_saved_green
			dd	aw__fb_saved_blue
			dd	aw__fb_saved_transparent

section .bss
				alignb 4
aw__fb_saved_red:		resw	16
aw__fb_saved_green:		resw	16
aw__fb_saved_blue:		resw	16
aw__fb_saved_transparent:	resw	16



;****************************************************************************
;* fb_fade_in ***************************************************************
;****************************************************************************
section .text
fb_fade_in:
			test	byte  [flag__fb_cmap_saved], NOT8
			jz	.end
			SYSP_IOCTL  [d__fb_handle], FBIOPUTCMAP, ad__fb_saved_cmap
.end:			ret



;****************************************************************************
;* fb_initialize ************************************************************
;****************************************************************************
section .text
fb_initialize:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			call	dword [fbfunc__fb_functions + FBFUNC_INITIALIZE]
			call	fb_fade_out
			SYSE_IOCTL  [d__fb_handle], FBIOPUTCMAP, .ad__cmap_color_table
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.cb__error_message:	db	"Error while setting C64 color table", 0

.aw__red:		dw	00000h, 0e000h, 06000h, 00000h
			dw	06000h, 00000h, 00000h, 0e000h
			dw	06000h, 04000h, 0e000h, 02000h
			dw	04000h, 06000h, 06000h, 06000h
.aw__green:		dw	00000h, 0e000h, 02000h, 0e000h
			dw	02000h, 06000h, 00000h, 0e000h
			dw	04000h, 02000h, 06000h, 02000h
			dw	04000h, 0e000h, 06000h, 06000h
.aw__blue:		dw	00000h, 0e000h, 02000h, 0e000h
			dw	06000h, 04000h, 06000h, 06000h
			dw	02000h, 00000h, 06000h, 02000h
			dw	04000h, 06000h, 0e000h, 06000h
.aw__transparent:	times 16 dw 00000h

.ad__cmap_color_table:
			dd	0		;start
			dd	16		;length
			dd	.aw__red	;pointer to color components
			dd	.aw__green
			dd	.aw__blue
			dd	.aw__transparent



;****************************************************************************
;* fb_restore ***************************************************************
;****************************************************************************
section .text
fb_restore:
			test	byte  [flag__fb_vga_registers_saved], NOT8
			jz	.end
			push	dword eax
			push	dword ebx
			push	dword edx

			;-------------------------------
			;restore the saved VGA Registers
			;-------------------------------

			xor	dword ebx, ebx
			mov	dword edx, FB_VGA_SEQUENCER_BASE
.loop_1:		mov	byte  al, bl
			mov	byte  ah, [ab__fb_vga_registers.sequencer + ebx]
			out	dx, ax
			inc	byte  bl
			cmp	byte  bl, FB_VGA_SEQUENCER_REGISTERS
			jb	.loop_1
			mov	dword edx, FB_VGA_GRAPHICS_BASE
.loop_2:		mov	byte  al, bl
			mov	byte  ah, [ab__fb_vga_registers.graphics + ebx]
			out	dx, ax
			inc	byte  bl
			cmp	byte  bl, FB_VGA_GRAPHICS_REGISTERS
			jb	.loop_2
			pop	dword edx
			pop	dword ebx
			pop	dword eax
.end:			ret



;****************************************************************************
;* fb_clear *****************************************************************
;****************************************************************************
section .text
fb_clear:
			jmp	dword [fbfunc__fb_functions + FBFUNC_CLEAR]



;****************************************************************************
;* fb_draw_line *************************************************************
;****************************************************************************
section .text
fb_draw_line:
			jmp	dword [fbfunc__fb_functions + FBFUNC_DRAW_LINE]



;****************************************************************************
;* fb_draw_led **************************************************************
;****************************************************************************
;* al==>  color
;* ebx=>  position x
;* ecx=>  position y
;****************************************************************************
section .text
fb_draw_led:
			jmp	dword [fbfunc__fb_functions + FBFUNC_DRAW_LED]



;****************************************************************************
;* fb_draw_text *************************************************************
;****************************************************************************
;* eax=>  pointer to text
;* ebx=>  position x
;* ecx=>  position y
;* edx=>  minimal length
;****************************************************************************
section .text
fb_draw_text:
			push	dword ebp
			mov	dword ebp, esp
			sub	dword esp, FB_DRAW_MAX_STRING_LENGTH
			push	dword ebx
			push	dword ecx
			mov	dword ebx, esp
			mov	dword ecx, FB_DRAW_MAX_STRING_LENGTH
			add	dword ebx, 8
			call	string_parse
			mov	dword eax, ebx
			pop	dword ecx
			pop	dword ebx
			call	dword [fbfunc__fb_functions + FBFUNC_DRAW]
			mov	dword esp, ebp
			pop	dword ebp
			ret



;****************************************************************************
;* fb_draw_string ***********************************************************
;****************************************************************************
;* eax=>  pointer to string
;* ebx=>  position x
;* ecx=>  position y
;* edx=>  minimal length
;****************************************************************************
section .text
fb_draw_string:
			sub	dword esp, FB_DRAW_MAX_STRING_LENGTH
			push	dword ebx
			push	dword ecx
			mov	dword ebx, esp
			mov	dword ecx, FB_DRAW_MAX_STRING_LENGTH
			add	dword ebx, 8
			call	string_parse
			mov	dword eax, ebx
			pop	dword ecx
			pop	dword ebx
			call	dword [fbfunc__fb_functions + FBFUNC_DRAW]
			add	dword esp, FB_DRAW_MAX_STRING_LENGTH
			ret



;****************************************************************************
;* fb_close *****************************************************************
;****************************************************************************
section .text
fb_close:
			push	dword eax
			push	dword ebx
			mov	dword eax, [d__fb_handle]
			test	dword eax, eax
			jz	.no_close
			push	dword eax
			call	fb_restore
			call	fb_fade_in
			SYS_IOCTL  eax, FBIOPUT_VSCREENINFO, ad__fb_saved_screen_info
			pop	dword ebx
			SYS_CLOSE
.no_close:		pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* fb_get_parameter *********************************************************
;****************************************************************************
;* <=eax  x resolution
;* <=ebx  x padding
;* <=ecx  y resolution
;* <=edx  pointer to framebuffer
;* <=ebp  access mode
;****************************************************************************
section .text
fb_get_parameter:
			xor	dword edx, edx
			mov	dword eax, [ad__fb_screen_info + d__screen_x_resolution]
			mov	byte  dl, [b__fb_access_mode]
			mov	dword ebx, [d__fb_x_padding]
			mov	dword ebp, edx
			mov	dword ecx, [ad__fb_screen_info + d__screen_y_resolution]
			mov	dword edx, [p__fb_address]
			ret



;****************************************************************************
;* fb_set_access_mode *******************************************************
;****************************************************************************
;* eax=> access mode
;****************************************************************************
section .text
fb_set_access_mode:
			mov	byte  [b__fb_access_mode], al
			cmp	byte  al, FB_8BPP_MODE
			je	.8bpp
.4bpp:			mov	dword [p__fb_access_mode_text], .cb__access_ioperm
			mov	dword [fbfunc__fb_functions + FBFUNC_INITIALIZE], initialize_4bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_CLEAR], clear_4bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW_LINE], draw_line_4bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW_LED], draw_led_4bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW], draw_4bpp
			ret
.8bpp:			mov	dword [p__fb_access_mode_text], .cb__access_linear
			mov	dword [fbfunc__fb_functions + FBFUNC_INITIALIZE], initialize_8bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_CLEAR], clear_8bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW_LINE], draw_line_8bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW_LED], draw_led_8bpp
			mov	dword [fbfunc__fb_functions + FBFUNC_DRAW], draw_8bpp
			ret

.cb__access_linear:	db	"[using linear access]", 0
.cb__access_ioperm:	db	"[using ioperm() and mmap() ]", 0



%ifdef FB_ENABLE_8BPP
;****************************************************************************
;* initialize_8bpp **********************************************************
;****************************************************************************
section .text
initialize_8bpp:
			ret



;****************************************************************************
;* clear_8bpp ***************************************************************
;****************************************************************************
section .text
clear_8bpp:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword edi
			xor	dword eax, eax
			mov	dword edi, [p__fb_address]
			cld
			mov	dword ebx, [ad__fb_screen_info + d__screen_x_resolution]
			shr	dword ebx, 2
			mov	dword edx, [ad__fb_screen_info + d__screen_y_resolution]
.clear_line:		mov	dword ecx, ebx
			rep stosd
			add	dword edi, [d__fb_x_padding]
			dec	dword edx
			jnz	.clear_line
			pop	dword edi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_line_8bpp ***********************************************************
;****************************************************************************
section .text
draw_line_8bpp:
			ret



;****************************************************************************
;* draw_led_8bpp ************************************************************
;****************************************************************************
;* al==>  color
;* ebx=>  position x
;* ecx=>  position y
;****************************************************************************
section .text
draw_led_8bpp:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword eax, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			mul	dword ecx
			add	dword ebx, eax
			mov	dword eax, [esp + 8]
			add	dword ebx, [p__fb_address]
			and	byte  al, 00fh
			and	byte  bl, 0fch		;dword align
			mov	byte  ah, al
			shl	dword eax, 8
			mov	byte  al, ah
			mov	dword ecx, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			shl	dword eax, 8
			mov	byte  al, ah
			mov	dword [ebx], eax
			mov	dword [ebx + ecx], eax
			and	dword eax, 000ffffffh
			mov	dword [ebx + 4], eax
			mov	dword [ebx + ecx + 4], eax
.end:			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_8bpp ****************************************************************
;****************************************************************************
;* eax=>  pointer to data
;* ebx=>  position x
;* ecx=>  position y
;* edx=>  minimal length (ignored)
;****************************************************************************
section .text
draw_8bpp:
			push	dword edi
			mov	dword edi, eax
			mov	dword eax, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			mul	dword ecx
			add	dword ebx, eax
			add	dword ebx, [p__fb_address]
			xor	dword ecx, ecx
.loop:			xor	dword edx, edx
			mov	byte  dl, [edi + ecx]
			or	byte  dl, dl
			jz	.end
			cmp	byte  dl, 060h
			jb	.no_correction
			sub	byte  dl, 060h
.no_correction:		shl	dword edx, 3
			push	dword ebx
			push	dword ecx
			mov	byte  ch, 008h
.draw_line:		mov	byte  al, [C64MEMORY_CHARACTER_ROM + 00800h + edx]
			mov	byte  cl, 008h
.draw_pixel:		xor	byte  ah, ah
			shl	byte  al, 1
			jnc	.pixel_clear
			mov	byte  ah, 00fh
.pixel_clear:		mov	byte  [ebx], ah
			inc	dword ebx
			dec	byte  cl
			jnz	.draw_pixel
			sub	dword ebx, 8
			inc	dword edx
			add	dword ebx, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			dec	byte  ch
			jnz	.draw_line
			pop	dword ecx
			pop	dword ebx
			inc	dword ecx
			add	dword ebx, 8
			cmp	dword ecx, FB_DRAW_MAX_STRING_LENGTH
			jb	.loop
.end:			pop	dword edi
			ret



%else
initialize_8bpp:
clear_8bpp:
draw_line_8bpp:
draw_led_8bpp:
draw_8bpp:
			ret
%endif



%ifdef FB_ENABLE_4BPP
;****************************************************************************
;* initialize_4bpp **********************************************************
;****************************************************************************
section .text
initialize_4bpp:
			;----------------------------------------------
			;save all VGA Registers, which could be changed
			;----------------------------------------------

			xor	dword ebx, ebx
			mov	dword edx, FB_VGA_SEQUENCER_BASE
.loop_1:		mov	byte  al, bl
			out	dx, al
			inc	dword edx
			in	al, dx
			dec	dword edx
			mov	byte  [ab__fb_vga_registers.sequencer + ebx], al
			inc	byte  bl
			cmp	byte  bl, FB_VGA_SEQUENCER_REGISTERS
			jb	.loop_1
			mov	dword edx, FB_VGA_GRAPHICS_BASE
.loop_2:		mov	byte  al, bl
			out	dx, al
			inc	dword edx
			in	al, dx
			dec	dword edx
			mov	byte  [ab__fb_vga_registers.graphics + ebx], al
			inc	byte  bl
			cmp	byte  bl, FB_VGA_GRAPHICS_REGISTERS
			jb	.loop_2
			inc	byte  [flag__fb_vga_registers_saved]
			ret



;****************************************************************************
;* clear_4bpp ***************************************************************
;****************************************************************************
section .text
clear_4bpp:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword edi

			;----------------------------------------
			;03c4:2  Memory Plane Write Enable = 00fh
			;----------------------------------------

			mov	dword edx, FB_VGA_SEQUENCER_BASE
			mov	byte  al, 002h
			mov	byte  ah, 00fh
			out	dx, ax

			;--------------------------------------------
			;03ce:3  Logical Operation = None, Rotate = 0
			;--------------------------------------------

			mov	dword edx, FB_VGA_GRAPHICS_BASE
			mov	byte  al, 003h
			xor	byte  ah, ah
			out	dx, ax

			;----------------------
			;03ce:5  Write Mode = 2
			;----------------------

			mov	byte  al, 005h
			mov	byte  ah, [ab__fb_vga_registers.graphics + 5]
			and	byte  ah, 0fch
			or	byte  ah, 002h
			out	dx, ax

			;-----------------------
			;03ce:8  Bit Mask = 0ffh
			;-----------------------

			mov	byte  al, 008h
			mov	byte  ah, 0ffh
			out	dx, ax

			xor	dword eax, eax
			mov	dword edi, [p__fb_address]
			cld
			mov	dword ebx, [ad__fb_screen_info + d__screen_x_resolution]
			shr	dword ebx, 5
			mov	dword edx, [ad__fb_screen_info + d__screen_y_resolution]
.clear_line:		mov	dword ecx, ebx
			rep stosd
			mov	dword ecx, [d__fb_x_padding]
			shr	dword ecx, 3
			add	dword edi, ecx
			dec	dword edx
			jnz	.clear_line
			pop	dword edi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_line_4bpp ***********************************************************
;****************************************************************************
;* eax=>  plane line length
;* ebx=>  pointer to data
;* edi=>  pointer to framebuffer
;****************************************************************************
section .text
draw_line_4bpp:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword esi
			mov	dword ecx, eax

			;-------------------------------
			;03ce:1  Enable Set/Reset = 000h
			;-------------------------------

			mov	dword edx, FB_VGA_GRAPHICS_BASE
			mov	byte  al, 001h
			xor	byte  ah, ah
			out	dx, ax

			;--------------------------------------------
			;03ce:3  Logical Operation = None, Rotate = 0
			;--------------------------------------------

			mov	byte  al, 003h
			xor	byte  ah, ah
			out	dx, ax

			;----------------------
			;03ce:5  Write Mode = 0
			;----------------------

			mov	byte  al, 005h
			mov	byte  ah, [ab__fb_vga_registers.graphics + 5]
			and	byte  ah, 0fch
			out	dx, ax

			;-----------------------
			;03ce:8  Bit Mask = 0ffh
			;-----------------------

			mov	byte  al, 008h
			mov	byte  ah, 0ffh
			out	dx, ax

			;-----------------
			;copy the 4 planes
			;-----------------

			cld
			mov	dword esi, ebx
			mov	dword edx, FB_VGA_SEQUENCER_BASE
			mov	byte  al, 002h	;03c4:2  Memory Plane
			mov	byte  ah, 001h	;Write Enable = 001h
			out	dx, ax
			mov	dword ebx, ecx
			push	dword esi
			push	dword edi
			rep movsd
			pop	dword edi
			pop	dword esi
			mov	dword ecx, ebx
			add	dword esi, 044h
			mov	byte  al, 002h	;03c4:2  Memory Plane
			mov	byte  ah, 002h	;Write Enable = 002h
			out	dx, ax
			push	dword esi
			push	dword edi
			rep movsd
			pop	dword edi
			pop	dword esi
			mov	dword ecx, ebx
			add	dword esi, 044h
			mov	byte  al, 002h	;03c4:2  Memory Plane
			mov	byte  ah, 004h	;Write Enable = 004h
			out	dx, ax
			push	dword esi
			push	dword edi
			rep movsd
			pop	dword edi
			pop	dword esi
			mov	dword ecx, ebx
			add	dword esi, 044h
			mov	byte  al, 002h	;03c4:2  Memory Plane
			mov	byte  ah, 008h	;Write Enable = 008h
			out	dx, ax
			rep movsd

			pop	dword esi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_led_4bpp ************************************************************
;****************************************************************************
;* al==>  color
;* ebx=>  position x
;* ecx=>  position y
;****************************************************************************
section .text
draw_led_4bpp:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx

			;----------------------------------------
			;03c4:2  Memory Plane Write Enable = 00fh
			;----------------------------------------

			mov	dword edx, FB_VGA_SEQUENCER_BASE
			mov	byte  al, 002h
			mov	byte  ah, 00fh
			out	dx, ax

			;-------------------------
			;03ce:0  Set/Reset = color
			;-------------------------

			mov	dword eax, [esp + 12]
			mov	dword edx, FB_VGA_GRAPHICS_BASE
			mov	byte  ah, al
			xor	byte  al, al
			and	byte  ah, 00fh
			out	dx, ax

			;-------------------------------
			;03ce:1  Enable Set/Reset = 00fh
			;-------------------------------

			mov	byte  al, 001h
			mov	byte  ah, 00fh
			out	dx, ax

			;--------------------------------------------
			;03ce:3  Logical Operation = None, Rotate = 0
			;--------------------------------------------

			mov	byte  al, 003h
			xor	byte  ah, ah
			out	dx, ax

			;----------------------
			;03ce:5  Write Mode = 0
			;----------------------

			mov	byte  al, 005h
			mov	byte  ah, [ab__fb_vga_registers.graphics + 5]
			and	byte  ah, 0fch
			out	dx, ax

			;-----------------------
			;03ce:8  Bit Mask = 0feh
			;-----------------------

			mov	byte  al, 008h
			mov	byte  ah, 0feh
			out	dx, ax

			mov	dword eax, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			shr	dword ebx, 3
			shr	dword eax, 3
			mul	dword ecx
			mov	dword ecx, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			add	dword ebx, eax
			shr	dword ecx, 3
			add	dword ebx, [p__fb_address]
			mov	byte  [ebx], al
			mov	byte  [ebx + ecx], al
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret



;****************************************************************************
;* draw_4bpp ****************************************************************
;****************************************************************************
;* eax=>  pointer to data
;* ebx=>  position x
;* ecx=>  position y
;* edx=>  minimal length (ignored)
;****************************************************************************
section .text
draw_4bpp:
			push	dword edi
			mov	dword edi, eax

			;----------------------------------------
			;03c4:2  Memory Plane Write Enable = 00fh
			;----------------------------------------

			mov	dword edx, FB_VGA_SEQUENCER_BASE
			mov	byte  al, 002h
			mov	byte  ah, 00fh
			out	dx, ax

			;-------------------------------
			;03ce:1  Enable Set/Reset = 000h
			;-------------------------------

			mov	dword edx, FB_VGA_GRAPHICS_BASE
			mov	byte  al, 001h
			xor	byte  ah, ah
			out	dx, ax

			;--------------------------------------------
			;03ce:3  Logical Operation = None, Rotate = 0
			;--------------------------------------------

			mov	byte  al, 003h
			xor	byte  ah, ah
			out	dx, ax

			;----------------------
			;03ce:5  Write Mode = 0
			;----------------------

			mov	byte  al, 005h
			mov	byte  ah, [ab__fb_vga_registers.graphics + 5]
			and	byte  ah, 0fch
			out	dx, ax

			;-----------------------
			;03ce:8  Bit Mask = 0ffh
			;-----------------------

			mov	byte  al, 008h
			mov	byte  ah, 0ffh
			out	dx, ax

			mov	dword eax, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			shr	dword ebx, 3
			shr	dword eax, 3
			mul	dword ecx
			add	dword ebx, eax
			add	dword ebx, [p__fb_address]
			xor	dword ecx, ecx
.loop:			xor	dword edx, edx
			mov	byte  dl, [edi + ecx]
			or	byte  dl, dl
			jz	.end
			cmp	byte  dl, 060h
			jb	.no_correction
			sub	byte  dl, 060h
.no_correction:		shl	dword edx, 3
			push	dword ebx
			push	dword ecx
			push	dword edi
			mov	dword edi, [ad__fb_screen_info + d__screen_virtual_x_resolution]
			mov	byte  ch, 008h
			shr	dword edi, 3
.draw_line:		mov	byte  al, [C64MEMORY_CHARACTER_ROM + 00800h + edx]
			mov	byte  [ebx], al
			inc	dword edx
			add	dword ebx, edi
			dec	byte  ch
			jnz	.draw_line
			pop	dword edi
			pop	dword ecx
			pop	dword ebx
			inc	dword ecx
			inc	dword ebx
			cmp	dword ecx, FB_DRAW_MAX_STRING_LENGTH
			jb	.loop
.end:			pop	dword edi
			ret



%else
initialize_4bpp:
clear_4bpp:
draw_line_4bpp:
draw_led_4bpp:
draw_4bpp:
			ret
%endif



%ifdef FB_TEST_VIDEO_MODE
;****************************************************************************
;* fb_test ******************************************************************
;****************************************************************************
section .text
fb_test:
			cmp	byte  [b__fb_access_mode], FB_4BPP_MODE
			je	fb_test_ioperm
			ret



;****************************************************************************
;* fb_test_ioperm ***********************************************************
;****************************************************************************
section .text
fb_test_ioperm:
			mov	dword edx, 003020100h
.fill:			mov	dword eax, edx
			mov	dword ebx, [p__fb_address]
			mov	dword ecx, ((640 * 400) / 32)
.fill_1:		mov	dword [ebx], eax
			add	dword eax, 004040404h
			add	dword ebx, 4
			and	dword eax, 00f0f0f0fh
			dec	dword ecx
			jnz	.fill_1
			add	dword edx, 001010101h
			and	dword edx, 00f0f0f0fh
.end:			jmp	.end



;****************************************************************************
;* fb_test_lseek ************************************************************
;****************************************************************************
section .text
fb_test_lseek:
			SYSE_LSEEK  [d__fb_handle], 0, SEEK_SET
			mov	byte  al, [.ab__data + 1]
			xor	dword ebx, ebx
.fill:			mov	byte  [.ab__data + ebx], al
			inc	byte  al
			and	byte  al, 00fh
			inc	byte  bl
			cmp	byte  bl, 16
			jb	.fill
			mov	dword ecx, 8192
.write:			push	dword ecx
			SYSE_WRITE  [d__fb_handle], .ab__data, 16
			pop	dword ecx
			dec	dword ecx
			jnz	.write
			jmp	fb_test

.cb__error_message:	db	"Error while fb_test", 0

section .data
.ab__data:		db	0, 0, 0, 0, 0, 0, 0, 0
			db	0, 0, 0, 0, 0, 0, 0, 0
%endif
;********************************************* karsten.scheibler@bigfoot.de *

