;****************************************************************************
;****************************************************************************
;*
;* log.nasm
;*
;****************************************************************************
;****************************************************************************





%define LOG_NASM
%include "log.inc"
%include "syscall.inc"
%include "error.inc"
%include "string.inc"
%include "print.inc"



%ifdef LOG_ENABLE
;****************************************************************************
;* some data ****************************************************************
;****************************************************************************
section .data
d__used_handles:	dd	0

section .bss
ad__file_handles:	resd	(2 * LOG_MAX_FILES)



;****************************************************************************
;* log_open *****************************************************************
;****************************************************************************
;* eax=> module number
;****************************************************************************
section .text
log_open:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx

			;------------------------------------------
			;check if this module has an already opened
			;log file
			;------------------------------------------

			mov	dword ecx, [d__used_handles]
			mov	dword ebx, ad__file_handles
			cmp	dword ecx, LOG_MAX_FILES
			jb	.check_handles
			mov	dword ebx, .cb__too_much
			jmp	log_error
.already_open:		mov	dword ebx, .cb__already_open
			jmp	log_error
.check_handles:		test	dword ecx, ecx
			jz	.open_file
.check_handles_1:	cmp	dword eax, [ebx]
			je	.already_open
			add	dword ebx, 8
			dec	dword ecx
			jnz	.check_handles_1
.open_file:		mov	dword [ebx], eax

			;-----------------------------------------
			;create the filename via string_parse in a
			;stackframe and open the file
			;-----------------------------------------

			push	dword ebp
			mov	dword ebp, esp
			mov	dword ecx, LOG_MAX_FILE_LENGTH
			push	dword eax
			sub	dword esp, ecx
			mov	dword eax, .cb__log_file
			mov	dword ebx, esp
			call	string_parse
			SYS_UNLINK
			mov	dword ecx, (O_CREAT | O_WRONLY)
			mov	dword edx, (S_IRUSR | S_IWUSR)
			SYSE_OPEN
			mov	dword ebx, ad__file_handles
			mov	dword ecx, [d__used_handles]
			mov	dword esp, ebp
			mov	dword [byte ebx + 8 * ecx + 4], eax
			pop	dword ebp
			inc	dword [d__used_handles]
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.cb__error_message:	db	"Error while opening logfile", 0
.cb__already_open:	db	"a logfile for this module is already open", 0
.cb__too_much:		db	"too much logfiles open", 0
.cb__log_file:		db	"ec64_", STRING_INSERT_HEX32, ".log", 0



;****************************************************************************
;* log_write ****************************************************************
;****************************************************************************
;* eax=> module number
;* ebx=> pointer to data
;* ecx=> length of data
;****************************************************************************
section .text
log_write:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			mov	dword edx, ecx
			mov	dword ebx, ad__file_handles
			mov	dword ecx, [d__used_handles]
			test	dword ecx, ecx
			jz	.handle_not_found
.search_handle:		cmp	dword eax, [ebx]
			je	.handle_found
			add	dword ebx, 8
			dec	dword ecx
			jnz	.search_handle
.handle_not_found:	mov	dword ebx, .cb__handle_not_found
			jmp	log_error
.handle_found:		mov	dword ebx, [byte  ebx + 4]
			mov	dword ecx, [byte  esp + 8]
			SYSE_WRITE
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.cb__error_message:	db	"Error while writing in logfile", 0
.cb__handle_not_found:	db	"no logfile for that module opened before", 0


;****************************************************************************
;* log_close ****************************************************************
;****************************************************************************
;* eax=> module number
;****************************************************************************
section .text
log_close:
			push	dword eax
			push	dword ebx
			push	dword ecx
			mov	dword ebx, ad__file_handles
			mov	dword ecx, [d__used_handles]
			test	dword ecx, ecx
			jz	.handle_not_found
.search_handle:		cmp	dword eax, [ebx]
			je	.handle_found
			add	dword ebx, 8
			dec	dword ecx
			jnz	.search_handle
.handle_not_found:	mov	dword ebx, .cb__handle_not_found
			jmp	log_error
.handle_found:		xor	dword eax, eax
			dec	dword eax
			mov	dword [ebx], eax
			mov	dword ebx, [byte  ebx + 4]
			SYSE_CLOSE
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.cb__error_message:	db	"Error while closing logfile", 0
.cb__handle_not_found:	db	"no logfile for that module opened before", 0



;****************************************************************************
;* log_error ****************************************************************
;****************************************************************************
;* eax=> module number
;* ebx=> pointer to error message
;****************************************************************************
section .text
log_error:
			PRINT_TEXT  .cb__module_number, eax
			ERROR_EXIT  ebx

.cb__module_number:	db	"ec64: Module: ", STRING_INSERT_HEX32, 10, 0

%endif
;********************************************* karsten.scheibler@bigfoot.de *

