;****************************************************************************
;****************************************************************************
;*
;* sync.nasm
;*
;****************************************************************************
;****************************************************************************





%define SYNC_NASM
%include "sync.inc"
%include "syscall.inc"
%include "tty.inc"
%include "link.inc"



;****************************************************************************
;* MAKE_PROFILING ***********************************************************
;****************************************************************************
%macro STORE_TIME 0
%ifdef SYNC_FRAME_PROFILING
			push	dword ebx
			push	dword ecx
			mov	dword ebx, [d__frame_counter]
			mov	dword ecx, [d__frame_sum]
			sub	dword ecx, [ad__frame_times + 4 * ebx]
			mov	dword [ad__frame_times + 4 * ebx], eax
			inc	dword ebx
			add	dword ecx, eax
			and	dword ebx, (SYNC_FRAMES - 1)
			mov	dword [d__frame_sum], ecx
			mov	dword [d__frame_counter], ebx
			pop	dword ecx
			pop	dword ebx
%endif
%endmacro



;****************************************************************************
;* sync_new_frame ***********************************************************
;****************************************************************************
section .text
sync_new_frame:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
			push	dword edi
			push	dword esi
			call	link_status
			call	tty_get_character
			SYS_GETTIMEOFDAY  .timeval__end, 0
			cmp	byte  [.flag__start_present], 0
			jne	.start_time_present
			inc	byte  [.flag__start_present]
			jmp	.exit
.start_time_present:	mov	dword eax, [.timeval__end + d__timeval_usec]
			mov	dword ebx, 20000		;20 ms
			mov	dword ecx, [.timeval__end + d__timeval_sec]
			sub	dword eax, [.timeval__start + d__timeval_usec]
			jnc	.no_underflow
			add	dword eax, 1000000		;1000 ms
			stc
.no_underflow:		sbb	dword ecx, [.timeval__end + d__timeval_sec]
			jnz	.exit			;a real huge delay
			STORE_TIME
			cmp	dword ebx, eax
			ja	.add_sleep_time
			je	.exit
			sub	dword eax, ebx
			mov	dword ebx, [.d__sleep_time]
			sub	dword ebx, eax
			jnc	.skip_sleep
			xor	dword ebx, ebx
			jmp	.skip_sleep
.add_sleep_time:	sub	dword ebx, eax
			add	dword ebx, [.d__sleep_time]
			cmp	dword ebx, 20000		;20 ms
			jb	.skip_sleep
%ifndef SYNC_DISABLE_SLEEP
			SYSP_NANOSLEEP  .timespec__data, 0
%endif
.skip_sleep:		mov	dword [.d__sleep_time], ebx
.exit:			mov	dword eax, [.timeval__end + d__timeval_sec]
			mov	dword ebx, [.timeval__end + d__timeval_usec]
			mov	dword [.timeval__start + d__timeval_sec], eax
			mov	dword [.timeval__start + d__timeval_usec], ebx
			pop	dword esi
			pop	dword edi
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.timespec__data:	istruc timespec
			at d__timespec_sec, dd 0
			at d__timespec_nsec, dd 20000000	;20000 ms
			iend

section .data
			align 4
.d__sleep_time:		dd	0

.timeval__start:	istruc timeval
			iend

.timeval__end:		istruc timeval
			iend

.flag__start_present:	db	0



%ifdef SYNC_FRAME_PROFILING
;****************************************************************************
;* sync_get_time ************************************************************
;****************************************************************************
section .text
sync_get_time:
			mov	dword eax, [d__frame_sum]
			shr	dword eax, SYNC_FRAME_BITS
			ret

section .data
			align 4
d__frame_counter:	dd	0
d__frame_sum:		dd	0
ad__frame_times:	times (SYNC_FRAMES) dd 0
%endif

;********************************************* karsten.scheibler@bigfoot.de *

