;****************************************************************************
;****************************************************************************
;*
;* syscall.inc
;*
;****************************************************************************
;****************************************************************************





%ifndef SYSCALL_INC
%define SYSCALL_INC

%include "defines.inc"
%include "tty_ioctl.inc"
%include "fb_ioctl.inc"
%include "audio_ioctl.inc"
%include "fcntl.inc"
%include "stat.inc"
%include "mmap.inc"
%include "time.inc"
%include "error.inc"
%include "signal.inc"





;****************************************************************************
;****************************************************************************
;*
;* System Call Numbers
;*
;****************************************************************************
;****************************************************************************





%assign SYSCALL_EXIT		1
%assign SYSCALL_READ		3
%assign SYSCALL_WRITE		4
%assign SYSCALL_OPEN		5
%assign SYSCALL_CLOSE		6
%assign SYSCALL_UNLINK		10
%assign SYSCALL_LSEEK		19
%assign SYSCALL_SIGNAL		48
%assign SYSCALL_IOCTL		54
%assign SYSCALL_FCNTL		55
%assign SYSCALL_GETTIMEOFDAY	78
%assign SYSCALL_MMAP		90
%assign SYSCALL_IOPERM		101
%assign SYSCALL_NANOSLEEP	162





;****************************************************************************
;****************************************************************************
;*
;* Global Macros for all System Calls
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* SYSCALL_SAVE_REGISTERS ***************************************************
;****************************************************************************
%macro SYSCALL_SAVE_REGISTERS 1
%ifdef SYSCALL_PUSH_REGISTERS
			push	dword eax
%if %1 > 0
			push	dword ebx
%endif
%if %1 > 1
			push	dword ecx
%endif
%if %1 > 2
			push	dword edx
%endif
%if %1 > 3
			push	dword esi
%endif
%if %1 > 4
			push	dword edi
%endif
%if %1 > 5
%error Illegal Number of Parameters passed to SYSCALL_SAVE_REGISTERS
%endif
%endif
%endmacro



;****************************************************************************
;* SYSCALL_SET_REGISTER *****************************************************
;****************************************************************************
%macro SYSCALL_SET_REGISTER 2
%ifnum %2
%if %2 = 0
			xor	dword %1, %1
%else
			mov	dword %1, %2
%endif
%else
			mov	dword %1, %2
%endif
%endmacro



;****************************************************************************
;* SYSCALL_SET_REGISTERS ****************************************************
;****************************************************************************
%macro SYSCALL_SET_REGISTERS 1-6
%if %1 > 0
			SYSCALL_SET_REGISTER  ebx, %2
%endif
%if %1 > 1
			SYSCALL_SET_REGISTER  ecx, %3
%endif
%if %1 > 2
			SYSCALL_SET_REGISTER  edx, %4
%endif
%if %1 > 3
			SYSCALL_SET_REGISTER  esi, %5
%endif
%if %1 > 4
			SYSCALL_SET_REGISTER  edi, %6
%endif
%endmacro



;****************************************************************************
;* SYSCALL_EXECUTE_INT ******************************************************
;****************************************************************************
%macro SYSCALL_EXECUTE_INT 1
			mov	dword eax, %1
			int	byte  080h
%ifdef SYSCALL_CHECK_FOR_ERROR
			test	dword eax, eax
			jns	%%no_error
			mov	dword ebx, .cb__error_message
			jmp	error_code_exit
%%no_error:
%endif
%endmacro



;****************************************************************************
;* SYSCALL_RESTORE_REGISTERS ************************************************
;****************************************************************************
%macro SYSCALL_RESTORE_REGISTERS 1
%ifdef SYSCALL_PUSH_REGISTERS
%if %1 > 5
%error Illegal Number of Parameters passed to SYSCALL_RESTORE_REGISTERS
%endif
%if %1 > 4
			pop	dword edi
%endif
%if %1 > 3
			pop	dword esi
%endif
%if %1 > 2
			pop	dword edx
%endif
%if %1 > 1
			pop	dword ecx
%endif
%if %1 > 0
			pop	dword ebx
%endif
			pop	dword eax
%endif
%endmacro



;****************************************************************************
;* SYSCALL_SYS **************************************************************
;****************************************************************************
%macro SYSCALL_SYS 2-7
			SYSCALL_SET_REGISTERS  %2, %3, %4, %5, %6, %7
			SYSCALL_EXECUTE_INT  %1
%endmacro



;****************************************************************************
;* SYSCALL_SYSP *************************************************************
;****************************************************************************
%macro SYSCALL_SYSP 2-7
%define SYSCALL_PUSH_REGISTERS
			SYSCALL_SAVE_REGISTERS  %2
			SYSCALL_SET_REGISTERS  %2, %3, %4, %5, %6, %7
			SYSCALL_EXECUTE_INT  %1
			SYSCALL_RESTORE_REGISTERS  %2
%undef SYSCALL_PUSH_REGISTERS
%endmacro



;****************************************************************************
;* SYSCALL_SYSE *************************************************************
;****************************************************************************
%macro SYSCALL_SYSE 2-7
%define SYSCALL_CHECK_FOR_ERROR
			SYSCALL_SET_REGISTERS  %2, %3, %4, %5, %6, %7
			SYSCALL_EXECUTE_INT  %1
%undef SYSCALL_CHECK_FOR_ERROR
%endmacro



;****************************************************************************
;* SYSCALL_SYSPE ************************************************************
;****************************************************************************
%macro SYSCALL_SYSPE 2-7
%define SYSCALL_PUSH_REGISTERS
%define SYSCALL_CHECK_FOR_ERROR
			SYSCALL_SAVE_REGISTERS  %2
			SYSCALL_SET_REGISTERS  %2, %3, %4, %5, %6, %7
			SYSCALL_EXECUTE_INT  %1
			SYSCALL_RESTORE_REGISTERS  %2
%undef SYSCALL_CHECK_FOR_ERROR
%undef SYSCALL_PUSH_REGISTERS
%endmacro





;****************************************************************************
;****************************************************************************
;*
;* System Call Macros
;*
;****************************************************************************
;****************************************************************************





;****************************************************************************
;* SYS_EXIT *****************************************************************
;****************************************************************************
%macro SYS_EXIT 0-1
			xor	dword eax, eax
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1
%else
			mov	dword ebx, eax
%endif
			inc	dword eax
			int	byte  080h
%endmacro



%ifndef SYSCALL_USE_OLD_MACROS
;****************************************************************************
;* SYS_READ *****************************************************************
;****************************************************************************
%macro SYS_READ 0-3
			SYSCALL_SYS  SYSCALL_READ, %0, %1, %2, %3
%endmacro
%macro SYSP_READ 0-3
			SYSCALL_SYSP  SYSCALL_READ, %0, %1, %2, %3
%endmacro
%macro SYSE_READ 0-3
			SYSCALL_SYSE  SYSCALL_READ, %0, %1, %2, %3
%endmacro
%macro SYSPE_READ 0-3
			SYSCALL_SYSPE  SYSCALL_READ, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_WRITE ****************************************************************
;****************************************************************************
%macro SYS_WRITE 0-3
			SYSCALL_SYS  SYSCALL_WRITE, %0, %1, %2, %3
%endmacro
%macro SYSP_WRITE 0-3
			SYSCALL_SYSP  SYSCALL_WRITE, %0, %1, %2, %3
%endmacro
%macro SYSE_WRITE 0-3
			SYSCALL_SYSE  SYSCALL_WRITE, %0, %1, %2, %3
%endmacro
%macro SYSPE_WRITE 0-3
			SYSCALL_SYSPE  SYSCALL_WRITE, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_OPEN *****************************************************************
;****************************************************************************
%macro SYS_OPEN 0-3
			SYSCALL_SYS  SYSCALL_OPEN, %0, %1, %2, %3
%endmacro
%macro SYSP_OPEN 0-3
			SYSCALL_SYSP  SYSCALL_OPEN, %0, %1, %2, %3
%endmacro
%macro SYSE_OPEN 0-3
			SYSCALL_SYSE  SYSCALL_OPEN, %0, %1, %2, %3
%endmacro
%macro SYSPE_OPEN 0-3
			SYSCALL_SYSPE  SYSCALL_OPEN, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_CLOSE ****************************************************************
;****************************************************************************
%macro SYS_CLOSE 0-1
			SYSCALL_SYS  SYSCALL_CLOSE, %0, %1
%endmacro
%macro SYSP_CLOSE 0-1
			SYSCALL_SYSP  SYSCALL_CLOSE, %0, %1
%endmacro
%macro SYSE_CLOSE 0-1
			SYSCALL_SYSE  SYSCALL_CLOSE, %0, %1
%endmacro
%macro SYSPE_CLOSE 0-1
			SYSCALL_SYSPE  SYSCALL_CLOSE, %0, %1
%endmacro



;****************************************************************************
;* SYS_UNLINK ***************************************************************
;****************************************************************************
%macro SYS_UNLINK 0-1
			SYSCALL_SYS  SYSCALL_UNLINK, %0, %1
%endmacro
%macro SYSP_UNLINK 0-1
			SYSCALL_SYSP  SYSCALL_UNLINK, %0, %1
%endmacro
%macro SYSE_UNLINK 0-1
			SYSCALL_SYSE  SYSCALL_UNLINK, %0, %1
%endmacro
%macro SYSPE_UNLINK 0-1
			SYSCALL_SYSPE  SYSCALL_UNLINK, %0, %1
%endmacro



;****************************************************************************
;* SYS_LSEEK ****************************************************************
;****************************************************************************
%macro SYS_LSEEK 0-3
			SYSCALL_SYS  SYSCALL_LSEEK, %0, %1, %2, %3
%endmacro
%macro SYSP_LSEEK 0-3
			SYSCALL_SYSP  SYSCALL_LSEEK, %0, %1, %2, %3
%endmacro
%macro SYSE_LSEEK 0-3
			SYSCALL_SYSE  SYSCALL_LSEEK, %0, %1, %2, %3
%endmacro
%macro SYSPE_LSEEK 0-3
			SYSCALL_SYSPE  SYSCALL_LSEEK, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_SIGNAL ***************************************************************
;****************************************************************************
%macro SYS_SIGNAL 0-2
			SYSCALL_SYS  SYSCALL_SIGNAL, %0, %1, %2
%endmacro
%macro SYSP_SIGNAL 0-2
			SYSCALL_SYSP  SYSCALL_SIGNAL, %0, %1, %2
%endmacro
%macro SYSE_SIGNAL 0-2
			SYSCALL_SYSE  SYSCALL_SIGNAL, %0, %1, %2
%endmacro
%macro SYSPE_SIGNAL 0-2
			SYSCALL_SYSPE  SYSCALL_SIGNAL, %0, %1, %2
%endmacro



;****************************************************************************
;* SYS_IOCTL ****************************************************************
;****************************************************************************
%macro SYS_IOCTL 0-3
			SYSCALL_SYS  SYSCALL_IOCTL, %0, %1, %2, %3
%endmacro
%macro SYSP_IOCTL 0-3
			SYSCALL_SYSP  SYSCALL_IOCTL, %0, %1, %2, %3
%endmacro
%macro SYSE_IOCTL 0-3
			SYSCALL_SYSE  SYSCALL_IOCTL, %0, %1, %2, %3
%endmacro
%macro SYSPE_IOCTL 0-3
			SYSCALL_SYSPE  SYSCALL_IOCTL, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_FCNTL ****************************************************************
;****************************************************************************
%macro SYS_FCNTL 0-3
			SYSCALL_SYS  SYSCALL_FCNTL, %0, %1, %2, %3
%endmacro
%macro SYSP_FCNTL 0-3
			SYSCALL_SYSP  SYSCALL_FCNTL, %0, %1, %2, %3
%endmacro
%macro SYSE_FCNTL 0-3
			SYSCALL_SYSE  SYSCALL_FCNTL, %0, %1, %2, %3
%endmacro
%macro SYSPE_FCNTL 0-3
			SYSCALL_SYSPE  SYSCALL_FCNTL, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_GETTIMEOFDAY *********************************************************
;****************************************************************************
%macro SYS_GETTIMEOFDAY 0-2
			SYSCALL_SYS  SYSCALL_GETTIMEOFDAY, %0, %1, %2
%endmacro
%macro SYSP_GETTIMEOFDAY 0-2
			SYSCALL_SYSP  SYSCALL_GETTIMEOFDAY, %0, %1, %2
%endmacro
%macro SYSE_GETTIMEOFDAY 0-2
			SYSCALL_SYSE  SYSCALL_GETTIMEOFDAY, %0, %1, %2
%endmacro
%macro SYSPE_GETTIMEOFDAY 0-2
			SYSCALL_SYSPE  SYSCALL_GETTIMEOFDAY, %0, %1, %2
%endmacro



;****************************************************************************
;* SYS_MMAP *****************************************************************
;****************************************************************************
%macro SYS_MMAP 0-1
			SYSCALL_SYS  SYSCALL_MMAP, %0, %1
%endmacro
%macro SYSP_MMAP 0-1
			SYSCALL_SYSP  SYSCALL_MMAP, %0, %1
%endmacro
%macro SYSE_MMAP 0-1
			SYSCALL_SYSE  SYSCALL_MMAP, %0, %1
%endmacro
%macro SYSPE_MMAP 0-1
			SYSCALL_SYSPE  SYSCALL_MMAP, %0, %1
%endmacro



;****************************************************************************
;* SYS_IOPERM ***************************************************************
;****************************************************************************
%macro SYS_IOPERM 0-3
			SYSCALL_SYS  SYSCALL_IOPERM, %0, %1, %2, %3
%endmacro
%macro SYSP_IOPERM 0-3
			SYSCALL_SYSP  SYSCALL_IOPERM, %0, %1, %2, %3
%endmacro
%macro SYSE_IOPERM 0-3
			SYSCALL_SYSE  SYSCALL_IOPERM, %0, %1, %2, %3
%endmacro
%macro SYSPE_IOPERM 0-3
			SYSCALL_SYSPE  SYSCALL_IOPERM, %0, %1, %2, %3
%endmacro



;****************************************************************************
;* SYS_NANOSLEEP ************************************************************
;****************************************************************************
%macro SYS_NANOSLEEP 0-2
			SYSCALL_SYS  SYSCALL_NANOSLEEP, %0, %1, %2
%endmacro
%macro SYSP_NANOSLEEP 0-2
			SYSCALL_SYSP  SYSCALL_NANOSLEEP, %0, %1, %2
%endmacro
%macro SYSE_NANOSLEEP 0-2
			SYSCALL_SYSE  SYSCALL_NANOSLEEP, %0, %1, %2
%endmacro
%macro SYSPE_NANOSLEEP 0-2
			SYSCALL_SYSPE  SYSCALL_NANOSLEEP, %0, %1, %2
%endmacro





;****************************************************************************
;****************************************************************************
;*
;* Old System Call Macros
;*
;****************************************************************************
;****************************************************************************





%else
;****************************************************************************
;* SYS_READ *****************************************************************
;****************************************************************************
%macro SYS_READ 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;address of the data
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;length of the data
%endif
			mov	dword eax, SYSCALL_READ
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_WRITE ****************************************************************
;****************************************************************************
%macro SYS_WRITE 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;address of the data
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;length of the data
%endif
			mov	dword eax, SYSCALL_WRITE
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_OPEN *****************************************************************
;****************************************************************************
%macro SYS_OPEN 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;address of nullterminated string
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;acces mode
%else
			SYSCALL_SET_REGISTER  ecx, O_RDONLY	;default: read only
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;permissions, only while creat necessary
%endif
			mov	dword eax, SYSCALL_OPEN
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_CLOSE ****************************************************************
;****************************************************************************
%macro SYS_CLOSE 0-1
%if %0 > 1
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
			mov	dword eax, SYSCALL_CLOSE
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_LSEEK ****************************************************************
;****************************************************************************
%macro SYS_LSEEK 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;offset
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;origin
%endif
			mov	dword eax, SYSCALL_LSEEK
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_SIGNAL ***************************************************************
;****************************************************************************
%macro SYS_SIGNAL 0-2
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;Signal Number
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;Signal Handler
%endif
			mov	dword eax, SYSCALL_SIGNAL
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_IOCTL ****************************************************************
;****************************************************************************
%macro SYS_IOCTL 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;ioctl number
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;address of the data
%endif
			mov	dword eax, SYSCALL_IOCTL
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_FCNTL ****************************************************************
;****************************************************************************
%macro SYS_FCNTL 0-3
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;file handle
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;fcntl number
%endif
%if %0 > 2
			SYSCALL_SET_REGISTER  edx, %3		;address of the data
%endif
			mov	dword eax, SYSCALL_FCNTL
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_FCNTL ****************************************************************
;****************************************************************************
%macro SYS_GETTIMEOFDAY 0-2
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;address of struct timeval
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;address of struct timezone
%endif
			mov	dword eax, SYSCALL_GETTIMEOFDAY
			int	byte  080h
%endmacro



;****************************************************************************
;* SYS_MMAP *****************************************************************
;****************************************************************************
%macro SYS_MMAP 0-1
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;address of the mmap structure
%endif
			mov	dword eax, SYSCALL_MMAP
			int	byte  080h
%endmacro

;****************************************************************************
;* SYS_NANOSLEEP ************************************************************
;****************************************************************************
%macro SYS_NANOSLEEP 0-2
%if %0 > 0
			SYSCALL_SET_REGISTER  ebx, %1		;address of timespec
%endif
%if %0 > 1
			SYSCALL_SET_REGISTER  ecx, %2		;address of timespec
%endif
			mov	dword eax, SYSCALL_NANOSLEEP
			int	byte  080h
%endmacro

%endif




%endif
;********************************************* karsten.scheibler@bigfoot.de *

