;****************************************************************************
;****************************************************************************
;*
;* system.nasm
;*
;****************************************************************************
;****************************************************************************





%define SYSTEM_NASM
%include "system.inc"
%include "core.inc"
%include "syscall.inc"
%include "print.inc"
%include "cmdline.inc"
%include "tty.inc"
%include "fb.inc"
%include "audio.inc"



global _start



;****************************************************************************
;* system_initialize ********************************************************
;****************************************************************************
section .text
system_initialize:
			call	fb_open
			call	audio_open
			call	tty_initialize
			ret



;****************************************************************************
;* system_restore ***********************************************************
;****************************************************************************
section .text
system_restore:
			call	fb_close
			call	audio_close
			call	tty_restore
			ret



;****************************************************************************
;* system_start *************************************************************
;****************************************************************************
section .text
_start:
system_start:
			;-------------------------------
			;install SIGINT (Ctrl-C) Handler
			;ignore SIGTSTP (Ctrl-Z)
			;install SIGCONT Handler
			;install SIGSEGV Handler
			;-------------------------------

			SYS_SIGNAL  SIGINT, system_end
			SYS_SIGNAL  SIGTSTP, SIG_IGN
			SYS_SIGNAL  SIGCONT, system_end
			SYS_SIGNAL  SIGSEGV, system_emergency

			;-----------------------------
			;process commandline arguments
			;-----------------------------

			CMDLINE_PARSE

			;----------
			;initialize
			;----------

			call	system_initialize

			;-----------------------------
			;press any key to continue ...
			;-----------------------------

			call	system_print_info
			call	fb_print_info
			call	audio_print_info
			PRINT_TEXT  .cb__press_any_key
			call	tty_press_any_key
			call	fb_initialize
%ifdef SYSTEM_TEST_VIDEO_MODE
			call	fb_test
%endif

			;------------------------
			;start the emulation core
			;------------------------

			call	core_start

			;-----------
			;exit nicely
			;-----------

			jmp	system_end

.cb__press_any_key:	db	10, "Press any key to continue ...", 0



;****************************************************************************
;* system_print_info ********************************************************
;****************************************************************************
section .text
system_print_info:
			PRINT_TEXT  .cb__ec64_message
			ret

.cb__ec64_message:	db	"ec64-", EC64_VERSION, ", Author:   Karsten Scheibler", 10
			db	"           Homepage: http://mars.wiwi.uni-halle.de/ec64/", 10
			db	"           eMail:    ec64@mars.wiwi.uni-halle.de", 10, 0



;****************************************************************************
;* system_exit **************************************************************
;****************************************************************************
section .text
system_exit:
			SYS_EXIT



;****************************************************************************
;* system_end ***************************************************************
;****************************************************************************
section .text
system_end:
			call	system_restore
			call	system_print_info
			call	core_print_info
			call	core_exit
			jmp	system_exit



;****************************************************************************
;* system_emergency *********************************************************
;****************************************************************************
section .text
system_emergency:
			call	system_restore
			call	system_print_info
			PRINT_TEXT  .cb__emergency
			jmp	system_exit

.cb__emergency:		db	10, "GRRRR!, THE EMULATOR ITSELF GENERATED A SEGMENTATION VIOLATION, EXITING ...", 10, 0
;********************************************* karsten.scheibler@bigfoot.de *

