;****************************************************************************
;****************************************************************************
;*
;* tty.nasm
;*
;****************************************************************************
;****************************************************************************




%define TTY_NASM
%include "tty.inc"
%include "syscall.inc"
%include "system.inc"
%include "link.inc"



;****************************************************************************
;* tty_initialize ***********************************************************
;****************************************************************************
section .text
tty_initialize:
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;test for allocating an own tty
%ifdef NOT_IN_USE
%include "print.inc"
			SYSE_IOCTL  STDIN, VT_OPENQRY, .d__terminal 
			PRINT_TEXT  .cb__next_tty, [.d__terminal]
			call	system_restore
			jmp	system_exit

.cb__error_message:	db	"Error while VT_OPENQRY", 0
.cb__next_tty:		db	"next available tty: /dev/tty", STRING_INSERT_DEC32, 10, 0

section .data
			align 4
.d__terminal:		dd	0
%endif
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

			SYS_WRITE  STDOUT, .ab__cursor_off, 12
			SYS_FCNTL  STDIN, F_SETFL, O_NONBLOCK
			SYS_IOCTL  STDIN, TC_GETS, termios__original
			SYS_IOCTL  STDIN, TC_GETS, termios__data
			and	dword [termios__data + d__termios_local_flags], (NOT32 - ICANON - ECHO)
			SYS_IOCTL  STDIN, TC_SETS, termios__data
			SYS_IOCTL  STDIN, VT_GETMODE, vtmode__original
			SYS_IOCTL  STDIN, VT_GETMODE, vtmode__data
			mov	byte  [vtmode__data + b__vtmode_mode], VT_PROCESS
			SYS_IOCTL  STDIN, VT_SETMODE, vtmode__data
			mov	byte  [flag__tty_initialized], 1
			ret

.ab__cursor_off:	db	01bh, "[?25l"
.ab__no_blanking:	db	01bh, "[9;0]"



;****************************************************************************
;* tty_restore **************************************************************
;****************************************************************************
section .text
tty_restore:
			test	byte  [flag__tty_initialized], NOT8
			jz	.skip
			SYSP_IOCTL  STDIN, VT_SETMODE, vtmode__original
			SYSP_IOCTL  STDIN, TC_SETS, termios__original
			SYSP_WRITE  STDOUT, .ab__cursor_on, 12
			call	tty_clear_screen
.skip:			ret

.ab__cursor_on:		db	01bh, "[?25h"
.ab__blank_5:		db	01bh, "[9;5]"



;****************************************************************************
;* tty_clear_screen *********************************************************
;****************************************************************************
section .text
tty_clear_screen:
			SYSP_WRITE  STDOUT, .ab__clear_screen, 10
			ret

.ab__clear_screen:	db	01bh, "[1;1H", 01bh, "[2J"



;****************************************************************************
;* tty_get_character ********************************************************
;****************************************************************************
section .text
tty_get_character:
%ifdef TTY_ENABLE_AUTOMATIC_INPUT
			mov	dword ecx, [.d__text_delay]
			mov	dword ebx, [.p__text]
			mov	dword edx, [.d__input_table_offset]
			or	dword ecx, ecx
			jnz	.decrement_delay
			or	dword ebx, ebx
			jnz	.type_character
			mov	dword ecx, [.ad__input_table + 8 * edx]
			or	dword ecx, ecx
			jz	near  .get_character
			mov	dword ebx, [.ad__input_table + 8 * edx + 4]
			inc	dword edx
			mov	dword [.d__text_delay], ecx
			mov	dword [.p__text], ebx
			mov	dword [.d__input_table_offset], edx
			jmp	.end
.type_character:	xor	dword ecx, ecx
			mov	byte  cl, [ebx]
			inc	dword ebx
			or	byte  cl, cl
			jz	.text_end
			mov	dword eax, [.ab__translate + 4 * ecx]
			mov	dword [.p__text], ebx
			call	link_keyboard
			ret
.text_end:		xor	dword ebx, ebx
			mov	dword [.p__text], ebx
			jmp	.end
.decrement_delay:	dec	dword ecx
			mov	dword [.d__text_delay], ecx
.end:			xor	dword eax, eax
			call	link_keyboard
			ret

			align 4
%ifdef NOT_IN_USE
;turrican-side1.d64
.ad__input_table:	dd	150, .cb__load
			dd	800, .cb__space
			dd	150, .cb__space
			dd	0, 0

.cb__load:		db	'llooaadd""**"",,88,,11', 10, 10, 0
.cb__space:		db	'  ', 0
%endif

;radio_napalm.d64
.ad__input_table:	dd	150, .cb__load
			dd	2200, .cb__run
			dd	0, 0

.cb__load:		db	'llooaadd""**"",,88,,11', 10, 10, 0
.cb__run:		db	'rruunn', 10, 10, 0

%ifdef NOT_IN_USE
;reflection.d64
.ad__input_table:	dd	150, .cb__load
			dd	3900, .cb__run
			dd	0, 0

.cb__load:		db	'llooaadd""**"",,88,,11', 10, 10, 0
.cb__run:		db	'rruunn', 10, 10, 0
%endif

section .data
			align 4
.d__input_table_offset:	dd	0
.d__text_delay:		dd	0
.p__text:		dd	0

section .text
.get_character:
%endif
			SYS_READ  0, b__character, 1
			test	dword eax, eax
			jz	.no_character
			js	.no_character
			xor	dword ebx, ebx
			mov	byte  bl, [b__character]
			and	byte  bl, 07fh
			cmp	byte  bl, 05bh
			jne	.no_escape
			SYS_READ  0, b__character, 1
			test	dword eax, eax
			jz	.no_character
			js	.no_character
			xor	dword ebx, ebx
			mov	byte  bl, [b__character]
			or	byte  bl, 080h
.no_escape:		mov	dword eax, [.ab__translate + 4 * ebx]
			call	link_keyboard
			ret
.no_character:		xor	dword eax, eax
			call	link_keyboard
			ret
.exit:			call	tty_clear_screen
			jmp	system_exit

			align 4
.ab__translate:
			;-----------
			;normal keys
			;-----------

			;-----------
			;000h - 00fh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 025h, 0
			dd	0, 025h, 0, 0

			;-----------
			;010h - 01fh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;020h - 02fh
			;-----------

			dd	020h, 01b31h, 01b32h, 01b33h
			dd	01b34h, 01b35h, 01b36h, 01b37h
			dd	01b38h, 01b39h, 02ah, 02bh
			dd	03ch, 02dh, 03eh, 03fh

			;-----------
			;030h - 03fh
			;-----------

			dd	030h, 031h, 032h, 033h
			dd	034h, 035h, 036h, 037h
			dd	038h, 039h, 03ah, 03bh
			dd	01b3ch, 03dh, 01b3eh, 01b3fh

			;-----------
			;040h - 04fh
			;-----------

			dd	040h, 01b01h, 01b02h, 01b03h
			dd	01b04h, 01b05h, 01b06h, 01b07h
			dd	01b08h, 01b09h, 01b0ah, 01b0bh
			dd	01b0ch, 01b0dh, 01b0eh, 01b0fh

			;-----------
			;050h - 05fh
			;-----------

			dd	01b10h, 01b11h, 01b12h, 01b13h
			dd	01b14h, 01b15h, 01b16h, 01b17h
			dd	01b18h, 01b19h, 01b1ah, 01b3ah
			dd	01ch, 01b3bh, 01eh, 01fh

			;-----------
			;060h - 06fh
			;-----------

			dd	040h, 001h, 002h, 003h
			dd	004h, 005h, 006h, 007h
			dd	008h, 009h, 00ah, 00bh
			dd	00ch, 00dh, 00eh, 00fh

			;-----------
			;070h - 07fh
			;-----------

			dd	010h, 011h, 012h, 013h
			dd	014h, 015h, 016h, 017h
			dd	018h, 019h, 01ah, 0
			dd	0, 0, 0, 022h

			;-------------------------
			;keys with escape sequence
			;-------------------------

			;-----------
			;080h - 08fh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;090h - 09fh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0a0h - 0afh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0b0h - 0bfh
			;-----------

			dd	0, 021h, 0, 024h
			dd	01b21h, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0c0h - 0cfh
			;-----------

			dd	0, 01b27h, 027h, 028h
			dd	01b28h, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0d0h - 0dfh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0e0h - 0efh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0

			;-----------
			;0f0h - 0ffh
			;-----------

			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0
			dd	0, 0, 0, 0




;****************************************************************************
;* tty_press_any_key ********************************************************
;****************************************************************************
section .text
tty_press_any_key:
			push	dword eax
			push	dword ebx
			push	dword ecx
			push	dword edx
.wait:			SYS_NANOSLEEP  .timespec__data, 0
			SYS_READ  STDIN, b__character, 1
			test	dword eax, eax
			jz	.wait
			js	.wait
			pop	dword edx
			pop	dword ecx
			pop	dword ebx
			pop	dword eax
			ret

.timespec__data:	istruc timespec
			at d__timespec_sec, dd 0
			at d__timespec_nsec, dd 50000000
			iend



;****************************************************************************
;* data *********************************************************************
;****************************************************************************
section .data
flag__tty_initialized:	db	0


section .bss
			alignb 4
termios__data:		resb	termios_size

			alignb 4
termios__original:	resb	termios_size

			alignb 4
vtmode__data:		resb	vtmode_size

			alignb 4
vtmode__original:	resb	vtmode_size


			alignb 4
b__character:		resb	1
;********************************************* karsten.scheibler@bigfoot.de *

